﻿
Imports Microsoft.DirectX
Imports Microsoft.DirectX.Direct3D

Module App

    ' ===============================================================================================
    '   OBJECTS
    ' ===============================================================================================

    Friend Structure Object_3D
        Dim IsMovingObject As Boolean
        Dim ObjType As ObjTypes
        Dim SoundType As SoundTypes
        '
        Dim Position As Vector3
        Dim BulletIndex As Int32
        Dim IsActive As Boolean
        Dim Activation As ActivationState
        ' --------------------------------------------------------------- Draw
        Dim DrawType As DrawTypes
        Dim Texture As Texture
        Dim MeshX As ExtendedMeshStruct
        ' --------------------------------------------------------------- RigidBody
        Dim GlobalSpeed As Vector3
        Dim GlobalAcc As Vector3
        Dim LocalSpeed As Vector3
        Dim LocalAcc As Vector3
        Dim Shooted As Boolean
        ' --------------------------------------------------------------- Soft Body
        Dim SoftObjNodes() As Vector3
        Dim RopeFaces As Int32
        Dim RopeRadius As Single
        ' --------------------------------------------------------------- Car
        'Dim MaxSpeed As Single              ' linear max speed (m/sec)
        Dim MotorRpm As Single              ' motor (0 to 1)
        Dim Gear As Int32
        Dim GearRapport As Single
        Dim WheelMatrix() As Matrix
        Dim Steering As Single
        ' --------------------------------------------------------------- Shadow params
        Dim ShadowCaster As Boolean
        Dim ShadowReceiver As Boolean
        ' --------------------------------------------------------------- 
        Dim DrawAsShadowCaster As Boolean
        Dim DrawAsShadowReceiver As Boolean
        Dim DrawAsVisible As Boolean
    End Structure

    Friend Objects_3D(-1) As Object_3D

    Friend LastBtIndex As Int32

    Friend CameraIndex As Int32

    Friend App_CurrentScene As Int32 = 15

    Friend SlotDir As Int32 = -1
    Friend SlotUp As Int32 = -1
    Friend SlotLbutton As Int32 = -1
    Friend SlotRbutton As Int32 = -1




    ' ===============================================================================================
    '   APPLICATION
    ' ===============================================================================================

    Friend Sub Application_Init(Optional ByVal scene As Int32 = -1)
        If scene < 0 Then scene = App_CurrentScene
        DX_Open(Form1)
        Bullet_Open()
        App_StartScene()
        Sounds_Init()
        StartTimers()
    End Sub

    Friend Sub Application_Close()
        StopTimers()
        Sounds_Close()
        Bullet_Close()
        DX_Close()
        Applicaton_DestroyObjects()
    End Sub

    Friend Sub Applicaton_DestroyObjects()
        For i As Int32 = 0 To UBound(Objects_3D)
            Objects_3D(i).MeshX.msh = Nothing
            Objects_3D(i).MeshX.mat = Nothing
            Objects_3D(i).MeshX.mtx = Nothing
        Next
        ReDim Objects_3D(-1)
    End Sub

    Friend Sub App_StartScene()
        Sounds_StopAllSounds()
        Bullet_Close()
        InitObjects()
        Form2.DisplaySceneParams()
    End Sub



    Private Const BuildingSize As Single = 0.7 '1
    Private Const ShapeSize As Single = 0.95 '1.5
    Private SphereSize As Vector3
    Private CubeSize As Vector3
    Private CylinderSize As Vector3


    Private Sub InitObjects()

        DX_SetCamera()
        ControlType = ControlTypes.MoveCamera

        ' -------------------------------------------------------------------------- 
        App_Vehicle_StopMotors()
        VehicleActive = -1

        ' -------------------------------------------------------------------------- init
        LastBtIndex = -1
        LookAtObjectIndex = -1
        ReDim Objects_3D(-1)

        SphereSize = VEC3(ShapeSize, ShapeSize, ShapeSize)
        CubeSize = VEC3(ShapeSize, ShapeSize, ShapeSize)
        CylinderSize = VEC3(ShapeSize, ShapeSize * 1.4, ShapeSize)

        f_ShootSpeed = 70
        Bullet_SetDeactivationSpeed(0)

        ' -------------------------------------------------------------------------- create objects
        Dim SceneTitle As String = ""

        Select Case App_CurrentScene


            ' =================================================================================
            '   0 to 9 - LIGHT SCENES
            ' =================================================================================




            ' =================================================================================
            '   10 to 19 - RIGID BODY 1
            ' =================================================================================

            Case 10
 

            Case 11
                App_CreateWall(VEC3(-7, 0, 1), 2, CubeSize, ObjTypes.BOX)
                App_CreatePyramid(VEC3(-1, 0, 1), 3, CubeSize, ObjTypes.BOX)
                App_CreateWall(VEC3(3, 0, -3), 3, CubeSize, ObjTypes.BOX)
                App_CreatePyramid(VEC3(2, 0, 5), 2, CylinderSize, ObjTypes.CYLINDER)
                CreateStaticObject(Meshes.Scene, Tex.Panels_Wood_H, DrawTypes.MILL, 0, 0, 0, 0.7, 0.7, 0.7)
                App_CreateObjectChain(VEC3(-4.6, 13, 5.2), 7, ObjTypes.CYLINDER)
                App_CreateObjectChain(VEC3(-16, 6, 3.5), 4, ObjTypes.BOX)
                CreateTerrain_Flat()
                App_CreateCar(VEC3(0, 2, 0))

            Case 12
                App_CreateWall(VEC3(0, 12, 3), 2, CubeSize, ObjTypes.BOX)
                App_CreatePyramid(VEC3(-2, 0, 4), 2, CubeSize, ObjTypes.BOX)
                App_CreateWall(VEC3(6, 0, -2), 4, CubeSize, ObjTypes.BOX)
                App_CreatePyramid(VEC3(0, 5, 3), 2, CylinderSize, ObjTypes.CYLINDER)
                CreateStaticObject(Meshes.Scene, Tex.Panels_Wood_H, DrawTypes.MILL, 0, 0, 0, 0.7, 0.7, 0.7)
                App_CreateObjectChain(VEC3(-4.6, 13, 5.2), 7, ObjTypes.CYLINDER)
                App_CreateObjectChain(VEC3(-16, 6, 3.5), 4, ObjTypes.BOX)
                CreateTerrain_Flat()
                CreateTerrain_Hill()
                For i As Int32 = 1 To 5
                    App_CreateCar(VEC3(2 - i, 2, 2 + i * 5), , , , , 0.7, , False)
                Next
                App_CreateCar(VEC3(-2, 2, 6), , , , , 0.7)
                ControlType = ControlTypes.MoveVehicle

            Case 13
                DX_SetCamera(VEC3(-10, 4, -12), VEC3(0, 4, 0))
                App_CreateWall(VEC3(4, 0, 30), 2, CylinderSize, ObjTypes.CYLINDER)
                App_CreateWall(VEC3(-2, 0, 30), 10, CubeSize, ObjTypes.BOX)
                SphereSize.X *= 1.5F
                CreateTerrain_Flat()
                CreateTerrain_Hill()
                'CreateTerrain_Island()
                For i As Int32 = 1 To 5
                    App_CreateCar(VEC3(-20 + i * 5, 0.5, 20), , i * 1.5F, , , 1.0, , False)
                    App_CreateCar(VEC3(-20 + i * 5, 0.5, 40), , i * 1.5F, , , 1.0, , False)
                Next
                App_CreateCar(VEC3(-7, 1, 0))
                ControlType = ControlTypes.MoveVehicle

            Case 14
                DX_SetCamera(VEC3(-25, 4, 35), VEC3(0, -5, 0))
                App_CreateWall(VEC3(-20, 0, 15), 8, CylinderSize, ObjTypes.CYLINDER)
                App_CreateWall(VEC3(-10, 0, 25), 10, CubeSize, ObjTypes.BOX)
                App_CreateWall(VEC3(4, 0, -10), 9, CubeSize, ObjTypes.BOX)
                SphereSize.X *= 1.5F
                CreateTerrain_Flat()
                CreateTerrain_Island()
                For i As Int32 = 1 To 5
                    App_CreateCar(VEC3(-120 + i * 20, 0.5, -60), , 1, , , 1.0, , False)
                    App_CreateCar(VEC3(-110 + i * 20, 0.5, 0), , 1, , , 1.0, , False)
                Next
                App_CreateCar(VEC3(-60, 1, 0))
                ControlType = ControlTypes.AnimateVehicles

            Case 15
                App_CreateWall(VEC3(-5, 0, -3), 6, CubeSize, ObjTypes.BOX)
                App_CreatePyramid(VEC3(-2, 0, -6), 3, CubeSize, ObjTypes.BOX)
                App_CreateWall(VEC3(0, 0, 4), 13, CubeSize, ObjTypes.BOX)
                App_CreatePyramid(VEC3(4, 0, -4), 3, CylinderSize, ObjTypes.CYLINDER)
                CreateStaticObject(Meshes.Scene, Tex.Panels_Wood_H, DrawTypes.MILL, 0, 0, 0, 0.7, 0.7, 0.7)
                App_CreateObjectChain(VEC3(-2, 8, 8), 5, ObjTypes.BOX)
                App_CreateObjectChain(VEC3(-4.6, 13, 5.2), 7, ObjTypes.CYLINDER)
                CreateTerrain_Flat()
                CreateTerrain_Island()
                App_CreateCar(VEC3(0, 2, 0))
                ControlType = ControlTypes.MoveVehicle

            Case 16
                DX_SetCamera(VEC3(-2, 3, -15), VEC3(1, 3, 0))
                Dim activated As Boolean = True
                App_CreateWall(VEC3(-3, 0, -3), 6, CylinderSize, ObjTypes.CYLINDER, activated)
                App_CreateWall(VEC3(-10, 0, 22), 10, CubeSize, ObjTypes.BOX, activated)
                App_CreateWall(VEC3(10, 0, 4), 8, CubeSize, ObjTypes.BOX, activated)
                App_CreateWall(VEC3(4, 0, -7), 4, CylinderSize, ObjTypes.CYLINDER, activated)
                App_CreateWall(VEC3(10, 0, 22), 9, CylinderSize, ObjTypes.CYLINDER, activated)
                App_CreateWall(VEC3(0, 0, 30), 9, CylinderSize, ObjTypes.CYLINDER, activated)
                App_CreateWall(VEC3(20, 0, 40), 11, CylinderSize, ObjTypes.CYLINDER, activated)
                App_CreateWall(VEC3(30, 0, 20), 10, CylinderSize, ObjTypes.CYLINDER, activated)
                'CreateTerrain_Flat()
                CreateTerrain_Island()
                App_CreateCar(VEC3(1, 0.3, -3))
                'ControlType = ControlTypes.MoveVehicle

            Case 17
                DX_SetCamera(VEC3(-2, 3, -15), VEC3(1, 3, 0))
                Dim activated As Boolean = True
                ' ----------------------------------------------------------------------------- 500 obj
                App_CreateWall(VEC3(-3, 0, -3), 6, CylinderSize, ObjTypes.CYLINDER, activated)
                App_CreateWall(VEC3(-10, 0, 22), 13, CubeSize, ObjTypes.BOX, activated)
                App_CreateWall(VEC3(10, 0, 4), 8, CubeSize, ObjTypes.BOX, activated)
                App_CreateWall(VEC3(4, 0, -7), 4, CylinderSize, ObjTypes.CYLINDER, activated)
                App_CreateWall(VEC3(10, 0, 22), 12, CylinderSize, ObjTypes.CYLINDER, activated)
                App_CreateWall(VEC3(0, 0, 30), 12, CylinderSize, ObjTypes.CYLINDER, activated)
                App_CreateWall(VEC3(20, 0, 40), 13, CylinderSize, ObjTypes.CYLINDER, activated)
                App_CreateWall(VEC3(30, 0, 20), 13, CylinderSize, ObjTypes.CYLINDER, activated)
                ' ----------------------------------------------------------------------------- 1000 obj
                'CreateWall(VEC3(-3, 0, -3), 6, CylinderSize, ObjTypes.CYLINDER, activated)
                'CreateWall(VEC3(-10, 0, 20), 18, CubeSize, ObjTypes.BOX, activated)
                'CreateWall(VEC3(10, 0, 4), 8, CubeSize, ObjTypes.BOX, activated)
                'CreateWall(VEC3(4, 0, -7), 4, CylinderSize, ObjTypes.CYLINDER, activated)
                'CreateWall(VEC3(10, 0, 22), 19, CylinderSize, ObjTypes.CYLINDER, activated)
                'CreateWall(VEC3(0, 0, 30), 19, CylinderSize, ObjTypes.CYLINDER, activated)
                'CreateWall(VEC3(20, 0, 40), 19, CylinderSize, ObjTypes.CYLINDER, activated)
                'CreateWall(VEC3(30, 0, 20), 19, CylinderSize, ObjTypes.CYLINDER, activated)
                'CreateTerrain_Flat()
                CreateTerrain_Island()
                App_CreateCar(VEC3(1, 0.3, -3))
                'ControlType = ControlTypes.MoveVehicle

            Case 18
            Case 19


                ' =================================================================================
                '   20 to 29 - RIGID BODY 2
                ' =================================================================================
            Case 20

            Case 21
                App_CreateWall(VEC3(-4, 0, -5), 5, CubeSize, ObjTypes.BOX)
                App_CreatePyramid(VEC3(4, 0, -4), 4, CubeSize, ObjTypes.BOX)
                App_CreateWall(VEC3(-2, 0, -3), 5, CubeSize, ObjTypes.BOX)
                App_CreatePyramid(VEC3(2, 30, -3), 5, CylinderSize, ObjTypes.CYLINDER)
                CreateStaticObject(Meshes.Scene, Tex.Panels_Wood_H, DrawTypes.MILL, 0, 0, 0, 0.7, 0.7, 0.7)
                App_CreateObjectChain(VEC3(-4.6, 13, 5.2), 7, ObjTypes.CYLINDER)
                App_CreateObjectChain(VEC3(-16, 6, 3.5), 4, ObjTypes.BOX)
                CreateTerrain_Flat()
                CreateTerrain_Hill()
                App_CreateCar(VEC3(0, 2, 0))

            Case 22
                App_CreatePyramid(VEC3(-4, 20, 6), 7, CylinderSize, ObjTypes.CYLINDER)
                CreateStaticObject(Meshes.Scene, Tex.Panels_Wood_H, DrawTypes.MILL, 0, 0, 0, 0.7, 0.7, 0.7)
                App_CreateObjectChain(VEC3(-4.6, 13, 5.2), 7, ObjTypes.CYLINDER)
                App_CreateObjectChain(VEC3(-16, 6, 3.5), 4, ObjTypes.BOX)
                CreateTerrain_Flat()
                CreateTerrain_Island()
                App_CreateCar(VEC3(0, 2, 0))

            Case 23
                App_CreateWall(VEC3(3, 15, -0.7), 4, CubeSize, ObjTypes.BOX)
                App_CreatePyramid(VEC3(-3, 20, -0.7), 4, CylinderSize, ObjTypes.CYLINDER)
                SphereSize.X *= 1.5F
                CreateStaticObject(Meshes.Scene, Tex.Panels_Wood_H, DrawTypes.MILL, 0, -0.7, 0, 0.7, 0.7, 0.7)
                App_CreateObjectChain(VEC3(-4.2, 9.8, 0), 7, ObjTypes.BOX)
                App_CreateObjectChain(VEC3(-2.8, 9.8, 0), 5, ObjTypes.CYLINDER)
                App_CreateObjectChain(VEC3(-1.4, 9.8, 0), 7, ObjTypes.BOX)
                App_CreateObjectChain(VEC3(0, 9.8, 0), 5, ObjTypes.CYLINDER)
                App_CreateObjectChain(VEC3(1.4, 9.8, 0), 7, ObjTypes.BOX)
                App_CreateObjectChain(VEC3(2.8, 9.8, 0), 5, ObjTypes.CYLINDER)
                App_CreateObjectChain(VEC3(4.2, 9.8, 0), 7, ObjTypes.BOX)
                App_CreateObjectChain(VEC3(5.6, 9.8, 0), 5, ObjTypes.CYLINDER)
                CreateTerrain_Flat()
                CreateTerrain_Hill()
                App_CreateCar(VEC3(2, 2, -4))

            Case 24
            Case 25
            Case 26
            Case 27
            Case 28
            Case 29



                ' =================================================================================
                '   30 to 39 - SOFT BODIES 1
                ' =================================================================================
            Case 30

            Case 31
                ' ---------------------------------------------------------------------------------------- papero
                ' DX_SetCamera(VEC3(-10, 3, -8), VEC3(-10, 4.5, 0))
                DX_SetCamera(VEC3(-1, 3, -10), VEC3(0, 1, 0))
                'CreateStaticObject(Meshes.Scene, Tex.Panels_Wood_H, DrawTypes.MILL, 0, 0, 0, 0.7, 0.7, 0.7)
                'CreatePyramid(VEC3(-2, 40, 4), 3, VEC3(1, 0.5, 1), ObjTypes.SPHERE)
                'CreateStaticObject(LoadMesh("Ciotola2.x"), Tex.OldWood, DrawTypes.GENERIC, 0, 3, 0, 20, 20, 20)



                For i As Int32 = 0 To 0
                    App_CreateSoftObject(LoadMesh("Duck812.x", True), Tex.Duck_spheric, VEC3(-1 + i * 3, 16, 0), VEC3(2, 2, 2), 500, 0.2, 1)
                    Bullet_SoftObj_TuneParams(LastBtIndex, 0.9, 0.9, 0.0, 0.2, 0.99, 0.99)
                    Bullet_SoftObj_TuneVolumeParams(LastBtIndex, 0.0, 0.0, 0.0, 0.2, 0.0)
                    Bullet_SoftObj_TuneSolverParams(LastBtIndex, 1, 0, 2, 0, 4, 1.0)
                Next


                App_CreateSoftObject(LoadMesh("Frog400.x", True), Tex.Frog_spheric, VEC3(-4, 12, 0), VEC3(2, 2, 2), 500, 0.2, 1)
                Bullet_SoftObj_TuneParams(LastBtIndex, 0.9, 0.9, 0.0, 0.2, 0.99, 0.99)
                Bullet_SoftObj_TuneVolumeParams(LastBtIndex, 0.0, 0.0, 0.0, 0.2, 0.0)
                Bullet_SoftObj_TuneSolverParams(LastBtIndex, 1, 0, 2, 0, 4, 1.0)


                'CreateSoftObject(LoadMesh("Spring.x"), Tex.Cloth2, VEC3(-4, 6, 4), VEC3(10, 10, 10), 500, 0.3, 1)
                'Bullet_SoftObj_TuneParams(LastBtIndex, 0.9, 0.9, 0.0, 0.2, 0.99, 0.99)
                'Bullet_SoftObj_TuneVolumeParams(LastBtIndex, 0.0, 0.0, 0.0, 0.2, 0.0)
                'Bullet_SoftObj_TuneSolverParams(LastBtIndex, 1, 0, 2, 0, 4, 1.0)

                'CreateSoftObject(LoadMesh("Spring.x"), Tex.Cloth3, VEC3(4, 5, 4), VEC3(10, 10, 10), 500, 0.3, 1)
                'Bullet_SoftObj_TuneParams(LastBtIndex, 0.9, 0.9, 0.0, 0.2, 0.99, 0.99)
                'Bullet_SoftObj_TuneVolumeParams(LastBtIndex, 0.0, 0.0, 0.0, 0.2, 0.0)
                'Bullet_SoftObj_TuneSolverParams(LastBtIndex, 1, 0, 2, 0, 4, 1.0)

                App_CreateSoftObject(LoadMesh("Spring300.x"), Tex.Cloth2, VEC3(-4, 6, 4), VEC3(5, 5, 5), 500, 0.3, 1)
                Bullet_SoftObj_TuneParams(LastBtIndex, 0.7, 0.3, 0.01, 0.5, 0.7, 0.9)
                Bullet_SoftObj_TuneVolumeParams(LastBtIndex, 0.0, 0.0, 0.0, 0.4, 0.0)
                Bullet_SoftObj_TuneSolverParams(LastBtIndex, 1, 0, 2, 0, 4, 1.0)

                App_CreateSoftObject(LoadMesh("Spring300.x"), Tex.Cloth3, VEC3(4, 5, 4), VEC3(5, 5, 5), 500, 0.3, 1)
                Bullet_SoftObj_TuneParams(LastBtIndex, 0.7, 0.3, 0.01, 0.5, 0.7, 0.9)
                Bullet_SoftObj_TuneVolumeParams(LastBtIndex, 0.0, 0.0, 0.0, 0.4, 0.0)
                Bullet_SoftObj_TuneSolverParams(LastBtIndex, 1, 0, 2, 0, 4, 1.0)

                'CreateSoftObject(LoadMesh("Band82.x"), Tex.Rope1, VEC3(4, 9, 4), VEC3(10, 10, 10), 500, 0.3, 0)
                'Bullet_SoftObj_TuneParams(LastBtIndex, 0.7, 0.1, 0.01, 0.5, 0.7, 0.9)
                'Bullet_SoftObj_TuneVolumeParams(LastBtIndex, 0.0, 0.0, 0.0, 0.003, 0.0)
                'Bullet_SoftObj_TuneSolverParams(LastBtIndex, 1, 0, 2, 0, 4, 1.0)

                'CreateSoftObject(LoadMesh("Skirt.x"), Tex.Cloth1, VEC3(0, 16, 5), VEC3(1, 1, 1), 100, 0.15, 0)
                'Bullet_SoftObj_TuneParams(LastBtIndex, 0.9, 0.9, 0.0, 0.2, 0.99, 0.99)
                'Bullet_SoftObj_TuneVolumeParams(LastBtIndex, 0.0, 0.0, 0.0, 0.05, 0.0)
                'Bullet_SoftObj_TuneSolverParams(LastBtIndex, 1, 0, 2, 0, 4, 1.0)


                'CreateTerrain_Flat()
                CreateTerrain_Island()
                'CreateStaticObject(LoadMesh("TERR_3impact.x"), Tex.Island, DrawTypes.GENERIC, 0, 0, 0, 0.3, 0.3, 0.3)
                App_CreateCar(VEC3(3, 4, -1))


                'CreateStaticObject(LoadMesh("Looping.x"), Tex.OldWood,DrawTypes.GENERIC, , 16, 200, , , , 4.5)
                'CreateStaticObject(LoadMesh("Ciotola2.x"), Tex.OldWood,DrawTypes.GENERIC, , 1, , 10, 10, 10)
                'CreateStaticObject(LoadMesh("Vasetto1.x"), Tex.OldWood,DrawTypes.GENERIC, , 1, , 1, 1, 1)
                'CreateCar(VEC3(0.5, 0.5, 1))
                'CreateSimpleObject(VEC3(0, 0.2, 0), VEC3(4, 0.1, 4), ObjTypes.BOX)

                'CreateSoftObject(LoadMesh("Starfish.x"), Tex.CastleGold, VEC3(0, 3, 0))
                'CreateSoftObject(LoadMesh("Cat.x"), Tex.CastleGold, VEC3(0, 3, 0))
                'CreateSoftObject(LoadMesh("Duck4.x"), Tex.Duck4, VEC3(0, 3, 0))


                ' --------------------------------------------------------- filo stendere
            Case 32
                DX_SetCamera(VEC3(0, 2, 0), VEC3(0, 3, 5))

                Dim rope As Int32 = App_CreateRope(VEC3(-6, 5, 9), VEC3(5.7, 4, 6.5), , 6, 0.03, RopeSides.startSide Or RopeSides.endSide, Tex.Rope2, , , 100)
                Bullet_SoftObj_TuneParams(LastBtIndex, 0.7, 0.3, 0.01, 0.5, 0.99, 0.99, , 0.99)

                Dim ropepos7 As Vector3 = Bullet_SoftObj_GetNodePosition(rope, 7)
                Dim band As Int32 = App_CreateSoftObject(LoadMesh("Band82.x"), Tex.Cloth3, ropepos7 + VEC3(0, -0.2, 0.48), VEC3(2, 2, 2), 10, 0.1, 1)
                Bullet_SoftObj_TuneParams(LastBtIndex, 0.7, 0.3, 0.01, 0.5, 0.7, 0.9, , 0.9)
                Bullet_SoftObj_TuneVolumeParams(LastBtIndex, 0.0, 0.0, 0.0, 0.003, 0.0)
                Bullet_SoftObj_TuneSolverParams(LastBtIndex, 1, 0, 2, 0, 4, 1.0)
                Dim n As Int32 = Bullet_SoftObj_FindNearestNode(band, ropepos7)
                'Dim bandpos As Vector3 = Bullet_SoftObj_GetNodePosition(band, n)
                'Debug.Print(ropepos.ToString)
                'Debug.Print(bandpos.ToString)
                'Debug.Print(n.ToString)

                Dim ropepos16 As Vector3 = Bullet_SoftObj_GetNodePosition(rope, 16)
                Dim duck As Int32 = App_CreateSoftObject(LoadMesh("Duck812.x", True), Tex.Duck_spheric, ropepos16 + VEC3(0, -0.5, 0), VEC3(1, 1, 1), 10, 0.1, 1)
                Bullet_SoftObj_TuneParams(LastBtIndex, 0.7, 0.3, 0.01, 0.5, 0.99, 0.99, , 0.99)
                Bullet_SoftObj_TuneVolumeParams(LastBtIndex, 0.0, 0.0, 0.0, 0.4, 0.0)
                Bullet_SoftObj_TuneSolverParams(LastBtIndex, 1, 0, 2, 0, 4, 1.0)
                'Dim duckpos As Vector3 = Bullet_SoftObj_GetNodePosition(duck, 242)
                'Debug.Print((ropepos16 - duckpos).ToString)

                Dim ropepos23 As Vector3 = Bullet_SoftObj_GetNodePosition(rope, 23)
                Dim duck2 As Int32 = App_CreateSoftObject(LoadMesh("Duck812.x", True), Tex.Duck_spheric, ropepos23 + VEC3(0, -0.5, 0), VEC3(1, 1, 1), 10, 0.1, 1)
                Bullet_SoftObj_TuneParams(LastBtIndex, 0.7, 0.3, 0.01, 0.5, 0.7, 0.9, , 0.9)
                Bullet_SoftObj_TuneVolumeParams(LastBtIndex, 0.0, 0.0, 0.0, 0.4, 0.0)
                Bullet_SoftObj_TuneSolverParams(LastBtIndex, 1, 0, 2, 0, 4, 1.0)
                'Dim duckpos As Vector3 = Bullet_SoftObj_GetNodePosition(duck, 242)
                'Debug.Print((ropepos16 - duckpos).ToString)

                Bullet_SoftObj_LinkToSoft(rope, 7, band, n, True)
                Bullet_SoftObj_LinkToSoft(rope, 16, duck, 288, True)
                Bullet_SoftObj_LinkToSoft(rope, 23, duck2, 288, True)

                CreateStaticObject(Meshes.Scene, Tex.Panels_Wood_H, DrawTypes.MILL, 0, 0, 0, 0.7, 0.7, 0.7)
                CreateTerrain_Island()

            Case 33
            Case 34
            Case 35
                ' ---------------------------------------------------------------------------------------- sfera
                DX_SetCamera(VEC3(-1, 3, -5), VEC3(0, 1, 0))
                CreateStaticObject(Meshes.Scene, Tex.Panels_Wood_H, DrawTypes.MILL, 0, 0, 0, 0.7, 0.7, 0.7)
                App_CreatePyramid(VEC3(0, 4, 0), 2, VEC3(1, 0.5, 1), ObjTypes.SPHERE)
                CreateStaticObject(LoadMesh("Ciotola2.x"), Tex.OldWood, DrawTypes.MILL, 0, 0.5, 0, 4, 4, 4)
                App_CreateSoftObject(LoadMesh("Sphere480.x"), Tex.CastleGold, VEC3(0, 2, 0), VEC3(1, 1, 1), 10, 0.1)
                'CreateSoftObject(LoadMesh("Spring.x"), Tex.CastleGold, VEC3(0, 2, 0), VEC3(1, 1, 1), 10, 0.1, , , , , , 1)
                Bullet_SoftObj_TuneParams(LastBtIndex, 0.3, 0.1, 0.0001, 0.05, 0.9, 0.9)
                Bullet_SoftObj_TuneVolumeParams(LastBtIndex, 0.001, 1000, 100, 0.01, 0.5)
                Bullet_SoftObj_TuneSolverParams(LastBtIndex, 1, 0, 2, 0, 4, 1.0)
                'Bullet_SoftObj_UseClusters(btIndex, 1000)

                CreateTerrain_Island()

                'CreateCar(VEC3(0.5, 0.5, 1))

            Case 36
                ' ----------------------------------------------------------------------------------------
                DX_SetCamera(VEC3(-1, 3, -5), VEC3(0, 1, 0))
                CreateStaticObject(Meshes.Scene, Tex.Panels_Wood_H, DrawTypes.MILL, 0, 0, 0, 0.7, 0.7, 0.7)
                App_CreateSoftObject(LoadMesh("Sphere480.x"), Tex.CastleGold, VEC3(0, 2, 0), VEC3(2, 2, 2), 10, 0.05)
                Bullet_SoftObj_TuneParams(LastBtIndex, 0.3, 0.1, 0.0001, 0.05, 0.9, 0.9)
                Bullet_SoftObj_TuneVolumeParams(LastBtIndex, 0, 0, 0, 0.01, 0)
                Bullet_SoftObj_TuneSolverParams(LastBtIndex, 1, 0, 20, 0, 4, 1.0)
                CreateTerrain_Island()
                App_CreateCar(VEC3(0.5, 0.5, 1))

            Case 37
                ' ----------------------------------------------------------------------------------------
                DX_SetCamera(VEC3(-3, 3, -3), VEC3(-6, 2, 0))
                CreateStaticObject(Meshes.Scene, Tex.Panels_Wood_H, DrawTypes.MILL, 0, 0, 0, 0.7, 0.7, 0.7)
                For i As Int32 = 1 To 8
                    App_CreateSimpleObject(VEC3(-9, 5 + i, 5), VEC3(0.6, 0.6, 0.6), DrawTypes.BOX, ObjTypes.BOX)
                Next
                App_CreateSoftObject(LoadMesh("Sphere480.x"), Tex.Cloth2, VEC3(-9, 9, 4.5), VEC3(0.9, 0.9, 0.9), 10, 0.05)
                Bullet_SoftObj_TuneParams(LastBtIndex, 0.3, 0.1, 0.0001, 0.05, 0.9, 0.9)
                Bullet_SoftObj_TuneVolumeParams(LastBtIndex, 0, 0, 0, 0.02, 0)
                Bullet_SoftObj_TuneSolverParams(LastBtIndex, 1, 0, 10, 0, 4, 1.0)
                CreateTerrain_Island()

            Case 38
            Case 39
                '' ---------------------------------------------------------------------------------------- 
                'DX_SetCamera(VEC3(-1, 3, -5), VEC3(0, 1, 0))
                'CreateSoftObject(LoadMesh("Sphere92.x"), Tex.CastleGold, VEC3(0, 2, 0), VEC3(0.8, 0.8, 0.8), 1, 0.15, 0)
                ''CreateSoftObject(LoadMesh("Duck4_D.x", True), Tex.Duck4, VEC3(0, 2, 0), VEC3(1, 1, 1), 1, 0.15)
                ''CreateSoftObject(LoadMesh("Ciotola.x"), Tex.CastleGold, VEC3(0, 12, 0))
                ''CreateSoftObject(LoadMesh("Cube_98_192.x"), Tex.CastleGold, VEC3(0, 10, 0))
                'Bullet_SoftObj_TuneParams(LastBtIndex, 0.0, 0.0, 0.0, 0.0, 0.1, 0.1, 0.1)
                'Bullet_SoftObj_TuneVolumeParams(LastBtIndex, 0.0, 0.0, 0.0, 0.002, 0.1)
                'Bullet_SoftObj_TuneSolverParams(LastBtIndex, 1, 0, 2, 0, 4, 1.0)

                'CreateFluidContainer(Matrix.Scaling(1, 0.3, 1) * Matrix.Translation(0, 0.5, 0), LoadMesh("Vasetto1.x"), Tex.OldWood)
                'CreateTerrain_Island()




                ' =================================================================================
                '   40 to 49 - SOFT BODIES 2
                ' =================================================================================
            Case 40
            Case 41
            Case 42
            Case 43
            Case 44
            Case 45
            Case 46
            Case 47
            Case 48
            Case 49


                ' =================================================================================
                '   50 to 59 - CLOTH 1
                ' =================================================================================
            Case 51
                DX_SetCamera(VEC3(0, 3, -8), VEC3(0, 3, 0))
                App_CreateWall(VEC3(-7, 0, 1), 2, CubeSize, ObjTypes.BOX)
                App_CreatePyramid(VEC3(-1, 0, 1), 3, CubeSize, ObjTypes.BOX)
                App_CreateWall(VEC3(3, 0, -3), 3, CubeSize, ObjTypes.BOX)
                App_CreatePyramid(VEC3(2, 0, 5), 2, CylinderSize, ObjTypes.CYLINDER)
                App_CreateCloth(8, 8, 15, 15, -1, 7, 3, , , , Sides.corner10 Or Sides.corner11, Tex.Cloth1, , 0.3)
                Bullet_SoftObj_TuneSolverParams(LastBtIndex, , , 6)
                Bullet_SoftObj_TuneParams(LastBtIndex, 0.1, , 0.005)
                CreateStaticObject(Meshes.Scene, Tex.Panels_Wood_H, DrawTypes.MILL, 0, 0, 0, 0.7, 0.7, 0.7)
                App_CreateObjectChain(VEC3(-4.6, 13, 5.2), 7, ObjTypes.CYLINDER)
                App_CreateObjectChain(VEC3(-16, 6, 3.5), 4, ObjTypes.BOX)
                CreateTerrain_Flat()
                App_CreateCar(VEC3(0, 2, 0))

            Case 52
                App_CreateWall(VEC3(0, 12, 3), 2, CubeSize, ObjTypes.BOX)
                App_CreatePyramid(VEC3(-2, 0, 4), 2, CubeSize, ObjTypes.BOX)
                App_CreateWall(VEC3(6, 0, -2), 4, CubeSize, ObjTypes.BOX)
                App_CreatePyramid(VEC3(0, 5, 3), 2, CylinderSize, ObjTypes.CYLINDER)
                App_CreateCloth(8, 8, 15, 15, 0, 3, 0, 0, -0.5, 0, Sides.all_corners, Tex.Cloth1)
                Bullet_SoftObj_TuneSolverParams(LastBtIndex, , , 12)
                CreateStaticObject(Meshes.Scene, Tex.Panels_Wood_H, DrawTypes.MILL, 0, 0, 0, 0.7, 0.7, 0.7)
                App_CreateObjectChain(VEC3(-4.6, 13, 5.2), 7, ObjTypes.CYLINDER)
                App_CreateObjectChain(VEC3(-16, 6, 3.5), 4, ObjTypes.BOX)
                CreateTerrain_Flat()
                CreateTerrain_Hill()
                App_CreateCar(VEC3(3, 2, -6))

            Case 53
                App_CreatePyramid(VEC3(4, 3, -2), 3, CylinderSize, ObjTypes.CYLINDER)
                SphereSize.X *= 1.5F
                App_CreateCloth(12, 12, 19, 19, 0, 3, 0, 0, -0.2, 0, Sides.all_corners, Tex.Cloth1, 200, 0.3)
                Bullet_SoftObj_TuneSolverParams(LastBtIndex, , , 12)
                CreateTerrain_Flat()
                CreateTerrain_Hill()
                App_CreateCar(VEC3(-7, 1, 0))

            Case 54
                DX_SetCamera(10)
                App_CreateCloth(8, 8, 12, 12, 0, 40, 0, 1, 1, 0, 0, Tex.Cloth1, 10)
                Bullet_SoftObj_TuneParams(LastBtIndex, 0.2, , 0.001)
                Bullet_SoftObj_TuneAeroParams(LastBtIndex, VEC3(0, 0, 0))
                'Bullet_SoftObj_UseClusters(n, -1, 0.1, 0.1, 0.1,  0.1)
                App_CreateCloth(8, 8, 12, 12, 0, 42, 0, 1, 1, 0, 0, Tex.Cloth2, 10)
                Bullet_SoftObj_TuneParams(LastBtIndex, 0.2, , 0.001)
                Bullet_SoftObj_TuneAeroParams(LastBtIndex, VEC3(0, 0, 0))
                'Bullet_SoftObj_UseClusters(n, -1, 0.1, 0.1, 0.1, 0.1)
                LookAtObjectIndex = LastBtIndex
                'CreateTerrain_Flat()
                CreateTerrain_Island()

            Case 55
                DX_SetCamera(10)
                App_CreateCloth(8, 8, 12, 12, 0, 32, 0, 0, 1, 1, 0, Tex.Cloth2, 10)
                Bullet_SoftObj_TuneParams(LastBtIndex, 0.2, , 0.002)
                Bullet_SoftObj_TuneAeroParams(LastBtIndex, VEC3(0, 0, 0))
                App_CreateCloth(8, 8, 12, 12, 0, 34, 0, 0, 1, 1, 0, Tex.Cloth1, 10)
                Bullet_SoftObj_TuneParams(LastBtIndex, 0.2, , 0.002)
                Bullet_SoftObj_TuneAeroParams(LastBtIndex, VEC3(0, 0, 0))
                App_CreateCloth(8, 8, 12, 12, 0, 36, 0, 0, 1, 1, 0, Tex.Cloth2, 10)
                Bullet_SoftObj_TuneParams(LastBtIndex, 0.2, , 0.002)
                Bullet_SoftObj_TuneAeroParams(LastBtIndex, VEC3(0, 0, 0))
                App_CreateCloth(8, 8, 12, 12, 0, 38, 0, 0, 1, 1, 0, Tex.Cloth1, 10)
                Bullet_SoftObj_TuneParams(LastBtIndex, 0.2, , 0.002)
                Bullet_SoftObj_TuneAeroParams(LastBtIndex, VEC3(0, 0, 0))
                LookAtObjectIndex = LastBtIndex
                'CreateTerrain_Flat()
                CreateTerrain_Island()

            Case 56
                'CreateWall(VEC3(-4, 0, -4), 5, CubeSize, ObjTypes.BOX)
                'CreatePyramid(VEC3(4, 0, -4), 4, CubeSize, ObjTypes.BOX)
                'CreateWall(VEC3(-2, 0, -3), 3, CubeSize, ObjTypes.BOX)
                'CreatePyramid(VEC3(4, 3, -2), 3, CylinderSize, ObjTypes.CYLINDER)
                SphereSize.X *= 1.5F
                'CylinderSize.Z *= 0.5F
                App_CreateCloth(9, 9, 20, 20, 0, 3, 0, 0, 0, 0, Sides.all_corners, Tex.Cloth1)
                'CreateSoftObj(12, 12, 15, 15, 0, 0.1, 0, 0, 0, 0, 1 + 2 + 4 + 8, Tex.SoftObj1)

                App_CreateSimpleObject(VEC3(-16, 6, 0), VEC3(30, 0.1, 5), DrawTypes.BOX, ObjTypes.BOX)

                App_CreateSimpleObject(VEC3(0, 4, 0), VEC3(5, 0.1, 5), DrawTypes.BOX, ObjTypes.BOX)

                'CreateObjectChain(VEC3(-4.6, 13, 5.2), 7, ObjTypes.CYLINDER)
                'CreateObjectChain(VEC3(-16, 6, 3.5), 4, ObjTypes.BOX)
                CreateTerrain_Flat()
                CreateTerrain_Hill()
                App_CreateCar(VEC3(-7, 1, 0))

            Case 57
                DX_SetCamera(VEC3(2, 3, -10), VEC3(1, 3, 0))
                'CreateWall(VEC3(-5, 0, -2), 6, CubeSize, ObjTypes.BOX)
                'CreatePyramid(VEC3(0.5, 0, -1), 3, CubeSize, ObjTypes.BOX)
                'CreateWall(VEC3(0, 0, 2), 4, CubeSize, ObjTypes.BOX)
                'CreatePyramid(VEC3(20, 0, 16), 5, CylinderSize, ObjTypes.CYLINDER)
                App_CreateWall(VEC3(8, 0, 0), 3, CubeSize, ObjTypes.BOX)
                App_CreatePyramid(VEC3(7, 0, 3), 2, CylinderSize, ObjTypes.CYLINDER)
                App_CreateCloth(8, 8, 19, 19, 0, 3, 0, 0, 0, 0, Sides.all_corners, Tex.Cloth1, 800, 0.3)
                'Bullet_SoftObj_TuneSolverParams(LastBtIndex, , , 12)
                Bullet_SoftObj_TuneParams(LastBtIndex, , 0.9, 0.03, , 0.7)
                CreateTerrain_Flat()
                CreateTerrain_Island()
                App_CreateCar(VEC3(0, 4, 0))

            Case 58
                SceneTitle = "Without clusters the cloth is stable but can self-penetrate"
                App_CreateWall(VEC3(-5, 0, -2), 2, CubeSize, ObjTypes.BOX)
                App_CreateCloth(8, 8, 29, 29, -2, 5, -3, 1, 0, 0, 0, Tex.Cloth1)
                Bullet_SoftObj_TuneParams(LastBtIndex, 0.5, 0.99, 0.005)
                CreateTerrain_Island()
                App_CreateCar(VEC3(-2, 2, -3))

            Case 59
                SceneTitle = "Using clusters the cloth is not self-penetrating but can explode and never stops to move"
                App_CreateWall(VEC3(-5, 0, -2), 2, CubeSize, ObjTypes.BOX)
                App_CreateCloth(8, 8, 29, 29, -2, 5, -3, 1, 0, 0, 0, Tex.Cloth1)
                Bullet_SoftObj_TuneParams(LastBtIndex, 0.5, 0.99, 0.01)
                Bullet_SoftObj_UseClusters(LastBtIndex, 1000, 0.7, 0.1, 0.1, 0.1)
                CreateTerrain_Island()
                App_CreateCar(VEC3(-2, 2, -3))

            Case 50
                App_CreateWall(VEC3(-3, 0, 6), 3, CubeSize, ObjTypes.BOX)
                App_CreatePyramid(VEC3(3, 0, 6), 3, CylinderSize, ObjTypes.CYLINDER)


                App_CreateCloth(8, 8, 15, 15, -2, 8, -5, -0.5, 0, 0, Sides.corner10 Or Sides.corner11, Tex.Cloth1)
                'Bullet_SoftObj_TuneAeroParams(LastBtIndex, VEC3(4, -4, 4))

                App_CreateCloth(8, 8, 15, 15, 0, 9, -6, 0, 0, 0, Sides.corner11, Tex.Cloth1)
                Bullet_SoftObj_UseClusters(LastBtIndex, 1000, 0.1, 0.1, 0.1, 0.1)
                'Bullet_SoftObj_TuneAeroParams(LastBtIndex, VEC3(4, -4, 4))

                CreateTerrain_Flat()
                CreateTerrain_Hill()
                App_CreateCar(VEC3(0, 2, 0))



                ' =================================================================================
                '   60 to 69 - CLOTH 2
                ' =================================================================================
            Case 60
            Case 61
            Case 62
            Case 63
            Case 64
            Case 65
            Case 66
            Case 67
            Case 68
            Case 69




                ' =================================================================================
                '   70 to 79 - ROPES 1
                ' =================================================================================




            Case 71
                App_CreateWall(VEC3(0, 0, 7), 6, CubeSize, ObjTypes.BOX)
                'Dim s1 As Int32 = CreateCloth(8, 8, 15, 15, 0, 3, 0, 0, -0.5, 0, Sides.all_corners, Tex.Cloth1)

                Dim p0 As Vector3 = VEC3(-4.5, 1.4, 5)
                Dim r1 As Int32 = App_CreateSimpleObject(p0, VEC3(2, 2, 1.5), DrawTypes.BOX, ObjTypes.BOX, , 20, , , 0.001, , 0.001)

                Dim p1 As Vector3 = VEC3(-4.6, 13.65, 5.2)
                Dim p2 As Vector3 = p0 + VEC3(0, 0.9, 0)
                Dim s2 As Int32 = App_CreateRope(p1, p2, , , , RopeSides.startSide, Tex.Rope1, , , 20, 0.3, )
                Bullet_SoftObj_TuneParams(s2, 0.5, 0.5, , 0.0001)
                Bullet_SoftObj_LinkToRigid(s2, -1, r1, False)

                CreateStaticObject(Meshes.Scene, Tex.Panels_Wood_H, DrawTypes.MILL, 0, -0.7, 0, 0.7, 0.7, 0.7)
                CreateTerrain_Hill()

            Case 72
                'CreatePyramid(VEC3(0, 5, -5), 2, CylinderSize, ObjTypes.CYLINDER)
                'CreateWall(VEC3(-3, 12, -6), 2, CubeSize, ObjTypes.BOX)
                'CreatePyramid(VEC3(-4, 0, -4), 2, CubeSize, ObjTypes.BOX)
                App_CreateWall(VEC3(2, 0, 7), 5, CubeSize, ObjTypes.BOX)

                'Dim s1 As Int32 = CreateCloth(8, 8, 15, 15, 0, 3, 0, 0, -0.5, 0, Sides.all_corners, Tex.Cloth1)

                Dim p0 As Vector3 = VEC3(2, 1, 5)
                Dim r1 As Int32 = App_CreateSimpleObject(p0, VEC3(2, 2, 1.5), DrawTypes.BOX, ObjTypes.BOX, , 20, , , 0.001, 0.001)

                Dim p1 As Vector3 = VEC3(-4.6, 13.65, 5.2)
                Dim p2 As Vector3 = p0 + VEC3(-0.4, 0.9, 0)
                Dim s2 As Int32 = App_CreateRope(p1, p2, , , , RopeSides.startSide, Tex.Rope1, , , 10, 0.3, )
                Bullet_SoftObj_TuneParams(s2, 0.5, 0.5, , 0.0001)
                Bullet_SoftObj_LinkToRigid(s2, -1, r1, False)

                Dim p3 As Vector3 = VEC3(6.9, 10.8, 4.1)
                Dim p4 As Vector3 = p0 + VEC3(0.4, 0.9, 0)
                Dim s3 As Int32 = App_CreateRope(p3, p4, , , , RopeSides.startSide, Tex.Rope1, , , 10, 0.3, )
                Bullet_SoftObj_TuneParams(s3, 0.5, 0.5, , 0.0001)
                Bullet_SoftObj_LinkToRigid(s3, -1, r1, False)

                CreateStaticObject(Meshes.Scene, Tex.Panels_Wood_H, DrawTypes.MILL, 0, -0.7, 0, 0.7, 0.7, 0.7)
                'CreateObjectChain(VEC3(-4.6, 13, 5.2), 7, ObjTypes.CYLINDER)
                'CreateObjectChain(VEC3(-16, 6, 3.5), 4, ObjTypes.BOX)
                CreateTerrain_Hill()


                ' ======================================= CARRRUCOLAS
            Case 73
                DX_SetCamera(VEC3(-15, 5, -6), VEC3(-15, 4.5, 0))

                Dim p1 As Vector3 = VEC3(-18, 8, 5)
                Dim p2 As Vector3 = VEC3(-10, 8, 4)

                Dim r1, r2, rope As Int32

                For i As Int32 = 1 To 4
                    r1 = App_CreateSimpleObject(p1, VEC3(0.9, 0.9, 0.9), DrawTypes.BOX, ObjTypes.BOX, , 5, , , 0.001, 0.001)
                    r2 = App_CreateSimpleObject(p2, VEC3(0.9, 0.9, 0.9), DrawTypes.BARREL, ObjTypes.CYLINDER, , 5, , , 0.001, 0.001)

                    rope = App_CreateRope(p1 + VEC3(0, 0.5, 0), _
                                        p2 + VEC3(0, 0.3, 0), _
                                        , , , , Tex.Rope1, , , 50, _
                                        0.15, 1)
                    Bullet_SoftObj_TuneParams(rope, 0.9, 0.9, 0.02, 0.00001, 0.5, 0.99, 0.5, 0.9)
                    'Bullet_SoftObj_UseClusters(rope, 0, 0.99, 0.5, 0.99)

                    Bullet_SoftObj_LinkToRigid(rope, 0, r1, False)
                    Bullet_SoftObj_LinkToRigid(rope, -1, r2, False)

                    p1.Y += 2
                    p2.Y += 2
                Next

                CreateStaticObject(Meshes.Scene, Tex.Panels_Wood_H, DrawTypes.MILL, 0, -0.7, 0, 0.7, 0.7, 0.7)
                CreateTerrain_Hill()


                ' ======================================= CAR with WEDDING barattols
            Case 74
                DX_SetCamera(VEC3(0, 5, -12), VEC3(0, 4.5, 0))
                ControlType = ControlTypes.MoveVehicle

                Dim p1 As Vector3 = VEC3(0, 0, 10)
                Dim p2 As Vector3 = VEC3(-6, 0, 4)

                Dim index As Int32 = App_CreateCar(p1)

                Dim r1, rope As Int32
                For i As Int32 = 1 To 6
                    If i < 5 Then
                        r1 = App_CreateSimpleObject(p2, VEC3(0.5, 0.5, 0.5), DrawTypes.BARREL, ObjTypes.CYLINDER, , 5, , , 0.001, 0.001)
                    Else
                        r1 = App_CreateSimpleObject(p2, VEC3(0.5, 0.5, 0.5), DrawTypes.BOX, ObjTypes.BOX, , 5, , , 0.001, 0.001)
                    End If
                    Bullet_Object_SetDamping(r1, 0.3, 0.01)

                    rope = App_CreateRope(p1 + VEC3(-0.2, 0.25, -3), _
                                      p2 + VEC3(0, 0.3, 0), _
                                      40, _
                                      , _
                                      0.025, _
                                      , _
                                      Tex.Rope3, , 1, 2.0, 0.1, 2)

                    Bullet_SoftObj_TuneParams(rope, 0.99, 0.9, 0.01, 0.01, , 0.99)

                    Bullet_SoftObj_LinkToRigid(rope, 0, index, False)
                    Bullet_SoftObj_LinkToRigid(rope, -1, r1, False)

                    p2.X += 2
                Next

                ' -------------------------------------------------- 
                CreateTerrain_Hill()
                CreateStaticObject(Meshes.Scene, Tex.Panels_Wood_H, DrawTypes.MILL, 0, -0.7, 0, 0.7, 0.7, 0.7)

                'CreateTerrain_Flat()
                'CreateTerrain_FromExtMesh(CastleTerrain, ObjTypes.CASTLE_TERRAIN)




                ' ======================================= NEWTON CRADLE - DUAL ROPES - ROBY
            Case 75
                Dim DeltaX As Single = 1.5
                Dim SphereDiam As Single = 1.3
                Dim RopeLen As Single = 7
                Dim dx0 As Single = 6

                For i As Int32 = 0 To 4
                    Dim p1a As Vector3 = VEC3(-3 + i * DeltaX, 9.5, 6)
                    Dim p1b As Vector3 = VEC3(-3 + i * DeltaX, 9.5, 12.8)
                    Dim p2 As Vector3 = (p1a + p1b) * 0.5 + VEC3(0, -RopeLen, 0)
                    If i = 0 Then
                        p2 += VEC3(-dx0, RopeLen - CSng(Math.Sqrt(RopeLen * RopeLen - dx0 * dx0)), 0)
                    End If

                    Dim p2a As Vector3 = p2 - VEC3(0, 0, 0.1)
                    Dim p2b As Vector3 = p2 + VEC3(0, 0, 0.1)

                    Dim ra As Int32 = App_CreateRope(p1a, p2a, 5, 6, 0.02, RopeSides.startSide, Tex.DefaultGrey, , , 30, , 1, False, False)
                    Bullet_SoftObj_TuneParams(ra, 1.0, 1.0, 0.0, , , , , 1.0)

                    Dim rb As Int32 = App_CreateRope(p1b, p2b, 5, 6, 0.02, RopeSides.startSide, Tex.DefaultGrey, , , 30, , 1, False, False)
                    Bullet_SoftObj_TuneParams(rb, 1.0, 1.0, 0.0, , , , , 1.0)

                    App_CreateSimpleObject(p2 + VEC3(0, -0.5, 0), VEC3(SphereDiam, SphereDiam, SphereDiam), DrawTypes.BALL, ObjTypes.SPHERE, , 600, , , 0.0, 0.5, 1.0, 0.0)

                    Bullet_SoftObj_LinkToRigid(ra, -1, LastBtIndex, False)
                    Bullet_SoftObj_LinkToRigid(rb, -1, LastBtIndex, False)
                Next

                CreateStaticObject(LoadMesh("Cradle.x"), Tex.DefaultGrey, DrawTypes.CHROME, 0, 4.75, 9.4, 1, 0.85, 1)
                DX_SetCamera(VEC3(0, 4, -7), VEC3(0, 3, 6))
                CreateTerrain_Hill()
                CreateTerrain_Flat()
                App_CreateCar(VEC3(4, 0.3, 3))



                ' ======================================= NEWTON CRADLE - DUAL ROPES - LITTLE  ( Linear damping = 0.04 and Restitution > 1 ) 
            Case 76
                Dim DeltaX As Single = 0.3
                Dim SphereDiam As Single = 0.26
                Dim RopeLen As Single = 1.4
                Dim dx0 As Single = 1

                For i As Int32 = 0 To 4
                    Dim p1a As Vector3 = VEC3(-0.6F + i * DeltaX, 1.9, 1.2)
                    Dim p1b As Vector3 = VEC3(-0.6F + i * DeltaX, 1.9, 2.56)
                    Dim p2 As Vector3 = (p1a + p1b) * 0.5 + VEC3(0, -RopeLen, 0)
                    If i = 0 Then
                        p2 += VEC3(-dx0, RopeLen - CSng(Math.Sqrt(RopeLen * RopeLen - dx0 * dx0)), 0)
                    End If

                    Dim p2a As Vector3 = p2 - VEC3(0, 0, 0.02)
                    Dim p2b As Vector3 = p2 + VEC3(0, 0, 0.02)

                    Dim ra As Int32 = App_CreateRope(p1a, p2a, 3, 6, 0.004, RopeSides.startSide, Tex.DefaultGrey, , , 1, , 1, False, False)
                    Bullet_SoftObj_TuneParams(ra, 1.0, 1.0, 0.0, , , , , 1.0)

                    Dim rb As Int32 = App_CreateRope(p1b, p2b, 3, 6, 0.004, RopeSides.startSide, Tex.DefaultGrey, , , 1, , 1, False, False)
                    Bullet_SoftObj_TuneParams(rb, 1.0, 1.0, 0.0, , , , , 1.0)

                    App_CreateSimpleObject(p2 + VEC3(0, -0.05, 0), VEC3(SphereDiam, SphereDiam, SphereDiam), DrawTypes.CHROME, ObjTypes.SPHERE, SoundTypes.LittleSphere, 5, 0.04, , 0.05, 0.5, 1.1, 0.0)

                    Bullet_SoftObj_LinkToRigid(ra, -1, LastBtIndex, False)
                    Bullet_SoftObj_LinkToRigid(rb, -1, LastBtIndex, False)
                Next

                CreateStaticObject(LoadMesh("Cradle.x"), Tex.DefaultGrey, DrawTypes.CHROME, 0, 0.95, 1.88, 0.2, 0.17, 0.2)
                DX_SetCamera(VEC3(0, 2, -2), VEC3(0, 1, 1))
                CreateTerrain_Hill()
                CreateTerrain_Flat()
                App_CreateCar(VEC3(4, 0.3, 3))


                ' ======================================= NEWTON CRADLE - DUAL ROPES - VERY LITTLE  
            Case 77
                Dim DeltaX As Single = 0.15
                Dim SphereDiam As Single = 0.135
                Dim RopeLen As Single = 0.6
                Dim dx0 As Single = 0.45

                For i As Int32 = 0 To 4
                    Dim p1a As Vector3 = VEC3(-0.3F + i * DeltaX, 1.75, 0.6)
                    Dim p1b As Vector3 = VEC3(-0.3F + i * DeltaX, 1.75, 1.28)
                    Dim p2 As Vector3 = (p1a + p1b) * 0.5 + VEC3(0, -RopeLen, 0)
                    If i = 0 Then
                        p2 += VEC3(-dx0, RopeLen - CSng(Math.Sqrt(RopeLen * RopeLen - dx0 * dx0)), 0)
                    End If

                    Dim p2a As Vector3 = p2 - VEC3(0, 0, 0.01)
                    Dim p2b As Vector3 = p2 + VEC3(0, 0, 0.01)

                    Dim ra As Int32 = App_CreateRope(p1a, p2a, 1, 6, 0.002, RopeSides.startSide, Tex.DefaultGrey, , , 0.01, 0.001, 0, False, False)
                    Bullet_SoftObj_TuneParams(ra, 1.0, 1.0, 0.0, , , , , 1.0)
                    'Bullet_SoftObj_TuneSolverParams(ra, , , 12)

                    Dim rb As Int32 = App_CreateRope(p1b, p2b, 1, 6, 0.002, RopeSides.startSide, Tex.DefaultGrey, , , 0.01, 0.001, 0, False, False)
                    Bullet_SoftObj_TuneParams(rb, 1.0, 1.0, 0.0, , , , , 1.0)
                    'Bullet_SoftObj_TuneSolverParams(rb, , , 12)

                    App_CreateSimpleObject(p2 + VEC3(0, -0.05, 0), VEC3(SphereDiam, SphereDiam, SphereDiam), DrawTypes.CHROME, ObjTypes.SPHERE, SoundTypes.VeryLittleSphere, 1, 0.04, , 0.0, 0.5, 1.0, 0.0)

                    Bullet_SoftObj_LinkToRigid(ra, -1, LastBtIndex, False)
                    Bullet_SoftObj_LinkToRigid(rb, -1, LastBtIndex, False)
                Next

                App_CreateSimpleObject(VEC3(0, 0.4, 1), VEC3(1.6, 0.8, 0.9), DrawTypes.TERRAIN_CASTLE, ObjTypes.BOX, , 0)
                CreateStaticObject(LoadMesh("Cradle.x"), Tex.DefaultGrey, DrawTypes.CHROME, 0, 1.275, 0.94, 0.1, 0.085, 0.1)
                DX_SetCamera(VEC3(0, 2, -1), VEC3(0, 1, 1))
                CreateTerrain_Hill()
                CreateTerrain_Flat()
                App_CreateCar(VEC3(4, 0.3, 3))


            Case 78
                Dim DeltaX As Single = 0.15
                Dim SphereDiam As Single = 0.135
                Dim RopeLen As Single = 0.6
                Dim dx0 As Single = 0.45

                For i As Int32 = 0 To 4
                    Dim p1a As Vector3 = VEC3(-0.3F + i * DeltaX, 1.75, 0.6)
                    Dim p1b As Vector3 = VEC3(-0.3F + i * DeltaX, 1.75, 1.28)
                    Dim p2 As Vector3 = (p1a + p1b) * 0.5 + VEC3(0, -RopeLen, 0)
                    If i = 0 Then
                        p2 += VEC3(-dx0, RopeLen - CSng(Math.Sqrt(RopeLen * RopeLen - dx0 * dx0)), 0)
                    End If

                    Dim p2a As Vector3 = p2 - VEC3(0, 0, 0.01)
                    Dim p2b As Vector3 = p2 + VEC3(0, 0, 0.01)

                    Dim ra As Int32 = App_CreateRope(p1a, p2a, 3, 6, 0.002, RopeSides.startSide, Tex.DefaultGrey, , , 0.2, 0.001, 1, False, False)
                    Bullet_SoftObj_TuneParams(ra, 1.0, 1.0, 0.0, , , , , 1.0)
                    Bullet_SoftObj_TuneSolverParams(ra, , , 12)

                    Dim rb As Int32 = App_CreateRope(p1b, p2b, 3, 6, 0.002, RopeSides.startSide, Tex.DefaultGrey, , , 0.2, 0.001, 1, False, False)
                    Bullet_SoftObj_TuneParams(rb, 1.0, 1.0, 0.0, , , , , 1.0)
                    Bullet_SoftObj_TuneSolverParams(rb, , , 12)

                    App_CreateSimpleObject(p2 + VEC3(0, -0.05, 0), VEC3(SphereDiam, SphereDiam, SphereDiam), DrawTypes.CHROME, ObjTypes.SPHERE, SoundTypes.VeryLittleSphere, 1, 0.04, , 0.06, 0.5, 1.09, 0.0)

                    Bullet_SoftObj_LinkToRigid(ra, -1, LastBtIndex, False)
                    Bullet_SoftObj_LinkToRigid(rb, -1, LastBtIndex, False)
                Next

                App_CreateSimpleObject(VEC3(0, 0.4, 1), VEC3(1.6, 0.8, 0.9), DrawTypes.TERRAIN_CASTLE, ObjTypes.BOX, , 0)
                CreateStaticObject(LoadMesh("Cradle.x"), Tex.DefaultGrey, DrawTypes.CHROME, 0, 1.275, 0.94, 0.1, 0.085, 0.1)
                DX_SetCamera(VEC3(0, 2, -1), VEC3(0, 1, 1))
                CreateTerrain_Hill()
                CreateTerrain_Flat()
                App_CreateCar(VEC3(4, 0.3, 3))


            Case 79
            Case 70







                ' =================================================================================
                '   80 to 89 - ROPES 2
                ' =================================================================================
                ' ======================================= square and exagonal ropes
            Case 81
                DX_SetCamera(VEC3(0, 2, -3), VEC3(0, 3, 0))

                Dim p1 As Vector3 = VEC3(-4.6, 13.65, 5.2)
                Dim p2 As Vector3 = p1 + VEC3(-2, -12, -2)
                Dim s2 As Int32 = App_CreateRope(p1, p2, 31, 4, 0.2, RopeSides.startSide, Tex.Panels_Wood_H, , , 10, 0.2, )
                Bullet_SoftObj_TuneParams(s2, 0.99, 0.5, , 0.0001, , 1.0)

                Dim p3 As Vector3 = VEC3(0.3, 5, 0)
                Dim p4 As Vector3 = p3 + VEC3(1, -2.5, 1)
                Dim s3 As Int32 = App_CreateRope(p3, p4, 5, 6, 0.2, RopeSides.startSide, Tex.CastleTiles, , , 10, 0.2, )
                Bullet_SoftObj_TuneParams(s3, 0.99, 0.5, , 0.0001, , 1.0)

                CreateStaticObject(Meshes.Scene, Tex.Panels_Wood_H, DrawTypes.MILL, 0, -0.7, 0, 0.7, 0.7, 0.7)
                CreateTerrain_Hill()


            Case 82
            Case 83
            Case 84
            Case 85
            Case 86
            Case 87
            Case 88

                ' ======================================= parachute without link to soft
            Case 89
                Dim box As Int32 = App_CreateParachute(VEC3(0, 70, 5), False)
                LookAtObjectIndex = box

                App_CreateParachute(VEC3(-5, 70, -5), False)

                CreateStaticObject(Meshes.Scene, Tex.Panels_Wood_H, DrawTypes.MILL, 0, -0.7, 0, 0.7, 0.7, 0.7)
                CreateTerrain_Hill()


                ' ======================================= parachute with link to soft
            Case 80
                Dim box As Int32 = App_CreateParachute(VEC3(0, 70, 5), True)
                LookAtObjectIndex = box

                App_CreateParachute(VEC3(-5, 70, -5), True)

                'CreateParachute(VEC3(5, 70, -5), True)

                CreateStaticObject(Meshes.Scene, Tex.Panels_Wood_H, DrawTypes.MILL, 0, -0.7, 0, 0.7, 0.7, 0.7)
                CreateTerrain_Hill()





                ' =================================================================================
                '   MULTIOBJECT DESTRUCTIONS
                ' =================================================================================
            Case 91
                DX_SetCamera(VEC3(0, 7, -10), VEC3(0, 7, 0))

                App_CreateCastle(Matrix.Scaling(1, 1, 1) * Matrix.RotationY(0.4) * Matrix.Translation(10, 0, 25))
                'CreateMultimeshCastle()

                ExMeshes.CastleTerrain.mtx = Matrix.RotationY(0.4) * Matrix.Translation(10, 0, 25)

                App_CreateTerrain_FromExtMesh(ExMeshes.CastleTerrain, DrawTypes.TERRAIN_CASTLE, ObjTypes.GENERIC)
                'CreateTerrain_Island()
                'CreateTerrain_Flat()
                App_CreateCar(VEC3(4, 0.3, 4))

            Case 92
                DX_SetCamera(VEC3(0, 7, -10), VEC3(0, 7, 0))

                ExMeshes.CastleTerrain.mtx = Matrix.RotationY(0.4) * Matrix.Translation(10, 0, 25)
                App_CreateTerrain_FromExtMesh(ExMeshes.CastleTerrain, DrawTypes.TERRAIN_CASTLE, ObjTypes.GENERIC)

                Dim m As Matrix = Matrix.Scaling(1, 1, 1) * Matrix.RotationY(0.4) * Matrix.Translation(10, 0.05, 25)
                LoadMultimeshFile(MediaPath & "xFiles\" & "Castle\Castle_150.x", m, True, DrawTypes.CASTLE, ObjTypes.GENERIC, SoundTypes.Castle)
                Bullet_SetDeactivationSpeed(0.05)


            Case 93
                DX_SetCamera(VEC3(0, 7, -10), VEC3(0, 7, 0))

                ExMeshes.CastleTerrain.mtx = Matrix.RotationY(0.4) * Matrix.Translation(10, 0, 25)
                App_CreateTerrain_FromExtMesh(ExMeshes.CastleTerrain, DrawTypes.TERRAIN_CASTLE, ObjTypes.GENERIC)

                Dim m As Matrix = Matrix.Scaling(1, 1, 1) * Matrix.RotationY(0.4) * Matrix.Translation(10, 0.05, 25)
                LoadMultimeshFile(MediaPath & "xFiles\" & "Castle\Castle_250.x", m, True, DrawTypes.CASTLE, ObjTypes.GENERIC, SoundTypes.Castle)
                Bullet_SetDeactivationSpeed(0.05)

                App_CreateCar(VEC3(0, 2, 0))


            Case 94
                DX_SetCamera(VEC3(0, 7, -10), VEC3(0, 7, 0))

                ExMeshes.CastleTerrain.mtx = Matrix.RotationY(0.4) * Matrix.Translation(10, 0, 25)
                App_CreateTerrain_FromExtMesh(ExMeshes.CastleTerrain, DrawTypes.TERRAIN_CASTLE, ObjTypes.GENERIC)

                Dim m As Matrix = Matrix.Scaling(1, 1, 1) * Matrix.RotationY(0.4) * Matrix.Translation(10, 0.05, 25)
                LoadMultimeshFile(MediaPath & "xFiles\" & "Castle\Castle_500.x", m, True, DrawTypes.CASTLE, ObjTypes.GENERIC, SoundTypes.Castle)
                Bullet_SetDeactivationSpeed(0.05)


            Case 95
                DX_SetCamera(VEC3(0, 7, -10), VEC3(0, 7, 0))

                ExMeshes.CastleTerrain.mtx = Matrix.RotationY(0.4) * Matrix.Translation(10, 0, 25)
                App_CreateTerrain_FromExtMesh(ExMeshes.CastleTerrain, DrawTypes.TERRAIN_CASTLE, ObjTypes.GENERIC)

                Dim m As Matrix = Matrix.Scaling(1, 1, 1) * Matrix.RotationY(0.4) * Matrix.Translation(0, 10, 10)
                LoadMultimeshFile(ExtraMediaPath & "anim_ragno OK.x", m, False, DrawTypes.GENERIC, ObjTypes.GENERIC, SoundTypes.LittleSphere)


            Case 96
                DX_SetCamera(VEC3(0, 7, -10), VEC3(0, 7, 0))

                ExMeshes.CastleTerrain.mtx = Matrix.RotationY(0.4) * Matrix.Translation(10, 0, 25)
                App_CreateTerrain_FromExtMesh(ExMeshes.CastleTerrain, DrawTypes.TERRAIN_CASTLE, ObjTypes.GENERIC)

                Dim m As Matrix = Matrix.Scaling(3.5, 3.5, 3.5) * Matrix.RotationY(1.0) * Matrix.Translation(0, 6, -2)
                LoadMultimeshFile(ExtraMediaPath & "\moto2.x", m, False, DrawTypes.GENERIC, ObjTypes.GENERIC, SoundTypes.LittleSphere)


            Case 97
                DX_SetCamera(VEC3(-250, 50, 450), VEC3(-20, 7, 0))

                App_CreateEiffel(Matrix.Scaling(1, 1, 1) * Matrix.RotationY(0.4) * Matrix.Translation(10, 150, 150))

                CreateTerrain_Island()

                'For i As Int32 = 1 To 9
                '    CreateCar(VEC3(-200 + i * 20, 0.5, -60), , 1, , , 1.0, , False)
                '    CreateCar(VEC3(-100 + i * 20, 0.5, 20), , 1, , , 1.0, , False)
                'Next
                App_CreateCar(VEC3(4, 0.3, 4))


            Case 98
                DX_SetCamera(VEC3(-250, 50, 450), VEC3(-20, 7, 0))

                'CreateBuildings(VEC3(-280, 25, 340))
                App_CreateCastle(Matrix.Scaling(1, 1, 1) * Matrix.RotationY(3) * Matrix.Translation(-270, 26, 360))

                App_CreateEiffel(Matrix.Scaling(1, 1, 1) * Matrix.RotationY(0.4) * Matrix.Translation(10, 150, 150))
                'CreateEiffel(Matrix.Scaling(1, 1, 1) * Matrix.RotationY(0.4) * Matrix.Translation(100, 150, 120))
                'CreateEiffel(Matrix.Scaling(1, 1, 1) * Matrix.RotationY(0.4) * Matrix.Translation(300, 150, 400))

                CreateTerrain_Island()
                'CreateTerrain_Flat()
                'CreateCar(VEC3(4, 0.3, 4))


            Case 99
                'CreateBuildings(VEC3(0, 0, 0))
                App_CreateObjectChain(VEC3(14, 13, 5), 7, ObjTypes.CYLINDER)
                'CreateTerrain_Flat()
                CreateTerrain_Hill()

            Case 90
                'DX_SetCamera(VEC3(5, 2, -2), VEC3(0, 0, 0))
                'App_CreateFluid(200)
                ''CreateFluidContainer(Matrix.Scaling(30, 30, 30) * Matrix.Translation(0, 0, 0))
                'CreateTerrain_Hill()

        End Select


        ' ------------------------------ a simple "default" scene
        If LastBtIndex < 0 Then
            App_CreatePyramid(VEC3(4, 0, -2), 2, CylinderSize, ObjTypes.CYLINDER)
            SphereSize.X *= 1.5F
            CreateTerrain_Flat()
        End If

        ' ------------------------------ 99 is "special test scene" without camera
        If App_CurrentScene <> 99 Then
            App_CreateShootSpheres()
            App_CreateObjectCamera()
        Else
            App_CreateShootSpheres(0)
        End If

        ' ------------------------------ scene title
        If SceneTitle <> "" Then
            'Form1.Label99.Text = "Scene " & App_CurrentScene.ToString & " - " & SceneTitle ' & " - Press F1 for help."
            Form1.Label99.Text = SceneTitle
        Else
            Form1.Label99.Text = " Press ""H"" for help, ""F"" to toggle and ""ESC"" to close full screen."
        End If
  
        ' ------------------------------------------------------------------------------- mesh indexes for moving objects
        DX_SetMovingObjectsMeshIndex(DEBUG_HiQuality)

        ' ------------------------------------------------------------------------------- axis aligned bounding boxes
        DX_CalculateBoundingBoxForAllObjects()
    End Sub



    ' ===============================================================================================
    '   CREATION FUNCTIONS  ( Both Bullet and DirectX objects )
    ' ===============================================================================================



    Private Function App_Create3dObject(ByVal btIndex As Int32, _
                                        ByVal mtx As Matrix, _
                                        ByVal msh As Mesh, _
                                        ByVal tex As Texture, _
                                        ByVal drawType As DrawTypes, _
                                        ByVal objType As ObjTypes, _
                                        ByVal soundType As SoundTypes, _
                                        ByVal IsMovingObject As Boolean, _
                                        ByVal ShadowCaster As Boolean, _
                                        ByVal ShadowReceiver As Boolean) As Int32
        Dim dxObjIndex As Int32 = UBound(Objects_3D) + 1
        ReDim Preserve Objects_3D(dxObjIndex)
        With Objects_3D(dxObjIndex)
            .BulletIndex = btIndex
            .MeshX.mtx = mtx
            .MeshX.msh = msh
            .Texture = tex
            .DrawType = drawType
            .ObjType = objType
            .SoundType = soundType
            .IsMovingObject = IsMovingObject
            .ShadowCaster = ShadowCaster
            .ShadowReceiver = ShadowReceiver
        End With
        Return dxObjIndex
    End Function

    Private Function App_Create3dObject(ByVal btIndex As Int32, _
                                        ByVal exMesh As ExtendedMeshStruct, _
                                        ByVal drawType As DrawTypes, _
                                        ByVal objType As ObjTypes, _
                                        ByVal soundType As SoundTypes, _
                                        ByVal IsMovingObject As Boolean, _
                                        ByVal ShadowCaster As Boolean, _
                                        ByVal ShadowReceiver As Boolean) As Int32
        Dim dxObjIndex As Int32 = UBound(Objects_3D) + 1
        ReDim Preserve Objects_3D(dxObjIndex)
        With Objects_3D(dxObjIndex)
            .BulletIndex = btIndex
            .MeshX = exMesh
            .DrawType = drawType
            .ObjType = objType
            .SoundType = soundType
            .IsMovingObject = IsMovingObject
            .ShadowCaster = ShadowCaster
            .ShadowReceiver = ShadowReceiver
        End With
        Return dxObjIndex
    End Function

    Private Sub App_CreateObjectCamera()
        ' --------------------------------------------------------------------- BULLET OBJECT
        LastBtIndex = Bullet_CreateSimpleObject(Camera.Pos, _
                                                         VEC3(1, 1, 1), _
                                                         ObjTypes.BOX)
        Bullet_Object_SetFriction(LastBtIndex, 0.001)
        Bullet_Object_SetRestitution(LastBtIndex, 0.0)
        'Bullet_Object_SetKinematic(btIndex, True)
        Bullet_Object_SetActivationState(LastBtIndex, ActivationState.DISABLE_DEACTIVATION)
        ' --------------------------------------------------------------------- 3D OBJECT
        CameraIndex = App_Create3dObject(LastBtIndex, Matrix.Identity, Nothing, _
                                            Nothing, DrawTypes.CAMERA, ObjTypes.GENERIC, SoundTypes.NoSound, True, False, False)
    End Sub

    Private Function CreateTerrain_Flat() As Int32

        Dim m As Matrix = Matrix.Translation(VEC3(0, 0, 0)) * _
                            Matrix.Scaling(VEC3(1, 1, 1))
        ' --------------------------------------------------------------------- BULLET OBJECT
        LastBtIndex = Bullet_CreateFlatPlane(0)
        Bullet_Object_SetRestitution(LastBtIndex, 0.5)
        Bullet_Object_SetFriction(LastBtIndex, 1.0)
        ' --------------------------------------------------------------------- 3D OBJECT
        Return App_Create3dObject(LastBtIndex, m, Meshes.TerrainFlat, _
                                  Nothing, DrawTypes.TERRAIN_FLAT, ObjTypes.GENERIC, SoundTypes.NoSound, False, False, True)
    End Function




    Private Function CreateTerrain_Island() As Int32

        Dim m As Matrix = Matrix.Translation(VEC3(-105, 33.65, 270)) * Matrix.RotationY(-2)
        m = m * Matrix.Scaling(2, 1, 2)

        ' --------------------------------------------------------------------- BULLET OBJECT
        LastBtIndex = Bullet_CreateObjectFromMesh(Meshes.Terrain2, m, CollisionShapeTypes.TRIANGLE_MESH_SHAPE, 0)
        Bullet_Object_SetRestitution(LastBtIndex, 0.5)
        Bullet_Object_SetFriction(LastBtIndex, 1.0)
        ' --------------------------------------------------------------------- 3D OBJECT
        Return App_Create3dObject(LastBtIndex, m, Meshes.Terrain2, _
                                  Nothing, DrawTypes.TERRAIN_FLAT, ObjTypes.GENERIC, SoundTypes.NoSound, False, False, True)
    End Function


    Private Function CreateTerrain_Hill() As Int32
        Dim Scal As Matrix = Matrix.Scaling(0.5, 1, 0.5)
        Dim Rot As Matrix = Matrix.RotationY(-1)
        Dim Tras As Matrix = Matrix.Translation(0, -8, 230)
        ExMeshes.TerrainHill.mtx = Scal * Rot * Tras
        Return App_CreateTerrain_FromExtMesh(ExMeshes.TerrainHill, DrawTypes.TERRAIN_MESH, ObjTypes.GENERIC)
    End Function



    Private Function CreateStaticObject(ByVal _Mesh As Mesh, _
                                         ByVal Tex As Texture, _
                                         ByVal drawType As DrawTypes, _
                                         Optional ByVal PositionX As Single = 0, _
                                         Optional ByVal PositionY As Single = 0, _
                                         Optional ByVal PositionZ As Single = 0, _
                                         Optional ByVal SizeX As Single = 1, _
                                         Optional ByVal SizeY As Single = 1, _
                                         Optional ByVal SizeZ As Single = 1, _
                                         Optional ByVal RotationY As Single = 0) As Int32
        Dim m As Matrix
        ' --------------------------------------------------------------------- ROTATION / POSITION / SCALE
        m = Matrix.Scaling(SizeX, SizeY, SizeZ) * _
            Matrix.RotationY(RotationY) * _
            Matrix.Translation(PositionX, PositionY, PositionZ)
        ' --------------------------------------------------------------------- BULLET OBJECT
        LastBtIndex = Bullet_CreateObjectFromMesh(_Mesh, m, , 0)
        Bullet_Object_SetRestitution(LastBtIndex, 0.5)
        Bullet_Object_SetFriction(LastBtIndex, 1.0)
        ' --------------------------------------------------------------------- 3D OBJECT
        Return App_Create3dObject(LastBtIndex, m, _Mesh, _
                                   Tex, drawType, ObjTypes.GENERIC, SoundTypes.NoSound, False, True, True)
    End Function



    Friend Function App_CreateSimpleObject( _
                                        ByVal Position As Vector3, _
                                        ByVal ShapeSize As Vector3, _
                                        ByVal drawType As DrawTypes, _
                                        ByVal Shape As ObjTypes, _
                                        Optional ByVal SoundType As SoundTypes = SoundTypes.UseShape, _
                                        Optional ByVal Mass As Single = 1, _
                                        Optional ByVal Margin As Single = 0.05, _
                                        Optional ByVal Activated As Boolean = True, _
                                        Optional ByVal LinearDamping As Single = 0.05, _
                                        Optional ByVal AngularDamping As Single = 0.1, _
                                        Optional ByVal Restitution As Single = 0.5, _
                                        Optional ByVal Friction As Single = 0.8 _
                                        ) As Int32

        ' --------------------------------------------------------------------- BULLET OBJECT
        LastBtIndex = Bullet_CreateSimpleObject(Position, _
                                                 ShapeSize, _
                                                 Shape, _
                                                 Mass, _
                                                 Margin, _
                                                 Activated)

        Bullet_Object_SetDamping(LastBtIndex, LinearDamping, AngularDamping)
        Bullet_Object_SetRestitution(LastBtIndex, Restitution)
        Bullet_Object_SetFriction(LastBtIndex, Friction)

        If SoundType = SoundTypes.UseShape Then
            Select Case Shape
                Case ObjTypes.BOX : SoundType = SoundTypes.Box
                Case ObjTypes.SPHERE : SoundType = SoundTypes.Sphere
                Case ObjTypes.CYLINDER : SoundType = SoundTypes.Cylinder
            End Select
        End If

        ' --------------------------------------------------------------------- 3D OBJECT
        Return App_Create3dObject(LastBtIndex, _
                                  Bullet_Object_GetMatrixScaled(LastBtIndex), _
                                  Nothing, Nothing, drawType, Shape, SoundType, True, True, True)
    End Function





    Private Sub App_CreateObjectChain(ByVal position As Vector3, _
                                      ByVal nBox As Int32, _
                                      ByVal shape As ObjTypes, _
                                      Optional ByVal appended As Boolean = True)

        Dim Size As Vector3
        Dim drawType As DrawTypes

        If shape = ObjTypes.CYLINDER Then
            Size = CylinderSize
            drawType = DrawTypes.BARREL
        Else
            Size = CubeSize
            drawType = DrawTypes.BOX
        End If

        App_CreateSimpleObject(position, CubeSize, DrawTypes.BOX, ObjTypes.BOX, , 10)

        Bullet_Object_SetSleepingThresholds(LastBtIndex, 0.01, 0.01)

        If appended Then Bullet_Constraint_ObjectPoint(LastBtIndex, VEC3(0, 2, 0))

        Dim dist As Single = Size.Y * 0.85F

        For i As Int32 = 1 To nBox
            position += VEC3(-0.3, -(dist + 0.5F), 0)
            App_CreateSimpleObject(position, Size, drawType, shape, , 10)
            Bullet_Constraint_ObjectPointObject(LastBtIndex - 1, LastBtIndex, VEC3(0, -0.5, 0), VEC3(0, dist, 0))
        Next
    End Sub


    Private Sub App_CreateWall(ByVal Position As Vector3, _
                               ByVal StackSize As Int32, _
                               ByVal ShapeSize As Vector3, _
                               ByVal Shape As ObjTypes, _
                               Optional ByVal Activated As Boolean = True)

        Dim drawType As DrawTypes
        If Shape = ObjTypes.CYLINDER Then
            drawType = DrawTypes.BARREL
        Else
            drawType = DrawTypes.BOX
        End If

        Dim delta As Vector3
        delta.Z = 0
        delta.Y = ShapeSize.Y / 2

        While StackSize > 0
            delta.X = -StackSize * ShapeSize.X / 2

            For i As Int32 = 0 To StackSize - 1
                App_CreateSimpleObject(Position + delta, ShapeSize, drawType, Shape, , 10, , Activated)
                Bullet_Object_SetAdditionalDamping(LastBtIndex)
                delta.X += ShapeSize.X
            Next
            delta.Y += ShapeSize.Y
            StackSize -= 1
        End While
    End Sub

    Private Sub App_CreatePyramid(ByVal Position As Vector3, _
                                  ByVal StackSize As Int32, _
                                  ByVal ShapeSize As Vector3, _
                                  ByVal Shape As ObjTypes, _
                                  Optional ByVal Activated As Boolean = True)


        Dim drawType As DrawTypes
        If Shape = ObjTypes.CYLINDER Then
            drawType = DrawTypes.BARREL
        Else
            drawType = DrawTypes.BOX
        End If

        Dim delta As Vector3
        delta.Y = ShapeSize.Y / 2

        While StackSize > 0
            delta.Z = -StackSize * ShapeSize.Z / 2
            For j As Int32 = 0 To StackSize - 1
                delta.X = -StackSize * ShapeSize.X / 2
                For i As Int32 = 0 To StackSize - 1
                    App_CreateSimpleObject(Position + delta, ShapeSize, drawType, Shape, , 10, , Activated)
                    Bullet_Object_SetAdditionalDamping(LastBtIndex)
                    delta.X += ShapeSize.X
                Next
                delta.Z += ShapeSize.Z
            Next
            delta.Y += ShapeSize.Y
            StackSize -= 1
        End While
    End Sub

    Friend ShootSpheres_First As Int32
    Friend ShootSpheres_Last As Int32
    Private Sub App_CreateShootSpheres(Optional ByVal numSpheres As Int32 = 20, _
                                       Optional ByVal Activated As Boolean = False)
        ShootSpheres_First = UBound(Objects_3D) + 1
        ShootSpheres_Last = ShootSpheres_First + numSpheres - 1
        For i As Int32 = 0 To numSpheres - 1
            App_CreateSimpleObject(VEC3(-200, -200, i * 5), SphereSize, DrawTypes.BALL, ObjTypes.SPHERE, , 10, , Activated, 0.1, 0.5)
            'Bullet_Object_SetAdditionalDamping(LastBtIndex, 5, 0.7, 8, 0.7)
            'Bullet_Object_SetAdditionalDamping(LastBtIndex)

            With Objects_3D(ShootSpheres_First + i)
                .Shooted = True
            End With
        Next
    End Sub




    ' ===============================================================================================
    '   VEHICLE
    ' ===============================================================================================

    Private Function App_CreateCar(ByVal Position As Vector3, _
                                   Optional ByVal RotationX As Single = 0, _
                                   Optional ByVal RotationY As Single = 0, _
                                   Optional ByVal RotationZ As Single = 0, _
                                   Optional ByVal ScaleX As Single = 1, _
                                   Optional ByVal ScaleY As Single = 1, _
                                   Optional ByVal ScaleZ As Single = 1, _
                                   Optional ByVal MasterCar As Boolean = True) As Int32


        Dim m As Matrix = Matrix.RotationYawPitchRoll(RotationY, RotationX, RotationZ) * Matrix.Scaling(ScaleX, ScaleY, ScaleZ) * Matrix.Translation(Position)

        ' --------------------------------------------------------------------- BULLET OBJECT
        LastBtIndex = Bullet_CreateVehicle(ExMeshes.CarBody.msh, _
                                            VEC3(0, 0.2, 0), _
                                            VEC3(0, 0.0, 1), _
                                            m, _
                                            500)

        Bullet_Object_SetDamping(LastBtIndex, 0.05, 0.1)
        Bullet_Object_SetRestitution(LastBtIndex, 0.3)
        Bullet_Object_SetFriction(LastBtIndex, 0.8)

        Bullet_Vehicle_AddWheel(LastBtIndex, -0.7, 0.5, 2.1F, 0.75, 1.1, True)
        Bullet_Vehicle_AddWheel(LastBtIndex, 0.7, 0.5, 2.1F, 0.75, 1.1, True)
        Bullet_Vehicle_AddWheel(LastBtIndex, -0.7, 0.5, -0.7F, 0.75, 1.1, False)
        Bullet_Vehicle_AddWheel(LastBtIndex, 0.7, 0.5, -0.7F, 0.75, 1.1, False)

        Bullet_Vehicle_SetWheelTuning(LastBtIndex, 0, , , , 6)
        Bullet_Vehicle_SetWheelTuning(LastBtIndex, 1, , , , 6)
        Bullet_Vehicle_SetWheelTuning(LastBtIndex, 2, , , , 4.5)
        Bullet_Vehicle_SetWheelTuning(LastBtIndex, 3, , , , 4.5)


        'Dim m As Mesh = Bullet_Object_GetMesh(LastBtIndex)

        ' --------------------------------------------------------------------- 3D OBJECT
        Dim dxObjIndex As Int32 = App_Create3dObject(LastBtIndex, _
                                                     Matrix.Identity, _
                                                     ExMeshes.CarBody.msh, _
                                                     Nothing, _
                                                     DrawTypes.CAR, _
                                                     ObjTypes.CAR, _
                                                     SoundTypes.Car, _
                                                     True, _
                                                     True, _
                                                     True)

        Dim t As Texture = Tex.Car3_Body
        If Not MasterCar Then
            Select Case Int(5 * Rnd())
                Case 0 : t = Tex.Cloth1
                Case 1 : t = Tex.Rope2
                Case 2 : t = Tex.grassdark
                Case 3 : t = Tex.OldWood
                Case 4 : t = Tex.CastleGold
            End Select
        End If

        With Objects_3D(dxObjIndex)
            .Texture = t
            ReDim .WheelMatrix(3)
        End With

        If MasterCar Then
            VehicleActive = dxObjIndex
        End If

        Return dxObjIndex
    End Function



    Friend VehicleActive As Int32 = -1
    Friend VehicleCameraHeight As Single = 3

    Friend Sub App_Vehicle_StopMotors()
        If Objects_3D Is Nothing Then Exit Sub
        For i As Int32 = 0 To Objects_3D.Length - 1
            Objects_3D(i).MotorRpm = 0
        Next
        App_Vehicle_SetMotors(0)
    End Sub

    Friend Sub App_AnimateVehicles()
        App_Vehicle_SetMotors(1100)
        App_Vehicle_SetSteering(0)
        App_Vehicle_UpdateGearAndSpeed()
    End Sub

    Friend Sub App_MoveVehiclesWithKeys()
        If Not Form1.Focused Then Exit Sub

        If Key(Keys.Up) Then
            App_Vehicle_SetMotors(1000)
        ElseIf Key(Keys.Down) Then
            App_Vehicle_SetMotors(-1500)
        Else
            If SlotUp >= 0 Then
                Dim push As Single = Slots.ReadSlot_NoNan_0_1000(SlotUp)
                push = -10.0F * (push - 500)
                App_Vehicle_SetMotors(push)
            Else
                App_Vehicle_SetMotors(0)
            End If
        End If
        If Key(Keys.Left) Then
            App_Vehicle_SetSteering(-0.03)
        ElseIf Key(Keys.Right) Then
            App_Vehicle_SetSteering(0.03)
        Else
            If SlotDir >= 0 Then
                Dim dir As Single = Slots.ReadSlot_NoNan_0_1000(SlotDir)
                dir = 0.002F * (dir - 500)
                App_Vehicle_SetSteeringAbsolute(dir)
            Else
                App_Vehicle_SetSteering(0)
            End If
        End If
        If Key(Keys.PageUp) Then
            VehicleCameraHeight += 0.1F
        End If
        If Key(Keys.PageDown) Then
            VehicleCameraHeight -= 0.1F
        End If
        App_Vehicle_UpdateGearAndSpeed()
    End Sub

    Private Sub App_Vehicle_SetMotors(ByVal Motor As Single)

        Dim brake As Single = 0
        If Motor = 0 Then brake = 10

        For i As Int32 = 0 To Objects_3D.Length - 1
            With Objects_3D(i)
                If .ObjType = ObjTypes.CAR Then
                    ' ----------------------------------------- limit the speed
                    Dim speed As Single = .LocalSpeed.Z
                    If speed > 80 Then Motor /= (speed / 80)
                    If speed < -20 Then Motor /= (-speed / 20)
                    ' --------------------------------------------------------
                    Bullet_Vehicle_SetEngineForce(.BulletIndex, 0, Motor)
                    Bullet_Vehicle_SetEngineForce(.BulletIndex, 1, Motor)
                    Bullet_Vehicle_SetEngineForce(.BulletIndex, 2, Motor)
                    Bullet_Vehicle_SetEngineForce(.BulletIndex, 3, Motor)
                    Bullet_Vehicle_SetBrakingForce(.BulletIndex, 0, brake)
                    Bullet_Vehicle_SetBrakingForce(.BulletIndex, 1, brake)
                    Bullet_Vehicle_SetBrakingForce(.BulletIndex, 2, brake)
                    Bullet_Vehicle_SetBrakingForce(.BulletIndex, 3, brake)
                End If
            End With
        Next
    End Sub


    Private Sub App_Vehicle_SetSteering(ByVal DeltaSteering As Single)
        For i As Int32 = 0 To Objects_3D.Length - 1
            With Objects_3D(i)
                If .ObjType = ObjTypes.CAR Then

                    If i <> VehicleActive Or ControlType = ControlTypes.AnimateVehicles Then
                        If .Position.Length > 30 Then
                            'Dim t As PrecisionTimer = New PrecisionTimer


                            Dim m As Matrix = .MeshX.mtx
                            Dim ZeroDir As Vector3 = -Vector3.Normalize(.Position)
                            Dim right As Vector3 = Vector3.Normalize(Matrix_GetVectorRight(m))
                            Dim dir As Vector3 = Vector3.Normalize(Matrix_GetVectorDir(m))
                            ' -------------------------------------------------- Versus = 1 if going to center
                            Dim Versus As Single = Vector3.Dot(ZeroDir, dir)
                            ' -------------------------------------------------- DeltaDir = 1 if going too left
                            Dim DeltaDir As Single = Vector3.Dot(ZeroDir, right)

                            'If DeltaDir < 0.001 Then DeltaDir = Math.Sign(DeltaDir)
                            If Versus < 0 Then
                                If DeltaDir < 0.01 Then
                                    DeltaDir = Math.Sign(DeltaDir)
                                End If
                            End If



                            'Form1.Label99.Text = DeltaDir.ToString("0.000000000")
                            'Form1.Label99.Text = Versus.ToString("0.000000000")

                            .Steering = 0.3F * DeltaDir
                            Bullet_Vehicle_SetSteeringValue(.BulletIndex, 0, .Steering)
                            Bullet_Vehicle_SetSteeringValue(.BulletIndex, 1, .Steering)


                            'Form1.Label99.Text = t.GetTimeMicrosec.ToString
                            Continue For
                        End If
                    End If

                    If DeltaSteering <> 0 Then
                        Dim kspeed As Single = 0.5F + 0.05F * Math.Abs(.LocalSpeed.Z)
                        .Steering += DeltaSteering / kspeed
                        If .Steering > 0.6 Then .Steering = 0.6
                        If .Steering < -0.6 Then .Steering = -0.6
                    Else
                        If .Steering <> 0 Then
                            .Steering *= 0.6F
                        End If
                    End If
                    Bullet_Vehicle_SetSteeringValue(.BulletIndex, 0, .Steering)
                    Bullet_Vehicle_SetSteeringValue(.BulletIndex, 1, .Steering)
                    'Bullet_Vehicle_SetSteeringValue(.BulletIndex, 2, -.Steering)
                    'Bullet_Vehicle_SetSteeringValue(.BulletIndex, 3, -.Steering)
                End If
            End With
        Next
    End Sub

    Private Sub App_Vehicle_SetSteeringAbsolute(ByVal SteeringAbsolute As Single)
        For i As Int32 = 0 To Objects_3D.Length - 1
            With Objects_3D(i)
                If .ObjType = ObjTypes.CAR Then
                    .Steering = SteeringAbsolute
                    If .Steering > 0.8 Then .Steering = 0.8
                    If .Steering < -0.8 Then .Steering = -0.8
                    Bullet_Vehicle_SetSteeringValue(.BulletIndex, 0, .Steering)
                    Bullet_Vehicle_SetSteeringValue(.BulletIndex, 1, .Steering)
                    'Bullet_Vehicle_SetSteeringValue(.BulletIndex, 2, -.Steering)
                    'Bullet_Vehicle_SetSteeringValue(.BulletIndex, 3, -.Steering)
                End If
            End With
        Next
    End Sub


    Private Sub App_Vehicle_UpdateGearAndSpeed()
        For i As Int32 = 0 To Objects_3D.Length - 1
            With Objects_3D(i)
                If .ObjType = ObjTypes.CAR Then
                    Dim maxspeed As Single
                    If .LocalSpeed.Z < 0 Then
                        maxspeed = 60
                        .GearRapport = 4
                        .GearRapport = .GearRapport / maxspeed
                    Else
                        maxspeed = 90
                        Select Case .Gear
                            Case 0 : .GearRapport = 3.64
                            Case 1 : .GearRapport = 2.38
                            Case 2 : .GearRapport = 1.76
                            Case 3 : .GearRapport = 1.35
                            Case 4 : .GearRapport = 0.971
                            Case Else : .GearRapport = 0.756
                        End Select
                        .GearRapport = .GearRapport / maxspeed
                    End If

                    .MotorRpm = Math.Abs(.LocalSpeed.Z) * .GearRapport
                    If .MotorRpm > 1 Then .MotorRpm = 1

                    ' --------------------------------------------------- gear
                    If .MotorRpm > 0.95 Then
                        If .Gear < 5 Then .Gear = .Gear + 1
                    End If
                    If .MotorRpm < 0.6 Then
                        If .Gear > 0 Then .Gear = .Gear - 1
                    End If
                End If
            End With
        Next
    End Sub




    ' ===============================================================================================
    '   SoftObj
    ' ===============================================================================================


    Private Function App_CreateSoftObject( _
                                            ByVal _Mesh As Mesh, _
                                            ByVal _Texture As Texture, _
                                            ByVal Position As Vector3, _
                                            ByVal Scale As Vector3, _
                                            Optional ByVal Mass As Single = 1, _
                                            Optional ByVal Margin As Single = 0.1, _
                                            Optional ByVal BendingConstraintsDist As Int32 = 2 _
                                          ) As Int32

        Dim m As Matrix = Matrix.Scaling(Scale) * Matrix.RotationY(4) * Matrix.Translation(Position)

        'Dim msh As Mesh = _Mesh.Clone(MeshFlags.Managed, _Mesh.VertexFormat, DX_Device)
        Dim msh As Mesh = _Mesh

        ' --------------------------------------------------------------------- BULLET OBJECT
        LastBtIndex = Bullet_CreateSoftObjectFromMesh(msh, _
                                                        m, _
                                                        Mass, _
                                                        Margin, _
                                                        BendingConstraintsDist)

        ' --------------------------------------------------------------------- 3D OBJECT
        Dim dxObjIndex As Int32 = App_Create3dObject(LastBtIndex, _
                                                      Bullet_Object_GetMatrixScaled(LastBtIndex), _
                                                      msh, _Texture, DrawTypes.SOFT_OBJECT, ObjTypes.SOFT_OBJECT, SoundTypes.Cloth, False, True, True)

        ReDim Objects_3D(dxObjIndex).SoftObjNodes(Bullet_SoftObj_GetNodesCount(LastBtIndex) - 1)

        Return dxObjIndex
    End Function



    Private Function App_CreateCloth(ByVal Size_x As Single, _
                                        ByVal Size_z As Single, _
                                        Optional ByVal NodeCount_x As Int32 = 11, _
                                        Optional ByVal NodeCount_z As Int32 = 11, _
                                        Optional ByVal Pos_x As Single = 0, _
                                        Optional ByVal Pos_y As Single = 0, _
                                        Optional ByVal Pos_z As Single = 0, _
                                        Optional ByVal Yaw As Single = 0, _
                                        Optional ByVal Pitch As Single = 0, _
                                        Optional ByVal Roll As Single = 0, _
                                        Optional ByVal FixedSides As Sides = Sides.none, _
                                        Optional ByVal Tex As Texture = Nothing, _
                                        Optional ByVal TotalMass As Single = 100, _
                                        Optional ByVal Margin As Single = 0.2, _
                                        Optional ByVal BendingConstraintsDist As Integer = 2 _
                                      ) _
                                      As Int32

        Dim btIndex As Int32 = Bullet_SoftObj_CreateCloth( _
                                                            Size_x, _
                                                            Size_z, _
                                                            NodeCount_x, _
                                                            NodeCount_z, _
                                                            Pos_x, _
                                                            Pos_y, _
                                                            Pos_z, _
                                                            Yaw, _
                                                            Pitch, _
                                                            Roll, _
                                                            FixedSides, _
                                                            TotalMass, _
                                                            Margin, _
                                                            BendingConstraintsDist _
                                                          )
        LastBtIndex = btIndex

        ' --------------------------------------------------------------------- limit the shoot speed when using softbody
        f_ShootSpeed = Math.Min(50, f_ShootSpeed)


        ' --------------------------------------------------------------------- 3D OBJECT
        Dim dxObjIndex As Int32 = App_Create3dObject(btIndex, _
                                                     Matrix.Identity, _
                                                     App_CreateFlatMesh(NodeCount_x, NodeCount_z), _
                                                     Tex, _
                                                     DrawTypes.CLOTH, _
                                                     ObjTypes.CLOTH, _
                                                     SoundTypes.Cloth, _
                                                     False, _
                                                     True, _
                                                     True)

        ReDim Objects_3D(dxObjIndex).SoftObjNodes(Bullet_SoftObj_GetNodesCount(btIndex) - 1)

        Return dxObjIndex
    End Function



    Private Function App_CreateRope( _
                                    ByVal FromPoint As Vector3, _
                                    ByVal ToPoint As Vector3, _
                                    Optional ByVal SegmentCount As Int32 = 30, _
                                    Optional ByVal RopeFaces As Int32 = 4, _
                                    Optional ByVal RopeRadius As Single = 0.05, _
                                    Optional ByVal FixedSides As RopeSides = RopeSides.none, _
                                    Optional ByVal Tex As Texture = Nothing, _
                                    Optional ByVal Tex_ScaleV As Single = 30.0, _
                                    Optional ByVal Tex_ScaleU As Single = 0.5, _
                                    Optional ByVal TotalMass As Single = 1, _
                                    Optional ByVal Margin As Single = 0.2, _
                                    Optional ByVal BendingConstraintsDist As Integer = 2, _
                                    Optional ByVal ShadowCaster As Boolean = True, _
                                    Optional ByVal ShadowReceiver As Boolean = True _
                                    ) As Int32

        ' -------------------------------------------------------------------- BULLET OBJECT
        Dim btIndex As Int32 = Bullet_SoftObj_CreateRope( _
                                                            FromPoint, _
                                                            ToPoint, _
                                                            SegmentCount, _
                                                            FixedSides, _
                                                            TotalMass, _
                                                            Margin, _
                                                            BendingConstraintsDist _
                                                         )


        ' --------------------------------------------------------------------- 3D OBJECT
        Dim dxObjIndex As Int32 = App_Create3dObject(btIndex, _
                                                     Matrix.Identity, _
                                                     App_CreateRopeMesh(SegmentCount, RopeFaces, Tex_ScaleV, Tex_ScaleU), _
                                                     Tex, _
                                                     DrawTypes.ROPE, _
                                                     ObjTypes.ROPE, _
                                                     SoundTypes.NoSound, _
                                                     False, _
                                                     ShadowCaster, _
                                                     ShadowReceiver)

        ReDim Objects_3D(dxObjIndex).SoftObjNodes(Bullet_SoftObj_GetNodesCount(btIndex) - 1)
        Objects_3D(dxObjIndex).RopeFaces = RopeFaces
        Objects_3D(dxObjIndex).RopeRadius = RopeRadius

        Return dxObjIndex
    End Function




    Private Function App_CreateFlatMesh(ByVal nw As Int32, _
                                        ByVal nh As Int32, _
                                        Optional ByVal DiagonalLeft As Boolean = True, _
                                        Optional ByVal InvertDiagonalEachQuad As Boolean = True, _
                                        Optional ByVal InvertDiagonalEachLine As Boolean = True) As Mesh
        ' 
        If nw < 2 Then nw = 2
        If nh < 2 Then nh = 2
        Dim numFaces As Integer = 2 * (nw - 1) * (nh - 1)
        Dim numVerts As Integer = nw * nh
        ' ----------------------------------------------------------------------- mesh
        Dim m As Mesh = New Mesh(numFaces, _
                                 numVerts, _
                                 MeshFlags.Managed, _
                                 CustomVertex.PositionNormalTextured.Format, _
                                 DX_Device)
        ' ----------------------------------------------------------------------- vertices
        Dim x, y As Single
        Dim gs As GraphicsStream = m.VertexBuffer.Lock(0, 0, LockFlags.None)
        For h As Int32 = 0 To nh - 1
            y = CSng(h / (nh - 1))
            For w As Int32 = 0 To nw - 1
                x = CSng(w / (nw - 1))
                gs.Write(New CustomVertex.PositionNormalTextured(x, 0, y, 0, 1, 0, x, (1 - y)))
            Next
        Next
        m.VertexBuffer.Unlock()
        ' ----------------------------------------------------------------------- indices
        Dim indices(3 * numFaces - 1) As Int16
        Dim i As Int32 = 0
        Dim b1, b2 As Int32
        For h As Int32 = 0 To nh - 2
            b1 = h * nw
            b2 = (h + 1) * nw
            Dim DiagonalLeft_Actual As Boolean = DiagonalLeft
            For w As Int32 = 0 To nw - 2
                If DiagonalLeft_Actual Then
                    ' ---------------------------------- first face
                    indices(i) = CShort(b1 + w)
                    indices(i + 1) = CShort(b2 + w)
                    indices(i + 2) = CShort(b1 + w + 1)
                    ' ---------------------------------- second face
                    indices(i + 3) = CShort(b2 + w)
                    indices(i + 4) = CShort(b2 + w + 1)
                    indices(i + 5) = CShort(b1 + w + 1)
                    i += 6
                Else
                    ' ---------------------------------- first face
                    indices(i) = CShort(b1 + w)
                    indices(i + 2) = CShort(b2 + w + 1)
                    indices(i + 1) = CShort(b2 + w)
                    ' ---------------------------------- second face
                    indices(i + 3) = CShort(b1 + w)
                    indices(i + 5) = CShort(b1 + w + 1)
                    indices(i + 4) = CShort(b2 + w + 1)
                    i += 6
                End If
                If InvertDiagonalEachQuad Then DiagonalLeft_Actual = Not DiagonalLeft_Actual
            Next
            If InvertDiagonalEachLine Then DiagonalLeft = Not DiagonalLeft
        Next
        m.IndexBuffer.SetData(indices, 0, LockFlags.None)
        '
        Return m
    End Function



    Private Function App_CreateRopeMesh(ByVal nSegments As Int32, _
                                        ByVal nSides As Int32, _
                                        ByVal multiplierV As Single, _
                                        ByVal multiplierU As Single) As Mesh
        ' 
        If nSegments < 1 Then nSegments = 1
        If nSides < 1 Then nSides = 1

        Dim numFaces As Integer = 2 * nSides * nSegments
        Dim numVerts As Integer = (nSides + 1) * (nSegments + 1)
        ' ----------------------------------------------------------------------- mesh
        Dim m As Mesh = New Mesh(numFaces, _
                                 numVerts, _
                                 MeshFlags.Managed, _
                                 CustomVertex.PositionNormalTextured.Format, _
                                 DX_Device)
        ' ----------------------------------------------------------------------- vertices
        Dim x, y As Single
        Dim gs As GraphicsStream = m.VertexBuffer.Lock(0, 0, LockFlags.None)
        For h As Int32 = 0 To nSegments
            y = CSng(h / (nSegments - 1))
            For w As Int32 = 0 To nSides
                x = CSng(w / (nSides - 1))
                gs.Write(New CustomVertex.PositionNormalTextured(x, 0, y, 0, 1, 0, x * multiplierU, (1 - y) * multiplierV))
            Next
        Next
        m.VertexBuffer.Unlock()
        ' ----------------------------------------------------------------------- indices
        Dim indices(3 * numFaces - 1) As Int16
        Dim i As Int32 = 0
        Dim b1, b2 As Int32
        For h As Int32 = 0 To nSegments - 1
            b1 = h * (nSides + 1)
            b2 = (h + 1) * (nSides + 1)
            For w As Int32 = 0 To nSides - 1
                ' ---------------------------------- first face
                indices(i) = CShort(b1 + w)
                indices(i + 1) = CShort(b1 + w + 1)
                indices(i + 2) = CShort(b2 + w)
                ' ---------------------------------- second face
                indices(i + 3) = CShort(b2 + w)
                indices(i + 4) = CShort(b1 + w + 1)
                indices(i + 5) = CShort(b2 + w + 1)
                i += 6
            Next
        Next
        m.IndexBuffer.SetData(indices, 0, LockFlags.None)
        '
        Return m
    End Function



    ' ===========================================================================================================
    '   PARACHUTE
    ' ===========================================================================================================

    ' TODO - with "LinkTosoft" only the first parachute is connected - why ???   ( see also BulletWrapper.cpp )

    Private Function App_CreateParachute(ByVal position As Vector3, ByVal UseLinkTosoft As Boolean) As Int32

        Dim boxsize As Vector3 = VEC3(1, 1, 1)
        Dim clearance As Vector3 = VEC3(0, 0.1, 0)

        Dim nodesX As Int32 = 15
        Dim nodesZ As Int32 = 15
        Dim dx As Single = 4
        Dim dz As Single = 4

        ' ---------------------------------------------------------------- cloth central point
        Dim p0 As Vector3 = position
        ' ---------------------------------------------------------------- low point
        Dim p5 As Vector3 = p0 - VEC3(0, 1, 0)
        ' ---------------------------------------------------------------- cloth side points
        Dim p1 As Vector3 = p0 + VEC3(-dx, 0, -dz)
        Dim p2 As Vector3 = p0 + VEC3(dx, 0, -dz)
        Dim p3 As Vector3 = p0 + VEC3(-dx, 0, dz)
        Dim p4 As Vector3 = p0 + VEC3(dx, 0, dz)

        Dim cloth As Int32 = App_CreateCloth(dx * 2, dz * 2, nodesX, nodesZ, p0.X, p0.Y, p0.Z, 0, 0, 0, Sides.none, Tex.Cloth1, 10, 0.3)
        Bullet_SoftObj_TuneAeroParams(cloth, VEC3(0, 0, 0))

        Dim rope1 As Int32 = App_CreateRope(p5 + VEC3(-0.2, 0, -0.2), p1 - clearance, , , 0.05, , Tex.Rope1, , 0.3)
        Dim rope2 As Int32 = App_CreateRope(p5 + VEC3(0.2, 0, -0.2), p2 - clearance, , , 0.05, , Tex.Rope1, , 0.3)
        Dim rope3 As Int32 = App_CreateRope(p5 + VEC3(-0.2, 0, 0.2), p3 - clearance, , , 0.05, , Tex.Rope1, , 0.3)
        Dim rope4 As Int32 = App_CreateRope(p5 + VEC3(0.2, 0, 0.2), p4 - clearance, , , 0.05, , Tex.Rope1, , 0.3)

        Dim box As Int32 = App_CreateSimpleObject(p5 - VEC3(0, 0.6, 0), boxsize, DrawTypes.BOX, ObjTypes.BOX, , 10, , , 0.001, 0.001)


        If UseLinkTosoft Then
            Bullet_SoftObj_LinkToRigid(rope1, 0, box, False)
            Bullet_SoftObj_LinkToRigid(rope2, 0, box, False)
            Bullet_SoftObj_LinkToRigid(rope3, 0, box, False)
            Bullet_SoftObj_LinkToRigid(rope4, 0, box, False)

            Bullet_SoftObj_LinkToSoft(rope1, -1, cloth, 0, False)
            Bullet_SoftObj_LinkToSoft(rope2, -1, cloth, nodesX - 1, False)
            Bullet_SoftObj_LinkToSoft(rope3, -1, cloth, nodesX * nodesZ - nodesX, False)
            Bullet_SoftObj_LinkToSoft(rope4, -1, cloth, nodesX * nodesZ - 1, False)

        Else
            ' ----------------------------------------------------------------- dummy objects
            Dim ringsize As Vector3 = VEC3(0.1, 0.1, 0.1)
            Dim r1 As Int32 = App_CreateSimpleObject(p1, ringsize, DrawTypes.Invisible, ObjTypes.BOX, SoundTypes.NoSound, 1, , , 0.001, 0.001)
            Dim r2 As Int32 = App_CreateSimpleObject(p2, ringsize, DrawTypes.Invisible, ObjTypes.BOX, SoundTypes.NoSound, 1, , , 0.001, 0.001)
            Dim r3 As Int32 = App_CreateSimpleObject(p3, ringsize, DrawTypes.Invisible, ObjTypes.BOX, SoundTypes.NoSound, 1, , , 0.001, 0.001)
            Dim r4 As Int32 = App_CreateSimpleObject(p4, ringsize, DrawTypes.Invisible, ObjTypes.BOX, SoundTypes.NoSound, 1, , , 0.001, 0.001)
            ' -------------------------------------------------------------------------------

            Bullet_SoftObj_LinkToRigid(cloth, 0, r1, False)
            Bullet_SoftObj_LinkToRigid(cloth, nodesX - 1, r2, False)
            Bullet_SoftObj_LinkToRigid(cloth, nodesX * nodesZ - nodesX, r3, False)
            Bullet_SoftObj_LinkToRigid(cloth, nodesX * nodesZ - 1, r4, False)

            Bullet_SoftObj_LinkToRigid(rope1, -1, r1, False)
            Bullet_SoftObj_LinkToRigid(rope2, -1, r2, False)
            Bullet_SoftObj_LinkToRigid(rope3, -1, r3, False)
            Bullet_SoftObj_LinkToRigid(rope4, -1, r4, False)

            Bullet_SoftObj_LinkToRigid(rope1, 0, box, False)
            Bullet_SoftObj_LinkToRigid(rope2, 0, box, False)
            Bullet_SoftObj_LinkToRigid(rope3, 0, box, False)
            Bullet_SoftObj_LinkToRigid(rope4, 0, box, False)

        End If

        Return box
    End Function






    ' ===========================================================================================================
    '   CASTLE
    ' ===========================================================================================================

    Private Function App_CreateTerrain_FromExtMesh(ByVal exMesh As ExtendedMeshStruct, _
                                                   ByVal drawType As DrawTypes, _
                                                   ByVal objType As ObjTypes) As Int32

        ' --------------------------------------------------------------------- BULLET OBJECT
        LastBtIndex = Bullet_CreateObjectFromMesh(exMesh.msh, exMesh.mtx, , 0)
        Bullet_Object_SetRestitution(LastBtIndex, 0.5)
        Bullet_Object_SetFriction(LastBtIndex, 1.0)

        ' --------------------------------------------------------------------- 3D OBJECT
        Return App_Create3dObject(LastBtIndex, _
                                  exMesh, _
                                  drawType, _
                                  objType, _
                                  SoundTypes.NoSound, _
                                  False, _
                                  False, _
                                  True)
    End Function



    Private Function App_CreateCastle(ByVal m As Matrix) As Int32

        ' --------------------------------------------------------------------- 
        If ExMeshes.Castle.msh Is Nothing Then
            LoadExtendedMesh(ExMeshes.Castle, "Castle\Castle_SingleMesh.x", m)
        End If
        ' --------------------------------------------------------------------- BULLET OBJECT
        LastBtIndex = Bullet_CreateObjectFromMesh(ExMeshes.Castle.msh, m, , 0)
        Bullet_Object_SetRestitution(LastBtIndex, 0.5)
        Bullet_Object_SetFriction(LastBtIndex, 1.0)
        ' --------------------------------------------------------------------- 3D OBJECT
        Return App_Create3dObject(LastBtIndex, _
                                  ExMeshes.Castle, _
                                  DrawTypes.CASTLE, _
                                  ObjTypes.GENERIC, _
                                  SoundTypes.Castle, _
                                  False, _
                                  True, _
                                  True)
    End Function

    Private Function App_CreateEiffel(ByVal m As Matrix) As Int32

        ' --------------------------------------------------------------------- 
        'If Eiffel.msh Is Nothing Then
        LoadExtendedMesh(ExMeshes.Eiffel, "Eiffel.x", m)
        'End If
        ' --------------------------------------------------------------------- BULLET OBJECT
        LastBtIndex = Bullet_CreateObjectFromMesh(ExMeshes.Eiffel.msh, m, , 0)
        Bullet_Object_SetRestitution(LastBtIndex, 0.5)
        Bullet_Object_SetFriction(LastBtIndex, 1.0)
        ' --------------------------------------------------------------------- 3D OBJECT
        Return App_Create3dObject(LastBtIndex, _
                                  ExMeshes.Eiffel, _
                                  DrawTypes.EIFFEL, _
                                  ObjTypes.GENERIC, _
                                  SoundTypes.NoSound, _
                                  False, _
                                  True, _
                                  True)
    End Function




    Friend Sub App_CreateGenericObject(ByVal extMesh As ExtendedMeshStruct, _
                                       ByVal Mass As Single, _
                                       Optional ByVal drawType As DrawTypes = DrawTypes.GENERIC, _
                                       Optional ByVal objType As ObjTypes = ObjTypes.GENERIC, _
                                       Optional ByVal soundType As SoundTypes = SoundTypes.NoSound, _
                                       Optional ByVal activated As Boolean = True)

        ' --------------------------------------------------------------------- BULLET OBJECT
        'Bullet_CreateSimpleObjectFromMesh(extMesh.msh, extMesh.mtx)

        LastBtIndex = Bullet_CreateObjectFromMesh(extMesh.msh, _
                                                   extMesh.mtx, _
                                                   CollisionShapeTypes.BOX, 30, 0.01, activated)

        'Bullet_Object_SetAdditionalDamping(LastBtIndex, 1.6, 0.7, 0.04, 0.7)
        Bullet_Object_SetAdditionalDamping(LastBtIndex, 1.6, 0.7, 0.15, 0.7)

        Bullet_Object_SetSleepingThresholds(LastBtIndex, 0.5, 0.5)

        Bullet_Object_SetRestitution(LastBtIndex, 0.3)
        Bullet_Object_SetFriction(LastBtIndex, 0.9)
        Bullet_Object_SetDamping(LastBtIndex, 0.01, 0.2)

        ' --------------------------------------------------------------------- 3D OBJECT
        If Mass > 0 Then
            App_Create3dObject(LastBtIndex, extMesh, drawType, objType, soundType, True, True, True)
        Else
            App_Create3dObject(LastBtIndex, extMesh, drawType, objType, soundType, False, True, True)
        End If
    End Sub





    ' ===========================================================================================================
    '   FLUIDS
    ' ===========================================================================================================

    'Private Sub App_CreateFluid(ByVal numSpheres As Int32)
    '    Dim r As Single = 0.15
    '    Dim d As Single = 0.2
    '    Dim side As Int32 = CInt(numSpheres ^ (1 / 3))
    '    For x As Int32 = -side \ 2 To side \ 2
    '        For y As Int32 = -side \ 2 To side \ 2
    '            For z As Int32 = -side \ 2 To side \ 2
    '                App_CreateParticle(VEC3(x * d, 0.5F + y * d, z * d), 0.2)
    '            Next
    '        Next
    '    Next
    'End Sub

    'Private Sub App_CreateParticle(ByVal Position As Vector3, _
    '                                ByVal Radius As Single)
    '    ' --------------------------------------------------------------------- BULLET OBJECT
    '    LastBtIndex = Bullet_CreateSimpleObject(Position, _
    '                                                     VEC3(Radius, Radius, Radius), _
    '                                                     ObjTypes.SPHERE, _
    '                                                     0.1, _
    '                                                     0.02, _
    '                                                     True)
    '    Bullet_Object_SetDamping(LastBtIndex, 0, 0)
    '    Bullet_Object_SetRestitution(LastBtIndex, 0.6)
    '    Bullet_Object_SetFriction(LastBtIndex, 0)

    '    ' --------------------------------------------------------------------- 3D OBJECT
    '    App_Create3dObject(LastBtIndex, _
    '                       Bullet_Object_GetMatrixScaled(LastBtIndex), _
    '                       Meshes.Sphere, Nothing, DrawTypes.PARTICLE, ObjTypes.PARTICLE, SoundTypes.NoSound, True, True, True)
    'End Sub

    'Private Function App_CreateFluidContainer(ByVal m As Matrix, ByVal _Mesh As Mesh, ByVal _Tex As Texture) As Int32

    '    ' --------------------------------------------------------------------- BULLET OBJECT
    '    LastBtIndex = Bullet_CreateObjectFromMesh(_Mesh, m, CollisionShapeTypes.TRIANGLE_MESH_SHAPE, 0)
    '    Bullet_Object_SetRestitution(LastBtIndex, 0.5)
    '    Bullet_Object_SetFriction(LastBtIndex, 1.0)

    '    ' --------------------------------------------------------------------- 3D OBJECT
    '    Return App_Create3dObject(LastBtIndex, _
    '                              m, _
    '                              _Mesh, _
    '                              _Tex, _
    '                              ObjTypes.FLUID_CONTAINER, _
    '                              SoundTypes.NoSound, _
    '                              False, _
    '                              True, _
    '                              True)
    'End Function

End Module
