﻿

Imports Microsoft.DirectX
Imports Microsoft.DirectX.Direct3D

Module DirectX_Effects


    ' ===============================================================================================
    '   EFFECT - COMMON 
    ' ===============================================================================================
    Private mtx_LightViewProj As Matrix
    Private mtx_ViewProj As Matrix

    Private b_oldQuality As Boolean



    Private rt_ShadowTraspMap As RenderTarget
    Private rt_ShadowMap As RenderTarget

    Private f_ZetaRange As Single

    Private box_ShadowCasterPoints(-1) As BoundingBox
    Private box_ShadowReceiverPoints(-1) As BoundingBox

    Private mtx_TextureMatrix As Matrix
    Private mtx_texScaleBias As Matrix

    Private CompilationErrors As String

    Private VSM_Zstart As Single
    Private VSM_Zrange As Single

    ' --------------------------------------------------------------------------------------------------------
    '  the following four params are only for debugging and can be fixed to 0.01 / 0 / 0.5 / 0.85
    ' --------------------------------------------------------------------------------------------------------
    Friend SHAD_Bias As Single = 0.01
    Friend SHAD_BiasAutoRange As Single = 0
    Friend SHAD_Coefficent As Single = 0.5  ' constant tuned for near/far objects trade off and scene scale
    Friend SHAD_EpsilonW As Single = 0.85   ' how far to "infinity" warping can be pushed in critical case
    ' --------------------------------------------------------------------------------------------------------

    Friend SHAD_MaxDistance As Single = 50

    Friend SHAD_FilterGain As Single = 2
    Friend SHAD_FilterType As Int32 = 5
    Friend SHAD_ShadowDensity As Single = 0.45

    Friend SHAD_ShadowMapSize As Int32 = 1024
    Friend SHAD_FinalMapSize As Int32 = 512

    Friend SHAD_XpsmCorrectedRotation As Boolean = True
    Friend SHAD_ShadowAddSmooth As Boolean = True
    Friend SHAD_HiQualityFilters As Boolean = True
    Friend SHAD_HiQualityObjects As Boolean = True

    'Private v3_LightPos As Vector3
    Friend Shadow_LightDir As Vector3



    ' ===============================================================================================
    '   SHADOW MODES SELECTOR
    ' ===============================================================================================

    Friend Sub DX_ToggleShadows()
        DEBUG_DisableShadows = Not DEBUG_DisableShadows
        Effects_Init(False)
        Save_INI()
    End Sub


    Friend Sub DX_ToggleViewFinalMap()
        DEBUG_ViewFinalMap = Not DEBUG_ViewFinalMap
        Save_INI()
    End Sub



    ' ===============================================================================================
    '   EFFECT - COMMON CALLBACKS 
    ' ===============================================================================================

    Friend Sub Effects_Init(Optional ByVal ForceReinit As Boolean = True)
        ' -------------------------------------------- return if already initialized
        If SMAP IsNot Nothing And Not ForceReinit Then Return
        ' --------------------------------------------
        StopTimers()
        Try
            Effects_Close()
            If Not DEBUG_DisableShadows Then
                'Case ShadowModes.PHONG : Effect_Phong_Init()
                Effect_SMAP_Init()
            End If
        Catch
            If Trim(CompilationErrors) = "" Then
                MsgBox("Unable to create shadows with Pixel Shader 2.0", MsgBoxStyle.Information)
            Else
                'IO.File.WriteAllText(Application.StartupPath & "\SmapFx_Errors.htm", CompilationErrors)
                MsgBox(CompilationErrors, MsgBoxStyle.Information)
            End If
            DEBUG_DisableShadows = True
            Save_INI()
        End Try
        StartTimers()
    End Sub

    Friend Sub Effects_Close()
        'Effect_Phong_Close()
        Effect_SMAP_Close()
    End Sub

    Friend Sub Effects_UpdateMatrices()
        'Select Case ShadowMode
        '    Case ShadowModes.PHONG : Effect_Phong_UpdateMatrices()
        '    Case ShadowModes.SMAP_ORTHO : Effect_SMAP_UpdateMatrices()
        '    Case ShadowModes.SMAP_LISPSM : Effect_SMAP_UpdateMatrices()
        '    Case ShadowModes.SMAP_XPSM : Effect_SMAP_UpdateMatrices()
        'End Select
        If Not DEBUG_DisableShadows Then
            Effect_SMAP_UpdateMatrices()
        End If
    End Sub








    ' ===============================================================================================
    '   EFFECT - PER PIXEL PHONG
    ' ===============================================================================================

    'Private Phong As Effect

    'Private Sub Effect_Phong_Init()
    '    ' --------------------------------------------------------- for D3DX 1.0.2902
    '    Phong = Effect.FromFile(DX_Device, MediaPath & "Effects\perPixelPhong.fx", Nothing, ShaderFlags.None, Nothing)
    '    ' --------------------------------------------------------- for D3DX 1.0.2911
    '    'Phong = Effect.FromFile(DX_Device, MediaPath & "Effects\perPixelPhong.fx", Nothing, "", ShaderFlags.None, Nothing)

    '    Phong.SetValue(Phong.GetParameter(Nothing, "base_Tex"), Tex.DefaultGrey)
    'End Sub

    'Private Sub Effect_Phong_Close()
    '    If Phong Is Nothing Then Return
    '    Phong.Dispose()
    '    Phong = Nothing
    'End Sub

    'Friend Sub Effect_Phong_UpdateMatrices()
    '    With Phong
    '        .SetValue(.GetParameter(Nothing, "matViewProjection"), mtx_World * mtx_CameraView * mtx_CameraProj)
    '        .SetValue(.GetParameter(Nothing, "matView"), mtx_World * mtx_CameraView)
    '        .CommitChanges()
    '    End With
    'End Sub


    'Friend Sub Effect_Phong_StartPass()
    '    With Phong
    '        .Technique = .GetTechnique(0)

    '        .SetValue(.GetParameter(Nothing, "Ambient"), New Vector4(0.5, 0.5, 0.5, 1))
    '        .SetValue(.GetParameter(Nothing, "Diffuse"), New Vector4(0.5, 0.4, 0.3, 1))
    '        .SetValue(.GetParameter(Nothing, "Specular"), New Vector4(0.0, 0.0, 0.5, 1))
    '        .SetValue(.GetParameter(Nothing, "SpecularPower"), 3.0F)

    '        'Dim mat As Material = DX_Device.Material
    '        '.SetValue(.GetParameter(Nothing, "Ambient"), mat.AmbientColor)
    '        '.SetValue(.GetParameter(Nothing, "Diffuse"), mat.DiffuseColor)
    '        '.SetValue(.GetParameter(Nothing, "Specular"), mat.SpecularColor)
    '        '.SetValue(.GetParameter(Nothing, "SpecularPower"), mat.SpecularSharpness)

    '        '.SetValue(.GetParameter(Nothing, "base_Tex"), DX_Device.GetTexture(0))
    '        '.SetValue(.GetParameter(Nothing, "base_Tex"), Tex.Rope1)

    '        .SetValue(.GetParameter(Nothing, "LightPosition"), New Vector4(500, 500, -500, 0))
    '        '.SetValue(.GetParameter(Nothing, "EyePosition"), New Vector4(Camera.Dir.X, Camera.Dir.Y, Camera.Dir.Z, 1))
    '        .SetValue(.GetParameter(Nothing, "EyePosition"), New Vector4(Camera.Pos.X, Camera.Pos.Y, Camera.Pos.Z, 1))

    '        .Begin(FX.None)
    '        .BeginPass(0)
    '    End With
    'End Sub

    'Friend Sub Effect_Phong_EndPass()
    '    Phong.EndPass()
    '    Phong.End()
    'End Sub




    ' ===============================================================================================
    '   EFFECT - SMAP
    ' ===============================================================================================

    Private SMAP As Effect

    Private SMAP_WorldViewProj As EffectHandle
    Private SMAP_TexTransform As EffectHandle
    Private SMAP_ShadowMap As EffectHandle
    Private SMAP_Bias As EffectHandle
    Private SMAP_FilterGain As EffectHandle
    Private SMAP_VSM_Zstart As EffectHandle
    Private SMAP_VSM_Zrange As EffectHandle

    Private SMAP_RenderScene_Test As EffectHandle
    Private SMAP_RenderScene_Fast5 As EffectHandle
    Private SMAP_RenderScene_Fast4 As EffectHandle
    Private SMAP_RenderScene_Fast3 As EffectHandle
    Private SMAP_RenderScene_Fast2 As EffectHandle
    Private SMAP_RenderScene_Fast1 As EffectHandle
    Private SMAP_RenderScene_Hard As EffectHandle
    Private SMAP_RenderScene_Soft1 As EffectHandle
    Private SMAP_RenderScene_Soft2 As EffectHandle
    Private SMAP_RenderScene_Soft3 As EffectHandle
    Private SMAP_RenderScene_Soft4 As EffectHandle
    Private SMAP_RenderScene_Soft5 As EffectHandle
    Private SMAP_RenderScene_Soft6 As EffectHandle
    Private SMAP_RenderScene_Soft7 As EffectHandle
    Private SMAP_RenderScene_Soft8 As EffectHandle
    Private SMAP_RenderScene_Soft9 As EffectHandle
    Private SMAP_RenderShadow As EffectHandle
    Private SMAP_DrawShadowQuad As EffectHandle


    Private Sub Effect_SMAP_Init()

        ' ---------------------------------------------------------------------- bias matrix for the ShadowMap
        Dim OffsetX As Single = 0.5F + (0.5F / SHAD_ShadowMapSize)
        Dim OffsetY As Single = 0.5F + (0.5F / SHAD_ShadowMapSize)
        mtx_texScaleBias = Matrix_Build(0.5F, 0.0F, 0.0F, 0.0F, _
                                        0.0F, -0.5F, 0.0F, 0.0F, _
                                        0.0F, 0.0F, 1.0F, 0.0F, _
                                        OffsetX, OffsetY, 0.0F, 1.0F)

        ' ---------------------------------------------------------------------- compile the effect file
        'SMAP = Effect.FromFile(DX_Device, MediaPath & "Effects\SMAP.fx", _
        '                       Nothing, ShaderFlags.None, Nothing, CompilationErrors)

        Dim SMAP_String As String = IO.File.ReadAllText(MediaPath & "Effects\SMAP.fx")
        SMAP_String = Strings.Replace(SMAP_String, "#define SMAP_SIZE 1024", "#define SMAP_SIZE " & SHAD_ShadowMapSize.ToString)

        'Point Linear Anisotropic PyramidalQuad GaussianQuad
        If SHAD_HiQualityFilters Then
            SMAP_String = Strings.Replace(SMAP_String, "MinFilter = Point;", "MinFilter = Linear;")
            SMAP_String = Strings.Replace(SMAP_String, "MagFilter = Point;", "MagFilter = Linear;")
        End If

        ' --------------------------------------------------------- for D3DX 1.0.2902
        SMAP = Effect.FromString(DX_Device, SMAP_String, _
                                 Nothing, ShaderFlags.PartialPrecision, Nothing, CompilationErrors)

        ' --------------------------------------------------------- for D3DX 1.0.2911
        'SMAP = Effect.FromString(DX_Device, SMAP_String, _
        '                         Nothing, "", ShaderFlags.PartialPrecision, Nothing, CompilationErrors)


        IO.File.WriteAllText(Application.StartupPath & "\SmapFx.htm", SMAP.Disassemble(True))

        ' ---------------------------------------------------------------------- create the shadow map render target
        rt_ShadowMap.Init(SHAD_ShadowMapSize, SHAD_ShadowMapSize, Format.R32F)

        ' ---------------------------------------------------------------------- create the transparent render target
        rt_ShadowTraspMap.Init(SHAD_FinalMapSize, SHAD_FinalMapSize, Format.A8R8G8B8)


        With SMAP
            SMAP_WorldViewProj = .GetParameter(Nothing, "WorldViewProj")
            SMAP_TexTransform = .GetParameter(Nothing, "TexTransform")
            SMAP_ShadowMap = .GetParameter(Nothing, "ShadowMap")
            SMAP_Bias = .GetParameter(Nothing, "Bias")
            SMAP_FilterGain = .GetParameter(Nothing, "FilterGain")
            SMAP_VSM_Zstart = .GetParameter(Nothing, "VSM_Zstart")
            SMAP_VSM_Zrange = .GetParameter(Nothing, "VSM_Zrange")

            SMAP_RenderScene_Test = .GetTechnique("RenderScene_Test")
            SMAP_RenderScene_Fast5 = .GetTechnique("RenderScene_Fast5")
            SMAP_RenderScene_Fast4 = .GetTechnique("RenderScene_Fast4")
            SMAP_RenderScene_Fast3 = .GetTechnique("RenderScene_Fast3")
            SMAP_RenderScene_Fast2 = .GetTechnique("RenderScene_Fast2")
            SMAP_RenderScene_Fast1 = .GetTechnique("RenderScene_Fast1")
            SMAP_RenderScene_Hard = .GetTechnique("RenderScene_Hard")
            SMAP_RenderScene_Soft1 = .GetTechnique("RenderScene_Soft1")
            SMAP_RenderScene_Soft2 = .GetTechnique("RenderScene_Soft2")
            SMAP_RenderScene_Soft3 = .GetTechnique("RenderScene_Soft3")
            SMAP_RenderScene_Soft4 = .GetTechnique("RenderScene_Soft4")
            SMAP_RenderScene_Soft5 = .GetTechnique("RenderScene_Soft5")
            SMAP_RenderScene_Soft6 = .GetTechnique("RenderScene_Soft6")
            SMAP_RenderScene_Soft7 = .GetTechnique("RenderScene_Soft7")
            SMAP_RenderScene_Soft8 = .GetTechnique("RenderScene_Soft8")
            SMAP_RenderScene_Soft9 = .GetTechnique("RenderScene_Soft9")
            SMAP_RenderShadow = .GetTechnique("RenderShadow")
            SMAP_DrawShadowQuad = .GetTechnique("DrawShadowQuad")

            .SetValue(SMAP_ShadowMap, rt_ShadowMap.texture)
        End With
    End Sub

    Private Sub Effect_SMAP_Close()
        If SMAP Is Nothing Then Return
        rt_ShadowMap.Destroy()
        rt_ShadowTraspMap.Destroy()
        SMAP.Dispose()
        SMAP = Nothing
    End Sub



    Friend Sub Effect_SMAP_UpdateMatrices()
        With SMAP
            .SetValue(SMAP_WorldViewProj, mtx_World * mtx_ViewProj)
            .SetValue(SMAP_TexTransform, mtx_World * mtx_LightViewProj * mtx_texScaleBias)
            .CommitChanges()
        End With
    End Sub




    Friend Sub Effect_SMAP_PrepareForRenderShadow()
        With SMAP

            ' -------------- LightDir points from scene to the light ------------

            ' ---------------------------------------------------------------------------------- 1 ( prospectic shadows )
            'v3_LightPos = VEC3(30, 30, -30)
            'v3_LightDir = Vector3.Normalize(VEC3(1, 1, -1))
            'Dim mtx_LightView As Matrix = Matrix.LookAtLH(v3_LightPos, v3_LightPos - v3_LightDir, VEC3(0, 1, 0))
            'Dim mtx_LightProj As Matrix = Matrix.PerspectiveFovLH(1.0, 1, 0.8F, 200.0F)
            'mtx_LightViewProj = mtx_LightView * mtx_LightProj
            'ComputeVirtualCameraParameters()

            ' ---------------------------------------------------------------------------------- 2
            'Shadow_LightDir = Vector3.Normalize(VEC3(1, 1, -1))
            'v3_LightDir = Vector3.TransformNormal(v3_LightDir, Matrix.RotationY(Environment.TickCount / 2000.0F))

            'Select Case ShadowMode
            '    Case ShadowModes.SMAP_XPSM : BuildXPSMProjectionMatrix()
            'End Select
            BuildXPSMProjectionMatrix()


            mtx_ViewProj = mtx_LightViewProj


            b_oldQuality = DEBUG_HiQuality
            If Not SHAD_HiQualityObjects Then DEBUG_HiQuality = False


            rt_ShadowMap.SetRenderTargetToDevice()
            DX_Device.Clear(ClearFlags.Target Or ClearFlags.ZBuffer, Color.FromArgb(255, 255, 255, 255), 1, 0)

            .Technique = SMAP_RenderShadow

            .Begin(FX.None)
            .BeginPass(0)
        End With
    End Sub

    Friend Sub Effect_SMAP_EndRenderShadow()
        SMAP.End()
        SMAP.EndPass()
        rt_ShadowMap.RestoreOldRenderTarget()
    End Sub

    Friend Sub Effect_SMAP_PrepareForRenderScene()
        With SMAP


            mtx_ViewProj = mtx_CameraView * mtx_CameraProj


            ' ----------------------------------------------------- dynamic interpolation
            If Form2.chk_Test1.Checked Then
                Static t As Single
                t = (t + 1) Mod 2
                mtx_ViewProj *= Matrix.Translation(0, t * 0.002F, 0)
            End If


            ' -------------------------------------------------------------------------------
            .SetValue(SMAP_Bias, SHAD_Bias * 0.0001F + f_ZetaRange * SHAD_BiasAutoRange) ' / XPSM_ShadowMapSize)
            .SetValue(SMAP_FilterGain, SHAD_FilterGain)


            Select Case SHAD_FilterType
                Case -6 : .Technique = SMAP_RenderScene_Test
                Case -5 : .Technique = SMAP_RenderScene_Fast5
                Case -4 : .Technique = SMAP_RenderScene_Fast4
                Case -3 : .Technique = SMAP_RenderScene_Fast3
                Case -2 : .Technique = SMAP_RenderScene_Fast2
                Case -1 : .Technique = SMAP_RenderScene_Fast1
                Case 0 : .Technique = SMAP_RenderScene_Hard
                Case 1 : .Technique = SMAP_RenderScene_Soft1
                Case 2 : .Technique = SMAP_RenderScene_Soft2
                Case 3 : .Technique = SMAP_RenderScene_Soft3
                Case 4 : .Technique = SMAP_RenderScene_Soft4
                Case 5 : .Technique = SMAP_RenderScene_Soft5
                Case 6 : .Technique = SMAP_RenderScene_Soft6
                Case 7 : .Technique = SMAP_RenderScene_Soft7
                Case 8 : .Technique = SMAP_RenderScene_Soft8
                Case 9 : .Technique = SMAP_RenderScene_Soft9
            End Select

            If Not DEBUG_ViewFinalMap Then
                rt_ShadowTraspMap.SetRenderTargetToDevice()
                DX_Device.Clear(ClearFlags.Target Or ClearFlags.ZBuffer, Color.FromArgb(255, 255, 255, 0), 1, 0)

                If Not SHAD_HiQualityObjects Then DEBUG_HiQuality = False
            End If

            .Begin(FX.None)
            .BeginPass(0)
        End With
    End Sub

    Friend Sub Effect_SMAP_EndRenderScene()
        DEBUG_HiQuality = b_oldQuality
        SMAP.End()
        SMAP.EndPass()
        rt_ShadowTraspMap.RestoreOldRenderTarget()
    End Sub

    Friend Sub Effect_SMAP_DisplayTraspMap()
        With DX_Device

            .TextureState(0).ColorOperation = TextureOperation.SelectArg1
            .TextureState(0).ColorArgument1 = TextureArgument.TFactor
            If SHAD_ShadowAddSmooth Then
                .TextureState(0).AlphaOperation = TextureOperation.AddSmooth
            Else
                .TextureState(0).AlphaOperation = TextureOperation.Add
            End If
            .TextureState(0).AlphaArgument1 = TextureArgument.TextureColor
            .TextureState(0).AlphaArgument2 = TextureArgument.TFactor
            .TextureState(0).ResultArgument = TextureArgument.Current

            .RenderState.AlphaBlendEnable = True
            .RenderState.BlendOperation = BlendOperation.Add
            .RenderState.SourceBlend = Blend.InvSourceAlpha
            .RenderState.DestinationBlend = Blend.SourceAlpha


            .TextureState(0).TextureCoordinateIndex = 0
            .TextureState(0).TextureTransform = TextureTransform.Disable

            .TextureState(1).ColorOperation = TextureOperation.Disable
            .TextureState(1).AlphaOperation = TextureOperation.Disable

            If Form2.chk_Test3.Checked Then
                SetTextureFactor(SHAD_ShadowDensity, 0.2, 0.0, 0.0)
            Else
                SetTextureFactor(SHAD_ShadowDensity, 0.0, 0.0, 0.0)
            End If

            DX8_RenderFrontQuad(rt_ShadowTraspMap.texture, 0, 0, 1, 1)

            .RenderState.AlphaBlendEnable = False
        End With
    End Sub

    Private Sub SetTextureFactor(ByVal a As Single, ByVal r As Single, ByVal g As Single, ByVal b As Single)
        DX_Device.RenderState.TextureFactor = Color.FromArgb(255 - CInt(a * 255), CInt(r * 255), CInt(g * 255), CInt(b * 255)).ToArgb
    End Sub


    Friend Sub Effect_SMAP_DisplayShadowMap()
        With SMAP
            .SetValue(SMAP_VSM_Zstart, VSM_Zstart)
            .SetValue(SMAP_VSM_Zrange, VSM_Zrange)
            .Technique = SMAP_DrawShadowQuad
            .Begin(FX.None)
            .BeginPass(0)
            DX8_RenderFrontQuad(rt_ShadowMap.texture, 0.8, 0.8, 0.19, 0.19)
            .End()
            .EndPass()
        End With
    End Sub








    ' ===============================================================================================
    '   EFFECT - COMMON FUNCTIONS
    ' ===============================================================================================

    ' ----------------------------------------------------------------------------------------------
    '  ComputeSceneParams()
    '  Computes the near & far clip planes for the virtual camera based on the scene
    ' ----------------------------------------------------------------------------------------------
    '  Bounds the field of view for the virtual camera based on a swept-sphere/frustum intersection
    '  If the swept sphere representing the extrusion of an object's bounding sphere along the light
    '   direction intersects the view frustum, the object is added to a list of shadow casters  
    '  The field of view is the minimum cone containing all eligible bounding spheres
    ' ----------------------------------------------------------------------------------------------



    ' --------------------------------------------------------------------------------------------------
    '  10mS total if computing BoundingBox for 1020 objects
    '   1mS total if all BoundingBox are pre-calculated ( with 1020 objects )
    ' --------------------------------------------------------------------------------------------------
    Private Sub ComputeSceneParams()

        'Dim t As PrecisionTimer = New PrecisionTimer


        ' ----------------------------------------------------- LightDir points to the light 
        '                                                                SweepDir points from light to scene 
        Dim sweepDir As Vector3 = -Shadow_LightDir


        ' ----------------------------------------------------- frustum is in world space, bounding boxes are minimal size 
        '                                                       (xforming an AABB generally increases its volume)
        Dim sceneFrustum As Frustum = Nothing
        sceneFrustum.Init(mtx_CameraView * mtx_CameraProj)

        ' ----------------------------------------------------- preallocate to full size ( for speed )
        ReDim box_ShadowCasterPoints(Objects_3D.Length - 1)
        ReDim box_ShadowReceiverPoints(Objects_3D.Length - 1)

        Dim VisibleCount As Int32 = 0
        Dim CastersCount As Int32 = 0
        Dim ReceiversCount As Int32 = 0

        ' ------------------------------------------------------------------ iterate all the objects
        Dim hit As Boolean = False
        For i As Int32 = 0 To Objects_3D.Length - 1

            ' -------------------------------------------------------------- no objects without mesh
            If Objects_3D(i).MeshX.msh = Nothing Then
                Objects_3D(i).DrawAsVisible = True
                Continue For
            End If


            With Objects_3D(i)

                ' --------------------------------------------------------- init visible flags
                .DrawAsVisible = False
                .DrawAsShadowCaster = False
                .DrawAsShadowReceiver = False


                ' --------------------------------------------------------- compute BoundingBox
                Dim bbmin As Vector3
                Dim bbmax As Vector3

                If .ObjType = ObjTypes.SOFT_OBJECT Or .ObjType = ObjTypes.CLOTH Or .ObjType = ObjTypes.ROPE Then
                    DX_ComputeBoundingBoxFromMesh(.MeshX.msh, bbmin, bbmax)
                Else
                    bbmin = Objects_3D(i).MeshX.bbmin
                    bbmax = Objects_3D(i).MeshX.bbmax
                End If

                'bbmin.X = Math.Max(bbmin.X, -100)
                'bbmin.Y = Math.Max(bbmin.Y, -100)
                'bbmax.X = Math.Min(bbmax.X, 100)
                'bbmax.Y = Math.Min(bbmax.Y, 100)



                ' -------------------------------------------------------------- obj Box
                Dim objBox As BoundingBox
                objBox.Init(bbmin, bbmax)
                objBox = XFormBoundingBox(objBox, Objects_3D(i).MeshX.mtx)



                ' -------------------------------------------------------------- test obj distance
                Dim ValidDistance As Boolean
                If SHAD_MaxDistance > 0 Then
                    Dim cameraPos As Vector3 = Matrix_GetVectorPos(Matrix.Invert(mtx_CameraView))
                    Dim objPos As Vector3 = objBox.Centroid
                    Dim objDist As Single = (cameraPos - objPos).Length
                    ValidDistance = objDist < objBox.Size() * Form2.txt_Shadow_MaxDistance.NumericValue
                Else
                    ValidDistance = True
                End If


                ' -------------------------------------------------------------- test if obj is in frustum
                Select Case sceneFrustum.TestBox(objBox)

                    Case 0
                        ' ---------------------------- outside frustum -- test swept sphere for potential shadow caster
                        If ValidDistance Then
                            If Objects_3D(i).ShadowCaster Then
                                Dim instanceSphere As BoundingSphere
                                instanceSphere.Init(objBox)
                                If sceneFrustum.TestSweptSphere(instanceSphere, sweepDir) Then
                                    hit = True
                                    objBox = XFormBoundingBox(objBox, mtx_CameraView)
                                    box_ShadowCasterPoints(CastersCount) = objBox
                                    CastersCount += 1
                                    .DrawAsShadowCaster = True
                                End If
                            End If
                        End If

                    Case 1, 2
                        ' ------------------------------------- 1 = fully inside frustum.  so store large bounding box
                        '                                       2 = intersect frustum ( treated in the same way )
                        If ValidDistance Then
                            hit = True
                            objBox = XFormBoundingBox(objBox, mtx_CameraView)
                            If Objects_3D(i).ShadowCaster Then
                                box_ShadowCasterPoints(CastersCount) = objBox
                                CastersCount += 1
                                .DrawAsShadowCaster = True
                            End If
                            If Objects_3D(i).ShadowReceiver Then
                                box_ShadowReceiverPoints(ReceiversCount) = objBox
                                ReceiversCount += 1
                                .DrawAsShadowReceiver = True
                            End If
                        End If
                        VisibleCount += 1
                        .DrawAsVisible = True


                End Select

            End With
        Next

        ' -------------------------------------------------------------- redim to final size ( for speed )
        ReDim Preserve box_ShadowCasterPoints(CastersCount - 1)
        ReDim Preserve box_ShadowReceiverPoints(ReceiversCount - 1)




        ' -------------------------------------------------------------
        If Form2.Visible And DEBUG_ViewDebugParams Then
            Form2.Label_Debug1.Text = " Visible objects: " & VisibleCount.ToString & vbCrLf & _
                                      "  Shadow casters: " & CastersCount.ToString & vbCrLf & _
                                      "Shadow receivers: " & ReceiversCount.ToString
        End If

        'Form1.Label99.Text = t.GetFormattedTime("0.000")
    End Sub







    ' ----------------------------------------------------
    '  +++ 3 mS with 1020 objects +++
    ' ----------------------------------------------------
    Private Sub BuildXPSMProjectionMatrix()

        'Dim t As PrecisionTimer = New PrecisionTimer

        ' 
        ' XPSM_coef tuned for near/far objects trade off and scene scale
        '
        Dim coeff As Single = SHAD_Coefficent * 0.05F

        ' ---------------------------------------------------- compute shadow casters & receivers
        ComputeSceneParams()


        ' ---------------------------------------------------- shadow casters and receivers all defined in VIEW SPACE 
        '                                                      copy shadow casters and receivers
        Dim index As Int32
        Dim tempbox As BoundingBox
        Dim shadowCasters(box_ShadowCasterPoints.Length * 8 - 1) As Vector3
        Dim shadowReceivers(box_ShadowReceiverPoints.Length * 8 - 1) As Vector3

        If box_ShadowCasterPoints.Length > 0 Then
            index = 0
            For j As Int32 = 0 To box_ShadowCasterPoints.Length - 1
                tempbox.Init(box_ShadowCasterPoints(j))
                For i As Int32 = 0 To 7
                    shadowCasters(index) = tempbox.Point_N(i)
                    index += 1
                Next
            Next
        End If

        If box_ShadowReceiverPoints.Length > 0 Then
            index = 0
            For j As Int32 = 0 To box_ShadowReceiverPoints.Length - 1
                tempbox.Init(box_ShadowReceiverPoints(j))
                For i As Int32 = 0 To 7
                    shadowReceivers(index) = tempbox.Point_N(i)
                    index += 1
                Next
            Next
        End If


        ' ----------------------------------------------------- transform light direction into view space
        Dim viewLightDir As Vector3
        viewLightDir = Vector3.TransformNormal(Shadow_LightDir, mtx_CameraView)
        viewLightDir = -viewLightDir


        ' ----------------------------------------------------- build light space lookAt matrix
        Dim lightSpace As Matrix
        lightSpace = Matrix.LookAtLH(New Vector3(0, 0, 0), viewLightDir, New Vector3(0, 1, 0))

        ' ----------------------------------------------------- transform view vector into light space
        '                                                       (0, 0, 1) * LightSpace
        Dim LS_View As Vector3 = New Vector3(lightSpace.M31, _
                                             lightSpace.M32, _
                                             lightSpace.M33)

        ' ----------------------------------------------------- project view vector into xy plane 
        '                                                       and find unit projection vector
        Dim unitP As Vector2 = New Vector2(LS_View.X, _
                                           LS_View.Y)

        Dim unitPLength As Single = unitP.Length


        ' ----------------------------------------------------- PPLS (post projection light space) transformation
        Dim PPLSTransform As Matrix

        ' ----------------------------------------------------- enable warping if Coeff <> 0
        Dim WarpingEnabled As Boolean
        WarpingEnabled = coeff <> 0 'And unitPLength > 0.01F
        ' originally disabled only if unitPLength < 0.1F ( miner lamp case )

        ' ----------------------------------------------------- use perspective reparameterization
        If WarpingEnabled Then

            ' ------------------------------------------------- normalize projection vector
            unitP *= 1.0F / unitPLength

            ' ------------------------------------------------- transform shadow casters and receivers into light space
            Dim LSshadowCasters(shadowCasters.Length - 1) As Vector3
            Dim LSshadowReceivers(shadowReceivers.Length - 1) As Vector3
            LSshadowCasters = Vector3.TransformCoordinate(shadowCasters, lightSpace)
            LSshadowReceivers = Vector3.TransformCoordinate(shadowReceivers, lightSpace)


            ' ------------------------------------------------- project casters and receivers points into 
            '                                                   unit projection vector and find minimal value 
            Dim minCastersProj As Single = 1.0F
            For i As Int32 = 0 To shadowCasters.Length - 1
                minCastersProj = Math.Min(minCastersProj, _
                                          LSshadowCasters(i).X * unitP.X + LSshadowCasters(i).Y * unitP.Y)
            Next

            Dim minReceiversProj As Single = 1.0F
            For i As Int32 = 0 To shadowReceivers.Length - 1
                minReceiversProj = Math.Min(minReceiversProj, _
                                            LSshadowReceivers(i).X * unitP.X + LSshadowReceivers(i).Y * unitP.Y)
            Next


            ' ------------------------------------------------- find focus region interval minimal point
            '                                                   max(min Casters, min Receivers)
            Dim minFocusRegionProj As Single = Math.Max(minCastersProj, minReceiversProj)


            ' 
            '  XPSM_epsilonW defines how far to "infinity" warping can be pushed in critical case
            '
            ' ------------------------------------------------- find maximum projection vector length
            '                                                   1 / (minProj * maxProjeVectorLength  + 1) != 0
            '                                                   minProj * maxProjVectorLength + 1 = thresholdW
            '                                                   !!!! TEST + RECEIVERS && CASTERS CODE
            Dim maxLengthP As Single = (SHAD_EpsilonW - 1.0F) / minFocusRegionProj


            ' ------------------------------------------------- find optimal (fixed warping) projection vector length
            Dim cosGamma As Single = LS_View.X * unitP.X + LS_View.Y * unitP.Y
            Dim lengthP As Single = coeff / cosGamma

            ' ------------------------------------------------- clip projection vector length
            If maxLengthP > 0.0F And lengthP > maxLengthP Then
                lengthP = maxLengthP
            End If


            'If Form2.Visible And DEBUG_ViewDebugParams Then
            '    Form2.Label_Debug2.Text = vbCr & "MaxLengthP: " & maxLengthP.ToString
            '    Form2.Label_Debug2.Text &= vbCr & "   LengthP: " & lengthP.ToString
            '    Form2.Label_Debug2.Text &= vbCr & "  cosGamma: " & cosGamma.ToString
            '    Form2.Label_Debug2.Text &= vbCr & "minFocusRegionProj: " & minFocusRegionProj.ToString
            'End If


            ' ------------------------------------------------- calculate Projection matrix
            Dim Projection As Matrix = Matrix_Build(1.0F, 0.0F, 0.0F, unitP.X * lengthP, _
                                                    0.0F, 1.0F, 0.0F, unitP.Y * lengthP, _
                                                    0.0F, 0.0F, 1.0F, 0.0F, _
                                                    0.0F, 0.0F, 0.0F, 1.0F)


            ' ------------------------------------------------- to maximize shadow map resolution 
            '                                                   calculate Z rotation basis 
            Dim ZRotation As Matrix
            If SHAD_XpsmCorrectedRotation Then
                ' ----------------------------------------------------------------- Y flip
                ZRotation = Matrix_Build(unitP.X, -unitP.Y, 0.0F, 0.0F, _
                                         unitP.Y, unitP.X, 0.0F, 0.0F, _
                                         0.0F, 0.0F, 1.0F, 0.0F, _
                                         0.0F, 0.0F, 0.0F, 1.0F)
                ' ----------------------------------------------------------------- X and Y flip
                'ZRotation = Matrix_Build(-unitP.X, unitP.Y, 0.0F, 0.0F, _
                '                         -unitP.Y, -unitP.X, 0.0F, 0.0F, _
                '                         0.0F, 0.0F, 1.0F, 0.0F, _
                '                         0.0F, 0.0F, 0.0F, 1.0F)
            Else
                ' ----------------------------------------------------------------- original XPSM rotation
                ZRotation = Matrix_Build(unitP.X, unitP.Y, 0.0F, 0.0F, _
                                         unitP.Y, -unitP.X, 0.0F, 0.0F, _
                                         0.0F, 0.0F, 1.0F, 0.0F, _
                                         0.0F, 0.0F, 0.0F, 1.0F)
            End If



            ' ------------------------------------------------- PPLS = post projection light space
            PPLSTransform = Projection * ZRotation

            WarpingEnabled = True
        Else


            ' ------------------------------------------------- use orthogonal shadows
            PPLSTransform = Matrix.Identity

            WarpingEnabled = False
        End If

        ' ----------------------------------------------------- final combined PPLS transformation
        Dim PPLSFinal As Matrix = lightSpace * PPLSTransform


        ' ----------------------------------------------------- transform casters and receivers 
        '                                                       into PPLS (post projection light space)
        Dim PPLScasters(-1) As Vector3
        Dim PPLSreceivers(-1) As Vector3
        TransformWithWClip_2(shadowCasters, PPLScasters, PPLSFinal, SHAD_EpsilonW)
        TransformWithWClip_2(shadowReceivers, PPLSreceivers, PPLSFinal, SHAD_EpsilonW)


        ' ----------------------------------------------------- find PPLS AABB's for caster*s and receivers
        Dim PPLSCastersAABB As BoundingBox
        PPLSCastersAABB.Init(PPLScasters)
        Dim PPLSReceiversAABB As BoundingBox
        PPLSReceiversAABB.Init(PPLSreceivers)


        ' ----------------------------------------------------- find post projection space focus region
        Dim PPLSFocusRegionAABB As BoundingBox
        IntersectAABB2D(PPLSFocusRegionAABB, PPLSCastersAABB, PPLSReceiversAABB)

        Dim minPtBox As Vector3 = PPLSFocusRegionAABB.minPt
        Dim maxPtBox As Vector3 = PPLSFocusRegionAABB.maxPt

        ' ----------------------------------------------------- build focus region linear basis
        '
        Dim minZ As Single = Math.Min(PPLSCastersAABB.minPt.Z, PPLSReceiversAABB.minPt.Z)
        Dim maxZ As Single = Math.Max(PPLSCastersAABB.maxPt.Z, PPLSReceiversAABB.maxPt.Z)

        'Dim dz As Single = PPLSCastersAABB.maxPt.Z - PPLSCastersAABB.minPt.Z
        'If maxZ <= PPLSCastersAABB.maxPt.Z + dz Then maxZ = PPLSCastersAABB.maxPt.Z + dz
        'If minZ >= PPLSCastersAABB.minPt.Z - dz Then minZ = PPLSCastersAABB.minPt.Z - dz



        ' ----------------------------------------------------- find focusRegionBasis ( inverted )
        Dim unitCubeBasis As Matrix

        unitCubeBasis = Matrix_Build(1.0F / (maxPtBox.X - minPtBox.X), 0.0F, 0.0F, 0.0F, _
                                     0.0F, 1.0F / (maxPtBox.Y - minPtBox.Y), 0.0F, 0.0F, _
                                     0.0F, 0.0F, 1.0F / (maxZ - minZ), 0.0F, _
                                     -minPtBox.X / (maxPtBox.X - minPtBox.X), _
                                     -minPtBox.Y / (maxPtBox.Y - minPtBox.Y), _
                                     -minZ / (maxZ - minZ), _
                                     1.0F)

        ' ----------------------------------------------------- transform from a unit cube into d3d normalized space
        Dim normalizedSpaceD3D As Matrix
        normalizedSpaceD3D = Matrix_Build(2.0F, 0.0F, 0.0F, 0.0F, _
                                          0.0F, 2.0F, 0.0F, 0.0F, _
                                          0.0F, 0.0F, 1.0F, 0.0F, _
                                          -1.0F, -1.0F, 0.0F, 1.0F)


        ' ----------------------------------------------------- finally multiply all transformations
        '
        mtx_LightViewProj = mtx_CameraView * PPLSFinal * unitCubeBasis * normalizedSpaceD3D



        ' -------------------------------------------------------- compute correct PPLS Z bias ( only for debug )
        If WarpingEnabled Then
            f_ZetaRange = 1 / (maxZ - minZ)
        Else
            f_ZetaRange = 0
        End If

        ' -------------------------------------------------------- compute start and range for ViewShadowMap
        Dim TotalRange As Single = maxZ - minZ
        VSM_Zstart = (PPLSCastersAABB.minPt.Z - minZ) / TotalRange
        VSM_Zrange = (PPLSCastersAABB.maxPt.Z - PPLSCastersAABB.minPt.Z) / TotalRange



        If Form2.Visible And DEBUG_ViewDebugParams Then

            Form2.Label_Debug2.Text = vbCr & "f_ZetaRange: " & f_ZetaRange.ToString()

            Form2.Label_Debug2.Text &= vbCr & vbCr & "maxZ: " & maxZ.ToString & vbCr & _
                                                 "minZ: " & minZ.ToString

            'Form2.Label_Debug2.Text &= vbCr & vbCr & PPLSFinal.ToString

            'Form2.Label_Debug2.Text &= vbCr & vbCr & "PPLSCastersAABB.MaxPt" & vbCr & PPLSCastersAABB.maxPt.ToString
            'Form2.Label_Debug2.Text &= vbCr & vbCr & "PPLSCastersAABB.MinPt" & vbCr & PPLSCastersAABB.minPt.ToString

            'Form2.Label_Debug2.Text &= vbCr & vbCr & "PPLSReceiversAABB.MaxPt" & vbCr & PPLSReceiversAABB.maxPt.ToString
            'Form2.Label_Debug2.Text &= vbCr & vbCr & "PPLSReceiversAABB.MinPt" & vbCr & PPLSReceiversAABB.minPt.ToString


            'Form2.Label_Debug2.Text &= vbCr & vbCr & "XPSM_EpsilonW: " & XPSM_EpsilonW.ToString()




            'Form2.Label_Debug2.Text &= vbCr & vbCr & "PPLSCastersAABB.maxPt.Z: " & PPLSCastersAABB.maxPt.Z.ToString & vbCr & _
            '                                     "PPLSCastersAABB.minPt.Z: " & PPLSCastersAABB.minPt.Z.ToString
            'Form2.Label_Debug2.Text &= vbCr & vbCr & "maxPtBox" & vbCr & maxPtBox.ToString
            'Form2.Label_Debug2.Text &= vbCr & vbCr & "minPtBox" & vbCr & minPtBox.ToString
        End If

        'Form1.Label99.Text = t.GetFormattedTime("0.000")
    End Sub



    Private Function IntersectAABB2D(ByRef c As BoundingBox, ByVal a As BoundingBox, ByVal b As BoundingBox) As Boolean

        c.Init()
        c.minPt.Z = 0.0F
        c.maxPt.Z = 0.0F

        If a.maxPt.X < b.minPt.X Or a.minPt.X > b.maxPt.X Then Return False
        If a.maxPt.Y < b.minPt.Y Or a.minPt.Y > b.maxPt.Y Then Return False

        ' ---------------------------------------------- max x
        If a.maxPt.X <= b.maxPt.X Then
            c.maxPt.X = a.maxPt.X
        Else
            c.maxPt.X = b.maxPt.X
        End If
        ' ---------------------------------------------- min x
        If a.minPt.X <= b.minPt.X Then
            c.minPt.X = b.minPt.X
        Else
            c.minPt.X = a.minPt.X
        End If
        ' ---------------------------------------------- max y
        If a.maxPt.Y <= b.maxPt.Y Then
            c.maxPt.Y = a.maxPt.Y
        Else
            c.maxPt.Y = b.maxPt.Y
        End If
        ' ---------------------------------------------- min y
        If a.minPt.Y <= b.minPt.Y Then
            c.minPt.Y = b.minPt.Y
        Else
            c.minPt.Y = a.minPt.Y
        End If

    End Function




    ' -------------------------------------------------------------------------- original version
    'Private Sub TransformWithWClip_1(ByVal inVectors() As Vector3, _
    '                                 ByRef outVectors() As Vector3, _
    '                                 ByVal m As Matrix, _
    '                                 ByVal wThreshold As Single)

    '    ReDim outVectors(inVectors.Length - 1)
    '    Dim x, y, z, w As Single
    '    Dim counter As Int32 = 0

    '    For i As Int32 = 0 To inVectors.Length - 1
    '        x = inVectors(i).X * m.M11 + inVectors(i).Y * m.M21 + inVectors(i).Z * m.M31 + m.M41
    '        y = inVectors(i).X * m.M12 + inVectors(i).Y * m.M22 + inVectors(i).Z * m.M32 + m.M42
    '        z = inVectors(i).X * m.M13 + inVectors(i).Y * m.M23 + inVectors(i).Z * m.M33 + m.M43
    '        w = inVectors(i).X * m.M14 + inVectors(i).Y * m.M24 + inVectors(i).Z * m.M34 + m.M44

    '        ' The issue is that when the .w coordinate is negative, projection 
    '        ' "flips" your resulting x and y coordinates and screws up the bounding box calculation.
    '        ' If you think about the perspective transform as an "X" (draw it out) it'll make sense. 
    '        ' The solution I came up with was to "project" my points with the absolute value of w 
    '        ' instead of w directly. Then clamp it. This gave me the result I was looking for. 
    '        w = Math.Abs(w)

    '        If w >= wThreshold Then
    '            outVectors(counter) = VEC3(x / w, y / w, z / w)
    '            counter += 1
    '        End If
    '    Next

    '    ReDim Preserve outVectors(counter - 1)
    'End Sub


    ' -------------------------------------------------------------------------- patched version - OK
    Private Sub TransformWithWClip_2(ByVal inVectors() As Vector3, _
                                     ByRef outVectors() As Vector3, _
                                     ByVal m As Matrix, _
                                     ByVal wThreshold As Single)

        ReDim outVectors(inVectors.Length - 1)
        Dim x, y, z, w As Single

        For i As Int32 = 0 To inVectors.Length - 1
            x = inVectors(i).X * m.M11 + inVectors(i).Y * m.M21 + inVectors(i).Z * m.M31 + m.M41
            y = inVectors(i).X * m.M12 + inVectors(i).Y * m.M22 + inVectors(i).Z * m.M32 + m.M42
            z = inVectors(i).X * m.M13 + inVectors(i).Y * m.M23 + inVectors(i).Z * m.M33 + m.M43
            w = inVectors(i).X * m.M14 + inVectors(i).Y * m.M24 + inVectors(i).Z * m.M34 + m.M44

            ' The issue is that when the .w coordinate is negative, projection 
            ' "flips" your resulting x and y coordinates and screws up the bounding box calculation.
            ' If you think about the perspective transform as an "X" (draw it out) it'll make sense. 
            ' The solution I came up with was to "project" my points with the absolute value of w 
            ' instead of w directly. Then clamp it. This gave me the result I was looking for. 
            w = Math.Abs(w)

            If w < wThreshold Then
                outVectors(i) = New Vector3(x / w, y / w, 0)
            Else
                outVectors(i) = New Vector3(x / w, y / w, z / w)
            End If
        Next
    End Sub





End Module
