﻿
Imports Microsoft.DirectX
Imports Microsoft.DirectX.DirectSound

Module Sounds
    Private DS_Device As DirectSound.Device
    Private DS_Listener As DirectSound.Listener3D
    Private Sound3D(39) As Buffer3D
    Private Sounds(39) As SecondaryBuffer
    Private soundPath As String


    Private Const CYLINDER_FIRST As Int32 = 0
    Private Const CYLINDER_LAST As Int32 = 17

    Private Const BOX_FIRST As Int32 = 18
    Private Const BOX_LAST As Int32 = 24

    Private Const BALL_FIRST As Int32 = 25
    Private Const BALL_LAST As Int32 = 30

    Private Const CRASH_FIRST As Int32 = 31
    Private Const CRASH_LAST As Int32 = 34

    Private Const CAR_FIRST As Int32 = 35
    Private Const CAR_LAST As Int32 = 37

    Private CylIndex As Int32
    Private BoxIndex As Int32
    Private BallIndex As Int32
    Private CrashIndex As Int32
    Private CarIndex As Int32
    Private CarMotorIndex As Int32 = 38


    Friend Sub Sounds_Init()
        ' --------------------------------------------------------------------------
        '  it is important to "SHOW" the form before to play any sound
        '  sounds played before will play always at MAX-VOLUME 
        ' --------------------------------------------------------------------------
        Form1.Show()
        ' --------------------------------------------------------------------------

        ' =============================================================== DirectSoundDevice
        Try
            DS_Device = New Device()
        Catch
            MsgBox("Can not open Direct Sound Device")
            Return
        End Try
        DS_Device.SetCooperativeLevel(Form1.Handle, CooperativeLevel.Priority)

        'Dim sp As Speakers
        'sp.Stereo = True
        'DSound_Device.SpeakerConfig = sp

        ' ================================================================ LISTENER BUFFER
        Dim bd As BufferDescription = New BufferDescription
        bd.PrimaryBuffer = True
        bd.Control3D = True
        Dim b As Buffer = New Buffer(bd, DS_Device)

        ' ================================================================ LISTENER
        DS_Listener = New Listener3D(b)
        DS_Listener.Deferred = True
        '
        ' ================================================================ LISTENER ORIENTATION AND POSITION 
        '                                                                  the listener looks forward and is at zero pos      
        SetSoundListenerWithMatrix(Matrix.Identity)

        soundPath = Application.StartupPath & "\Media\Sounds\"
        Load3DSound(0, "Bar01.wav")
        Load3DSound(1, "Bar02.wav")
        Load3DSound(2, "Bar03.wav")
        Load3DSound(3, "Bar04.wav")
        Load3DSound(4, "Bar05.wav")
        Load3DSound(5, "Bar06.wav")
        Load3DSound(6, "Bar07.wav")
        Load3DSound(7, "Bar08.wav")
        Load3DSound(8, "Bar09.wav")
        Load3DSound(9, "Bar01.wav")
        Load3DSound(10, "Bar02.wav")
        Load3DSound(11, "Bar03.wav")
        Load3DSound(12, "Bar04.wav")
        Load3DSound(13, "Bar05.wav")
        Load3DSound(14, "Bar10.wav")
        Load3DSound(15, "Bar11.wav")
        Load3DSound(16, "Bar12.wav")
        Load3DSound(17, "Bar13.wav")

        Load3DSound(18, "Box01.wav")
        Load3DSound(19, "Box02.wav")
        Load3DSound(20, "Box03.wav")
        Load3DSound(21, "Box04.wav")
        Load3DSound(22, "Box05.wav")
        Load3DSound(23, "Box06.wav")
        Load3DSound(24, "Box07.wav")

        Load3DSound(25, "Ball05.wav")
        Load3DSound(26, "Ball05.wav")
        Load3DSound(27, "Ball05.wav")
        Load3DSound(28, "Ball05.wav")
        Load3DSound(29, "Ball05.wav")
        Load3DSound(30, "Ball05.wav")

        Load3DSound(31, "Crash01.wav")
        Load3DSound(32, "Crash02.wav")
        Load3DSound(33, "Crash03.wav")
        Load3DSound(34, "Crash04.wav")

        Load3DSound(35, "Car01.wav")
        Load3DSound(36, "Car02.wav")
        Load3DSound(37, "Car03.wav")

        Load3DSound(38, "Motor_Car1.wav")

        'LoadAmbientSound(39, "Out01.wav")
        'StartAmbientSound(39, -9)
    End Sub


    Friend Sub Sounds_Close()
        For i As Int32 = 0 To UBound(Sounds)
            If Sounds(i) IsNot Nothing Then
                Sounds(i).Stop()
                Sounds(i).Dispose()
                Sounds(i) = Nothing
            End If
        Next
    End Sub

    Friend Sub Sounds_StopAllSounds()
        For i As Int32 = 0 To UBound(Sounds)
            If Sounds(i) IsNot Nothing Then
                Sounds(i).Stop()
            End If
        Next
    End Sub



    Private Sub LoadAmbientSound(ByVal index As Int32, ByVal SoundFileName As String)
        Try
            Dim d As BufferDescription = New BufferDescription()
            d.ControlVolume = True
            Sounds(index) = New SecondaryBuffer(soundPath & SoundFileName, d, DS_Device)
        Catch
            MsgBox("Can not find: """ & soundPath & SoundFileName, MsgBoxStyle.Information)
        End Try
    End Sub

    Private Sub StartAmbientSound(ByVal SoundIndex As Int32, ByVal Volume As Single)
        If Sounds(SoundIndex) Is Nothing Then Return
        If Volume > 0 Then Volume = 0
        If Volume < -100 Then Volume = -100
        With Sounds(SoundIndex)
            .Volume = CInt(Volume * 100)
            .Play(0, BufferPlayFlags.Looping Or BufferPlayFlags.Default)
        End With
    End Sub


    Private Sub Load3DSound(ByVal index As Int32, ByVal SoundFileName As String)
        Try
            Dim d As BufferDescription = New BufferDescription()
            d.Control3D = True
            d.ControlVolume = True
            d.ControlFrequency = True
            d.Guid3DAlgorithm = DSoundHelper.Guid3DAlgorithmDefault
            Sounds(index) = New SecondaryBuffer(soundPath & SoundFileName, d, DS_Device)
        Catch
            MsgBox("Can not find: """ & soundPath & SoundFileName, MsgBoxStyle.Information)
        End Try
    End Sub

    Private Sub Start3DSound(ByVal SoundIndex As Int32, _
                             ByVal Position As Vector3, _
                             ByVal frequency As Single, _
                             ByVal Volume As Single, _
                             Optional ByVal Looping As BufferPlayFlags = BufferPlayFlags.Default)

        If Volume > 0 Then Volume = 0
        If Volume < -100 Then Volume = -100
        If frequency > 192000 Then frequency = 192000
        If frequency < 100 Then frequency = 100

        'If SoundIndex = 20 Or SoundIndex >= 18 Then Return

        With Sounds(SoundIndex)

            ' Debug.Print(SoundIndex.ToString)

            .Volume = CInt(Volume * 100)
            .Frequency = CInt(frequency)

            If Sound3D(SoundIndex) = Nothing Then
                Sound3D(SoundIndex) = New Buffer3D(Sounds(SoundIndex))
            End If

            With Sound3D(SoundIndex)
                .Mode = Mode3D.Normal    ' <<< remeber to use "NORMAL" and not "HEAD RELATIVE" !
                .MinDistance = 10
                .Position = Position
                .Deferred = True
            End With

            If Not .Status.Playing Then
                .Play(0, Looping)
            End If
        End With

    End Sub


    Friend Sub SetSoundListenerWithMatrix(ByVal mtx As Matrix)
        If DS_Listener Is Nothing Then Return
        '
        mtx.Invert()  ' <<< remeber to use INVERTED ViewMatrix !
        '
        Dim o As Listener3DOrientation
        Dim pos As Vector3
        With mtx
            o.Top = VEC3(.M21, .M22, .M23)
            o.Front = VEC3(.M31, .M32, .M33)
            pos = VEC3(.M41, .M42, .M43)
            'Debug.Print(o.Front.ToString)
            'Debug.Print(DS_Listener.Position.ToString)
        End With
        DS_Listener.Position = pos
        DS_Listener.Orientation = o
        DS_Listener.CommitDeferredSettings()
    End Sub


    Private Function FindNextFreeIndex(ByRef index As Int32, ByVal indexmin As Int32, ByVal indexmax As Int32) As Int32

        Dim bestvalue As Int32 = 0
        Dim value As Int32
        Dim bestcandidate As Int32

        For i As Int32 = 1 To indexmax - indexmin
            index += 1
            If index < indexmin Or index > indexmax Then index = indexmin

            If Sounds(index) Is Nothing Then Return -1
            ' ------------------------------------------------- use the first non-playing sound
            If Not Sounds(index).Status.Playing Then
                Return index
            End If
            ' ------------------------------------------------- or the sound with the highest play-position
            'value = Sounds(index).PlayPosition
            'If value > bestvalue Then
            '    bestvalue = value
            '    bestcandidate = index
            'End If

            value = Sounds(index).Volume
            If value <= bestvalue Then
                bestvalue = value
                bestcandidate = index
            End If
        Next

        index = bestcandidate
        Return index
    End Function



    Friend Enum SoundTypes As Int32
        NoSound
        Cylinder
        Box
        Sphere
        LittleSphere
        VeryLittleSphere
        Car
        Castle
        Cloth
        UseShape
    End Enum

    Friend Sub PlayObjectSound(ByVal objIndex As Int32)

        If DEBUG_DisableSounds Then Return
        If DS_Device Is Nothing Then Return
        If DX_EditingObject_DxObjectIndex = objIndex Then Exit Sub

        With Objects_3D(objIndex)
            Dim index As Int32
            Dim vol As Single
            Dim freq As Single

            'vol = -40 + CSng((.GlobalAcc.Length * 0.06) ^ 2)
            vol = -35 + CSng((.GlobalAcc.Length * 0.16))

            Select Case .SoundType
                Case SoundTypes.Castle
                    index = FindNextFreeIndex(BoxIndex, BOX_FIRST, BOX_LAST)
                    freq = 10000 + 10000 * Rnd()

                Case SoundTypes.Cloth
                    index = FindNextFreeIndex(BoxIndex, BOX_FIRST, BOX_LAST)
                    freq = 3000 + 5000 * Rnd()
                    vol += 12

                Case SoundTypes.Cylinder
                    index = FindNextFreeIndex(CylIndex, CYLINDER_FIRST, CYLINDER_LAST)
                    freq = 12000 + 10000 * Rnd()

                Case SoundTypes.Box
                    index = FindNextFreeIndex(BoxIndex, BOX_FIRST, BOX_LAST)
                    freq = 20000 + 20000 * Rnd()

                Case SoundTypes.Sphere
                    If .Shooted Then
                        .Shooted = False
                        index = -1
                    Else
                        index = FindNextFreeIndex(BallIndex, BALL_FIRST, BALL_LAST)
                        freq = 8000 + 15000 * Rnd()
                    End If

                Case SoundTypes.LittleSphere
                    index = FindNextFreeIndex(BallIndex, BALL_FIRST, BALL_LAST)
                    freq = 30000

                Case SoundTypes.VeryLittleSphere
                    index = FindNextFreeIndex(BallIndex, BALL_FIRST, BALL_LAST)
                    freq = 80000

                Case SoundTypes.Car
                    If Math.Abs(.LocalAcc.X) > Math.Abs(.LocalAcc.Z) + Math.Abs(.LocalAcc.Y) And Matrix_GetVectorUp(.MeshX.mtx).Y > 0.3 Then
                        vol += 4.0F
                        index = FindNextFreeIndex(CarIndex, CAR_FIRST, CAR_LAST)
                        freq = 20000 + 40000 * Rnd()
                    Else
                        vol -= 8.0F
                        index = FindNextFreeIndex(CrashIndex, CRASH_FIRST, CRASH_LAST)
                        freq = 25000 + 25000 * Rnd()
                    End If
                Case Else
                    index = -1
            End Select
            '
            If index >= 0 Then
                .Position = VEC3(.MeshX.mtx.M41, .MeshX.mtx.M42, .MeshX.mtx.M43)
                Start3DSound(index, .Position, freq, vol)
            End If
        End With
    End Sub

    Friend Sub PlayCarMotor(ByVal objIndex As Int32)

        If DEBUG_DisableSounds Then Return
        If DS_Device Is Nothing Then Return
        'Exit Sub
        If DX_EditingObject_DxObjectIndex = objIndex Then Exit Sub

        With Objects_3D(objIndex)
            If .ObjType = ObjTypes.CAR Then
                If objIndex = VehicleActive Then

                    Dim sbuf As SecondaryBuffer = Sounds(CarMotorIndex)
                    Dim vol As Single
                    Dim freq As Single

                    Const SoundVolume As Int32 = -6
                    Const SoundFreqMin As Int32 = 500
                    Const SoundFreqMax As Int32 = 15000
                    'If .MotorRpm < 0.15 Then .MotorRpm = 0.15
                    freq = CInt(SoundFreqMin + .MotorRpm * (SoundFreqMax - SoundFreqMin))
                    Dim db As Single = SoundVolume + (.MotorRpm - 0.9F) * 20
                    vol = db * 100
                    'If vol > 0 Then vol = 0

                    If freq < 1500 Then
                        If sbuf.Status.Playing Then
                            sbuf.Stop()
                        End If
                    Else
                        If Not sbuf.Status.Playing Then
                            'sbuf.Play(0, BufferPlayFlags.Looping)
                            Start3DSound(CarMotorIndex, VEC3(0, 0, 0), freq, vol, BufferPlayFlags.Looping)
                        End If
                        sbuf.Frequency = CInt(freq)
                        sbuf.Volume = CInt(vol)
                        Sound3D(CarMotorIndex).Position = VEC3(.MeshX.mtx.M41, .MeshX.mtx.M42, .MeshX.mtx.M43)
                        DS_Listener.CommitDeferredSettings()
                    End If
                End If
            End If
        End With
    End Sub


End Module
