
//-----------------------------------------------------------------------------
// File: perPixel.fx   
//
// From "RobyDX_ShadersWithEffectFiles"
//
//-----------------------------------------------------------------------------


//semantic respect my framework specification
//la semantica rispetta le specifiche del mio framework
float3 LightPosition;
float3 EyePosition;
float4x4 matView ;
float4x4 matViewProjection ;
float4 Ambient:Ambient;
float4 Specular:Specular;
float4 Diffuse:Diffuse;
float SpecularPower:Power;


//texture, like other variable
//texture come le altre variabili
texture base_Tex:BaseTex;

//but we use the sampler in pixel shader (we can set parameter in it)
//ma usiamo il sampler nel pixel shader (possiamo impostare i parametri)
sampler2D baseMap = sampler_state
{
//important, we must set texture in sampler
//importante, dobbiamo impostare la texture nel sampler
   Texture = (base_Tex);
   ADDRESSU = WRAP;
   ADDRESSV = WRAP;
   MINFILTER = LINEAR;
   MAGFILTER = LINEAR;
   MIPFILTER = LINEAR;
};

struct VS_INPUT 
{
   float4 Position : POSITION0;
   float2 Texcoord : TEXCOORD0;
   float3 Normal :   NORMAL0;
   
};

struct VS_OUTPUT 
{
   float4 Position :        POSITION0;
   float2 Texcoord :        TEXCOORD0;
   float3 ViewDirection :   TEXCOORD1;
   float3 LightDirection :  TEXCOORD2;
   float3 Normal :          TEXCOORD3;
   
};


//vertex shader
//we execute all linear operation in vertex shader and
//pass result to pixel shader
//eseguiamo tutte le operazioni lineari nel vertex shader e
//passiamo i risultati al pixel shader


VS_OUTPUT vs_main( VS_INPUT Input )
{
   VS_OUTPUT Output;

   Output.Position         = mul( Input.Position, matViewProjection );
   Output.Texcoord         = Input.Texcoord;
   
   float3 ObjectPosition = mul(Input.Position , matView );
   
   Output.ViewDirection    = EyePosition - ObjectPosition;
   Output.LightDirection   = LightPosition - ObjectPosition;
   Output.Normal           = mul(Input.Normal, matView  );
      
   return( Output );
   
}

struct PS_INPUT 
{
   float2 Texcoord :        TEXCOORD0;
   float3 ViewDirection :   TEXCOORD1;
   float3 LightDirection:   TEXCOORD2;
   float3 Normal :          TEXCOORD3;
   
};

//Pixel shader
//we use vertex shader results to calculate light in pixel shader (better result)
//usiamo i risultati del vertex shader per calcolare la luce nel pixel shade (migliori risultati)

float4 ps_mainSpecular( PS_INPUT Input ) : COLOR0
{      
   float3 LightDirection = normalize( Input.LightDirection );
   float3 Normal         = normalize( Input.Normal );
   float  NDotL           = dot( Normal, LightDirection ); 
   float3 Reflection     = normalize( ( ( 2.0f * Normal ) * ( NDotL ) ) - LightDirection ); 
   float3 ViewDirection  = normalize( Input.ViewDirection );
   float  RDotV           = max( 0.0f, dot( Reflection, ViewDirection ) );
   
   float4 BaseColor      = tex2D( baseMap, Input.Texcoord );
   float4 TotalAmbient   = Ambient * BaseColor; 
   float4 TotalDiffuse   = Diffuse * NDotL * BaseColor; 
   float4 TotalSpecular  = Specular * pow( RDotV, SpecularPower );
   
   return( saturate( TotalAmbient + TotalDiffuse + TotalSpecular ) );
      
}

//without specular (less heavy code)
//senza specular (codice meno pesante)

float4 ps_main( PS_INPUT Input ) : COLOR0
{      
   float3 LightDirection = normalize( Input.LightDirection );
   float3 Normal         = normalize( Input.Normal );
   float  NDotL           = dot( Normal, LightDirection ); 
   
   float4 BaseColor      = tex2D( baseMap, Input.Texcoord );
   float4 TotalAmbient   = Ambient * BaseColor; 
   float4 TotalDiffuse   = Diffuse * NDotL * BaseColor; 
   
   
   return( saturate( TotalAmbient + TotalDiffuse) );
      
}


//a technique is a set of shader
//una tecnica  un gruppo di shader

technique Textured_Phong
{
	
   //each pass is a couple of vertex and pixel shader
   //we must specify the shader version (for this we need at least 2.0)
   //ogni pass  una coppia di vertex e pixel shader
   //dobbiamo specificare la versione shader (in questo caso almeno 2.0)
   pass Pass_0
   {
      	VertexShader = compile vs_2_0 vs_main();
      	PixelShader = compile ps_2_0 ps_mainSpecular();
   }
   
   pass Pass_1
   {      
         VertexShader = compile vs_2_0 vs_main();
         PixelShader = compile ps_2_0 ps_main();
   }

}

