#pragma once

#include "D3D9.h"

#define RGB_MULTIPLIER 1.0f / 255.0f

struct Light : public D3DLIGHT9
{
public:
	Light(D3DLIGHTTYPE type = D3DLIGHT_DIRECTIONAL);		// Constructor creates a white directional light that looks at positive xz and negative y

	void SetDiffuseColor( float r, float g, float b );
	void SetDiffuseColor( D3DXCOLOR c )	{ Diffuse = c; }

	void SetAmbientColor( float r, float g, float b );
	void SetAmbientColor( D3DXCOLOR c )	{ Ambient = c; }

    void SetSpecularColor( float r, float g, float b );    
	void SetDirection( float x, float y, float z );
	void SetRange(float fRange) { Range = fRange; }
	void SetFalloff(float fFalloff) { Falloff = fFalloff; }
	void SetAttenuation0(float fAtt) { Attenuation0 = fAtt; }
	void SetAttenuation1(float fAtt){ Attenuation1 = fAtt; }
	void SetAttenuation2(float fAtt){ Attenuation2 = fAtt; }
	void SetTheta(float fTheta) { Theta = fTheta; }
	void SetPhi(float fPhi) { Phi = fPhi; }

	void Clear(); 

	void Normalize() { D3DXVec3Normalize( (D3DXVECTOR3*)&Direction, (D3DXVECTOR3*)&Direction ); }
};

class Material : public D3DMATERIAL9
{
public:
    Material();
	
	void SetGreen()			{ SetDiffuseColor( 0, 255, 0 ); }
	void SetRed()			{ SetDiffuseColor( 255, 0, 0 ); }
	void SetBlue()			{ SetDiffuseColor( 0, 0, 255 ); }	
	void SetColor( D3DXCOLOR  c ){ Diffuse = c; }

	void SetDiffuseColor( float r, float g, float b );	
	void SetDiffuseColor( D3DXCOLOR c )		{ Diffuse = c; }

	void SetAmbientColor( float r, float g, float b );
	void SetAmbientColor( D3DXCOLOR c )		{ Ambient = c; }

	void SetSpecularColor( float r, float g, float b );
	void SetSpecularColor( D3DXCOLOR c )	{ Specular = c; }

    void SetEmissiveColor( float r, float g, float b );    
	void SetEmissiveColor( D3DXCOLOR c )	{ Emissive = c; }

	void SetPower( float power ) { Power = power; }

	void Clear();
};

class Color 
{
public:	
	const static DWORD Black		= D3DCOLOR_RGBA( 0, 0, 0, 255 );
	const static DWORD Blue			= D3DCOLOR_RGBA( 0, 0, 255, 255 );
	const static DWORD Brown		= D3DCOLOR_RGBA( 165, 42, 42, 255 );
	const static DWORD Cyan			= D3DCOLOR_RGBA( 0, 255, 255, 255 );
	const static DWORD DarkBlue		= D3DCOLOR_RGBA( 0, 0, 139, 255 );
	const static DWORD Green		= D3DCOLOR_RGBA( 0, 255, 0, 255 );
	const static DWORD Grey			= D3DCOLOR_RGBA( 128, 128, 128, 255 );
	const static DWORD Gold			= D3DCOLOR_RGBA( 255, 215, 0, 255 );
	const static DWORD LightSkyBlue	= D3DCOLOR_RGBA( 135, 206, 250, 255 );
	const static DWORD LightBlue	= D3DCOLOR_RGBA( 173, 216, 230, 255 );
	const static DWORD Lime			= D3DCOLOR_RGBA( 210, 105, 30, 255 ); 
	const static DWORD Magenta		= D3DCOLOR_RGBA( 255, 0, 255, 255 );
	const static DWORD Maroon		= D3DCOLOR_RGBA( 128, 0, 0, 255 );
	const static DWORD Orange		= D3DCOLOR_RGBA( 255, 165, 0, 255 );
	const static DWORD Olive		= D3DCOLOR_RGBA( 128, 128, 0, 255 );
	const static DWORD Pink			= D3DCOLOR_RGBA( 255, 192, 203, 255 );	
	const static DWORD Purple		= D3DCOLOR_RGBA( 128, 0, 128, 255 );
	const static DWORD Red			= D3DCOLOR_RGBA( 255, 0, 0, 255 );
	const static DWORD Silver		= D3DCOLOR_RGBA( 192, 192, 192, 255 );
	const static DWORD Teal			= D3DCOLOR_RGBA( 0, 128, 128, 255 );
	const static DWORD Violet		= D3DCOLOR_RGBA( 238, 130, 238, 255 );
	const static DWORD White		= D3DCOLOR_RGBA( 255, 255, 255, 255 );	
	const static DWORD Yellow		= D3DCOLOR_RGBA( 255, 255, 0, 255 );	
};