#pragma once

#include "btBulletDynamicsCommon.h"
#include "D3D9.h"

class Mesh;
class ShapeDrawer;
struct Frame;

class Vehicle
{
public:
	Vehicle(void);	
	~Vehicle(void);

	bool Create( LPDIRECT3DDEVICE9 device, btDynamicsWorld *dynamicsWorld, btVector3 &position, btScalar mass );		
	void Render( ShapeDrawer *shapeDrawer = NULL );	
	void Update( float elapsedTime );
	void Destroy();

	btScalar GetSpeed()				{ return mRaycastVehicle->getCurrentSpeedKmHour(); }
	btScalar GetSteeringValue()		{ return mSteeringValue; }
	btVector3 GetDirection()		{ return mForward; }
	btTransform &GetWorldTransform() { return mRaycastVehicle->getRigidBody()->getWorldTransform(); } 
	Mesh * GetMesh()				{ return mChassisMesh; }

private:
	btRigidBody*		mChassisRigidBody;
	btCompoundShape*	mCompoundShape;
	btCollisionShape*	mChassisShape;
	btCollisionShape*	mSimplifiedConvexShape;

	btRaycastVehicle::btVehicleTuning	mVehicleTuning;
	btVehicleRaycaster*					mVehicleRaycaster;
	btRaycastVehicle*					mRaycastVehicle;

	btDynamicsWorld*		mDynamicsWorld;	

	// Stores vehicle direction
	btVector3	mForward; 

	Mesh		*mChassisMesh;	
	Mesh		*mWheelMesh;	

	// variables for motion
	float	mSteeringValue;
	float	mMaxEngineForce;//this should be engine/velocity dependent
	float	mMaxBreakingForce;
	float	mSteeringIncrement;
	float	mSteeringClamp;
	float	mEngineForce;
	float	mBreakingForce;

	// car characteristics
	float	mWheelRadius;
	float	mSuspensionStiffness;
	float	mSuspensionDamping;
	float	mSuspensionCompression;
	float	mWheelFriction;
	float	mRollInfluence;
	float	mWheelWidth;

	float	mSize;

	LPD3DXMESH	mWheelDebugMesh;
	LPDIRECT3DDEVICE9 mDevice;
};
