
#include "stdafx.h"

//#include "DXUT.h"

#include "Utils.h"
#include "Vertex.h"


// Bullet includes
#include <btBulletDynamicsCommon.h>
#include <LinearMath/btQuickprof.h>
#include "BulletCollision/CollisionShapes/btShapeHull.h"
//#include <BulletCollision/CollisionDispatch/btGhostObject.h>
//#include <BulletCollision/CollisionShapes/btHeightfieldTerrainShape.h>
//#include <BulletDynamics/Character/btKinematicCharacterController.h>




// ====================================================================================
//  D3DXMATRIX 4x4 are row-major matrices and btMatrix3x3 are column-major
//  reversed indexing implicitly transposes (flips along the diagonal) the matrix
// ====================================================================================

// ---------------------------------------------------------
// convert from D3DXMATRIX (DirectX) to btTransform (Bullet)
// ---------------------------------------------------------
//btTransform ConvertMatrix(D3DXMATRIX &mat) 
//{ 
//	btMatrix3x3 bulletRotation; 
//	btVector3 bulletPosition; 
//	
//	// --------------------------------------------------  copy rotation matrix 
//	for ( int row=0; row<3; ++row ) 
//		for ( int column=0; column<3; ++column ) 
//			bulletRotation[row][column] = mat.m[column][row];  
//
//	// --------------------------------------------------  copy position 
//	for ( int column=0; column<3; ++column ) 
//		 bulletPosition[column] = mat.m[3][column]; 
//	      
//	return btTransform( bulletRotation, bulletPosition ); 
//} 
// ------------------------------------------------------------
//  convert from btTransform (Bullet) to D3DXMATRIX (DirectX)
// ------------------------------------------------------------
//D3DXMATRIX ConvertMatrix(btTransform &trn) 
//{ 
//	D3DXMATRIX returnValue; 
//	D3DXMatrixIdentity(&returnValue); 
//	
//	btMatrix3x3 const & bulletRotation = trn.getBasis(); 
//	btVector3 const & bulletPosition = trn.getOrigin(); 
//       
//	// --------------------------------------------------  copy rotation matrix 
//	for ( int row=0; row<3; ++row ) 
//		for ( int column=0; column<3; ++column ) 
//			returnValue.m[row][column] = bulletRotation[column][row];  
//
//	// --------------------------------------------------  copy position 
//	for ( int column=0; column<3; ++column ) 
//		returnValue.m[3][column] = bulletPosition[column]; 
//	          
//	return returnValue; 
//} 
 

//D3DXMATRIX ConvertMatrix( btTransform &trn ) 
//{
//	btVector3 R = trn.getBasis().getColumn(0);
//	btVector3 U = trn.getBasis().getColumn(1);
//	btVector3 L = trn.getBasis().getColumn(2);
//	btVector3 P = trn.getOrigin();
//
//	D3DXVECTOR3 vR, vU, vL, vP;
//	vR.x = R.x(); vR.y = R.y(); vR.z = R.z();
//	vU.x = U.x(); vU.y = U.y(); vU.z = U.z();
//	vL.x = L.x(); vL.y = L.y(); vL.z = L.z();
//	vP.x = P.x(); vP.y = P.y(); vP.z = P.z();
//
//	D3DXMATRIX matOutput;
//	matOutput._11 = vR.x; matOutput._12 = vR.y; matOutput._13 = vR.z; matOutput._14 = 0.f;
//	matOutput._21 = vU.x; matOutput._22 = vU.y; matOutput._23 = vU.z; matOutput._24 = 0.f;
//	matOutput._31 = vL.x; matOutput._32 = vL.y; matOutput._33 = vL.z; matOutput._34 = 0.f;
//	matOutput._41 = vP.x; matOutput._42 = vP.y; matOutput._43 = vP.z; matOutput._44 = 1.f;
//
//	return matOutput;
//}

//D3DXMATRIX ConvertMatrix( btTransform &trn ) 
//{
//	btVector3 R = trn.getBasis().getColumn(0);
//	btVector3 U = trn.getBasis().getColumn(1);
//	btVector3 L = trn.getBasis().getColumn(2);
//	btVector3 P = trn.getOrigin();
//
//	D3DXMATRIX matOutput;
//	matOutput._11 = R.x(); matOutput._12 = R.y(); matOutput._13 = R.z(); matOutput._14 = 0.f;
//	matOutput._21 = U.x(); matOutput._22 = U.y(); matOutput._23 = U.z(); matOutput._24 = 0.f;
//	matOutput._31 = L.x(); matOutput._32 = L.y(); matOutput._33 = L.z(); matOutput._34 = 0.f;
//	matOutput._41 = P.x(); matOutput._42 = P.y(); matOutput._43 = P.z(); matOutput._44 = 1.f;
//
//	return matOutput;
//}



// ====================================================================================
//  D3DXMATRIX 4x4 are row-major matrices and btMatrix3x3 are column-major
// ====================================================================================

btTransform ConvertMatrix(D3DXMATRIX &m) 
{ 
	btMatrix3x3 bm; 
	btVector3 pos; 

	bm[0][0] = m._11; bm[1][0] = m._12; bm[2][0] = m._13;
	bm[0][1] = m._21; bm[1][1] = m._22; bm[2][1] = m._23;
	bm[0][2] = m._31; bm[1][2] = m._32; bm[2][2] = m._33;
	pos[0]   = m._41; pos[1]   = m._42; pos[2]   = m._43;

	return btTransform(bm, pos); 
} 

D3DXMATRIX ConvertMatrix( btTransform &trn ) 
{
	btMatrix3x3 bm = trn.getBasis();
	btVector3 pos = trn.getOrigin();

	D3DXMATRIX m;
	m._11 = bm[0][0]; m._12 = bm[1][0]; m._13 = bm[2][0]; m._14 = 0.f;
	m._21 = bm[0][1]; m._22 = bm[1][1]; m._23 = bm[2][1]; m._24 = 0.f;
	m._31 = bm[0][2]; m._32 = bm[1][2]; m._33 = bm[2][2]; m._34 = 0.f;
	m._41 = pos[0];  m._42 = pos[1];  m._43 = pos[2];  m._44 = 1.f;

	return m;
}


btVector3 Matrix_GetScale(D3DXMATRIX &m)
{
	return btVector3( btVector3(m._11, m._12, m._13).length(), 
					  btVector3(m._21, m._22, m._23).length(),
					  btVector3(m._31, m._32, m._33).length() );
}


void Matrix_SetScaleToOne(D3DXMATRIX* m)
{
	btVector3 scale = Matrix_GetScale(*m);
	if (scale.x() != 0) 
	{
		m->_11 /= scale.x();
		m->_12 /= scale.x();
		m->_13 /= scale.x();
	}
	if (scale.y() != 0) 
	{
		m->_21 /= scale.y();
		m->_22 /= scale.y();
		m->_23 /= scale.y();
	}
	if (scale.z() != 0) 
	{
		m->_31 /= scale.z();
		m->_32 /= scale.z();
		m->_33 /= scale.z();
	}
}


//void MatrixNormalize(D3DXMATRIX* m)
//{
//	btVector3 v1( m->_11, m->_12, m->_13);
//	v1.normalize();
//	m->_11 = v1.x();
//	m->_12 = v1.y();
//	m->_13 = v1.z();
//	btVector3 v2( m->_21, m->_22, m->_23);
//	v2.normalize();
//	m->_21 = v2.x();
//	m->_22 = v2.y();
//	m->_23 = v2.z();
//	btVector3 v3( m->_31, m->_32, m->_33);
//	v3.normalize();
//	m->_31 = v3.x();
//	m->_32 = v3.y();
//	m->_33 = v3.z();
//}




// ====================================================================================
//  VECTORS
// ====================================================================================

D3DXVECTOR3 ConvertVector( btVector3 &vec )
{
	return D3DXVECTOR3( vec.getX(), vec.getY(), vec.getZ() );
}
D3DXVECTOR3 ConvertVector( const btVector3 &vec )
{
	return D3DXVECTOR3( vec.getX(), vec.getY(), vec.getZ() );
}
btVector3 ConvertVector( D3DXVECTOR3 &vec )
{
	return btVector3(vec.x, vec.y, vec.z );
}
btVector3 ConvertVector( const D3DXVECTOR3 *vec )
{
	return btVector3(vec->x, vec->y, vec->z );
}


const btVector3 rotate( const btQuaternion& quat, const btVector3 & vec )
{
    float tmpX, tmpY, tmpZ, tmpW;
    tmpX = ( ( ( quat.getW() * vec.getX() ) + ( quat.getY() * vec.getZ() ) ) - ( quat.getZ() * vec.getY() ) );
    tmpY = ( ( ( quat.getW() * vec.getY() ) + ( quat.getZ() * vec.getX() ) ) - ( quat.getX() * vec.getZ() ) );
    tmpZ = ( ( ( quat.getW() * vec.getZ() ) + ( quat.getX() * vec.getY() ) ) - ( quat.getY() * vec.getX() ) );
    tmpW = ( ( ( quat.getX() * vec.getX() ) + ( quat.getY() * vec.getY() ) ) + ( quat.getZ() * vec.getZ() ) );
    return btVector3(
        ( ( ( ( tmpW * quat.getX() ) + ( tmpX * quat.getW() ) ) - ( tmpY * quat.getZ() ) ) + ( tmpZ * quat.getY() ) ),
        ( ( ( ( tmpW * quat.getY() ) + ( tmpY * quat.getW() ) ) - ( tmpZ * quat.getX() ) ) + ( tmpX * quat.getZ() ) ),
        ( ( ( ( tmpW * quat.getZ() ) + ( tmpZ * quat.getW() ) ) - ( tmpX * quat.getY() ) ) + ( tmpY * quat.getX() ) )
    );
}




// ====================================================================================
//   OBJ CREATIONS
// ====================================================================================


btRigidBody* CreateRigidBody(float mass, const btTransform& startTransform, btCollisionShape* shape)
{
	//rigidbody is dynamic if and only if mass is non zero, otherwise static
	bool isDynamic = (mass != 0.f);

	btVector3 localInertia(0,0,0);
	if (isDynamic) shape->calculateLocalInertia(mass,localInertia);
	
	// --------------------------------------------------------------------------------------------------------- 1
	// motionstate is recommended, provides interpolation capabilities, and only synchronizes 'active' objects
	//btDefaultMotionState* myMotionState = new btDefaultMotionState(startTransform);
	//btRigidBody::btRigidBodyConstructionInfo cInfo(mass, myMotionState, shape, localInertia);

	// --------------------------------------------------------------------------------------------------------- 2
	btRigidBody::btRigidBodyConstructionInfo cInfo(mass, NULL, shape, localInertia);
	cInfo.m_startWorldTransform = startTransform;

	btRigidBody* body = new btRigidBody(cInfo);

	body->m_vehicle = NULL;  // <<< added public "void* m_vehicle;" in btRigidBody.h

	// ----------------------------------------- a little default damping for all the objects
	body->setDamping(0.01f, 0.05f);

	return body;
}



btCollisionShape* CreateSimpleShape(const btVector3& ShapeSize, int shapeType)
{	
	btCollisionShape *collisionShape = NULL;
	switch (shapeType)
	{
		case BOX_SHAPE_PROXYTYPE:
			collisionShape = new btBoxShape(ShapeSize * 0.5f);
			break;
		case SPHERE_SHAPE_PROXYTYPE:
			{
				btVector3* positions = new btVector3[1];
				btScalar* radii = new btScalar[1];
				positions[0] = btVector3(0,0,0);
				radii[0] = ShapeSize.x()*0.5f;
				collisionShape = new btMultiSphereShape(positions, radii, 1);
				delete positions;
				delete radii;
			}
			break;
		case CAPSULE_SHAPE_PROXYTYPE:
			collisionShape = new btCapsuleShape(ShapeSize.x()*0.5f, ShapeSize.y());
			break;
		case CONE_SHAPE_PROXYTYPE:
			collisionShape = new btConeShape(ShapeSize.x()*0.5f, ShapeSize.y());
			break;
		case CYLINDER_SHAPE_PROXYTYPE:
			collisionShape = new btCylinderShape(ShapeSize*0.5f);
			break;
	}
	return collisionShape;
}



btRigidBody * CreateSimpleShapeBody(const btVector3& offsetPosition, 
									const btVector3& shapeSize, 
									int shapeType,
									float Margin,
									float Mass)
{	
	btCollisionShape *collisionShape = CreateSimpleShape(btVector3(1,1,1), shapeType);

	collisionShape->setLocalScaling(shapeSize);

	collisionShape->setMargin( Margin );

	btTransform trans;
	trans.setIdentity();
	trans.setOrigin(offsetPosition);

	btRigidBody *pBody = CreateRigidBody(Mass, trans, collisionShape);

	return pBody;
}




TriMeshData* CreateTriMeshData(float *pVertices,
						      __int32 *pIndices,
						      __int32 numVertices,
						      __int32 NumberBytesPerVertex,
						      __int32 numFaces,
						      bool Is16BitIndices)
{
	// ------------------------------------------------------------------------------------------------------
	TriMeshData* trimeshshape = new TriMeshData;
	//btBvhTriangleMeshShape *triangleMeshShape = CreateBvhTriangleMeshShape( pMesh, &trimeshshape );	
	// ------------------------------------------------------------------------------------------------------

	// ---------------------------------------------------------- extract VERTEXES
	__int32 VertexStride = NumberBytesPerVertex / 4;
	trimeshshape->vertices = new btScalar[numVertices * 3];	
	for(__int32 i = 0; i < numVertices; i++)
	{
		trimeshshape->vertices[i*3+0] = pVertices[i*VertexStride+0];
		trimeshshape->vertices[i*3+1] = pVertices[i*VertexStride+1];
		trimeshshape->vertices[i*3+2] = pVertices[i*VertexStride+2];		
	}	
	// ---------------------------------------------------------- extract FACES
	trimeshshape->indices = new int[numFaces * 3];
	if (Is16BitIndices)
	{
		WORD *pIndices16 = (WORD *)pIndices;
		for(__int32 i = 0; i < numFaces; i++)
		{	
			trimeshshape->indices[i*3+0] = pIndices16[i*3+0];
			trimeshshape->indices[i*3+1] = pIndices16[i*3+1];
			trimeshshape->indices[i*3+2] = pIndices16[i*3+2];		
		}
	}
	else
	{	
		for(__int32 i = 0; i < numFaces; i++)
		{	
			trimeshshape->indices[i*3+0] = pIndices[i*3+0];
			trimeshshape->indices[i*3+1] = pIndices[i*3+1];
			trimeshshape->indices[i*3+2] = pIndices[i*3+2];		
		}
	}

	return trimeshshape;
}



btBvhTriangleMeshShape *CreateBvhTriangleMeshShape(float *pVertices,
												   __int32 *pIndices,
												   __int32 numVertices,
												   __int32 NumberBytesPerVertex,
												   __int32 numFaces,
												   bool Is16BitIndices)
{

	// ---------------------------------------------------------- CREATE trimeshshape
	TriMeshData* trimeshshape = CreateTriMeshData(pVertices,
												  pIndices,
												  numVertices,
												  NumberBytesPerVertex,
												  numFaces,
												  Is16BitIndices);

	// ---------------------------------------------------------- create TriangleIndexVertexArray
	btTriangleIndexVertexArray* indexVertexArrays;
	indexVertexArrays = new btTriangleIndexVertexArray(numFaces, 
													   trimeshshape->indices, 
													   sizeof(int) * 3,
													   numVertices, 
													   trimeshshape->vertices, 
													   sizeof(btScalar) * 3);

	// ---------------------------------------------------------- create final SHAPE
	//
	//btBvhTriangleMeshShape *shape = new btScaledBvhTriangleMeshShape(indexVertexArrays, true);
	btBvhTriangleMeshShape *shape = new btBvhTriangleMeshShape(indexVertexArrays, true);
	//btBvhTriangleMeshShape *shape = new btBvhTriangleMeshShape(indexVertexArrays, false);

	// ---------------------------------------------------------- delete trimeshshape
	//	TODO - delete crashes
	//delete[] trimeshshape->indices;
	//delete[] trimeshshape->vertices;
	//delete trimeshshape;
	// ---------------------------------------------------------- delete all
	//delete indexVertexArrays;
	
	return shape;
}




btConvexHullShape *CreateConvexHullShape(float *pVertices,
									     __int32 *pIndices,
									     __int32 numVertices,
									     __int32 NumberBytesPerVertex,
									     __int32 numFaces,
									     bool Is16BitIndices)
{

	// ---------------------------------------------------------- CREATE triMeshShape
	//
	TriMeshData* trimeshshape = CreateTriMeshData(pVertices,
												   pIndices,
												   numVertices,
												   NumberBytesPerVertex,
												   numFaces,
												   Is16BitIndices);


	// --------------------------------------------------------- CREATE triangleMesh
	btTriangleMesh* triangleMesh = new btTriangleMesh();
	for(__int32 i = 0; i < numFaces; i++)
	{
		int index0 = trimeshshape->indices[i*3];
		int index1 = trimeshshape->indices[i*3+1];
		int index2 = trimeshshape->indices[i*3+2];

		btVector3 vertex0(trimeshshape->vertices[index0*3], trimeshshape->vertices[index0*3+1], trimeshshape->vertices[index0*3+2]);
		btVector3 vertex1(trimeshshape->vertices[index1*3], trimeshshape->vertices[index1*3+1], trimeshshape->vertices[index1*3+2]);
		btVector3 vertex2(trimeshshape->vertices[index2*3], trimeshshape->vertices[index2*3+1], trimeshshape->vertices[index2*3+2]);

		triangleMesh->addTriangle(vertex0,vertex1,vertex2);
	}

	btConvexShape* tmpConvexShape = new btConvexTriangleMeshShape(triangleMesh);




	// --------------------------------------------------------- Create a hull approximation
	btShapeHull* hull = new btShapeHull(tmpConvexShape);
	btScalar margin = tmpConvexShape->getMargin();
	hull->buildHull(margin);	
	
	btConvexHullShape* simplifiedConvexShape = new btConvexHullShape();
	for(int i= 0; i < hull->numVertices(); i++ )
	{
		simplifiedConvexShape->addPoint(hull->getVertexPointer()[i]);	
	}		


	// --------------------------------------------------------- delete triMeshShape
	delete[] trimeshshape->indices;
	delete[] trimeshshape->vertices;
	//delete trimeshshape;	// TODO delete trimeshshape crashes
	// --------------------------------------------------------- delete all
	delete triangleMesh;
	delete tmpConvexShape;
	delete hull;

	return simplifiedConvexShape;
}







// ========================================================================================================
//     btScaledBvhTriangleMeshShape
// ========================================================================================================

//#ifndef SCALED_BVH_TRIANGLE_MESH_SHAPE_H
//#define SCALED_BVH_TRIANGLE_MESH_SHAPE_H
//
//#include "BulletCollision/CollisionShapes/btBvhTriangleMeshShape.h"
//
//// The btScaledBvhTriangleMeshShape allows to instance a scaled version of an existing btBvhTriangleMeshShape.
//// Note that each btBvhTriangleMeshShape still can have its own local scaling, 
////  independent from this btScaledBvhTriangleMeshShape 'localScaling'
//
//ATTRIBUTE_ALIGNED16(class) btScaledBvhTriangleMeshShape : public btConcaveShape
//{
//      btVector3   m_localScaling;
//	  btBvhTriangleMeshShape* m_bvhTriMeshShape;
//
//public:
//
//      btScaledBvhTriangleMeshShape(btBvhTriangleMeshShape* childShape,const btVector3& localScaling);
//
//      virtual ~btScaledBvhTriangleMeshShape();
//
//      virtual void getAabb(const btTransform& t,btVector3& aabbMin,btVector3& aabbMax) const;
//
//      virtual void      setLocalScaling(const btVector3& scaling);
//
//      virtual const btVector3& getLocalScaling() const;
//
//      virtual void      calculateLocalInertia(btScalar mass,btVector3& inertia) const;
//
//      virtual void      processAllTriangles(btTriangleCallback* callback,const btVector3& aabbMin,const btVector3& aabbMax) const;
//
//      btBvhTriangleMeshShape* getChildShape()
//      {
//            return m_bvhTriMeshShape;
//      }
//
//      const btBvhTriangleMeshShape* getChildShape() const
//      {
//            return m_bvhTriMeshShape;
//      }
//
//      //debugging
//      virtual const char*     getName()const {return "SCALEDBVHTRIANGLEMESH";}
//};
//
//#endif //BVH_TRIANGLE_MESH_SHAPE_H
//
// 



 

